/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.commons.listing;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.jcr.PropertyType;

import org.apache.commons.lang3.StringUtils;
import org.apache.sling.api.SlingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.cq.social.commons.Comment;
import com.adobe.cq.social.commons.CommentSystem;
import com.adobe.cq.social.ugc.api.ComparisonType;
import com.adobe.cq.social.ugc.api.Constraint;
import com.adobe.cq.social.ugc.api.ConstraintGroup;
import com.adobe.cq.social.ugc.api.FullTextConstraint;
import com.adobe.cq.social.ugc.api.Operator;
import com.adobe.cq.social.ugc.api.UgcFilter;
import com.adobe.cq.social.ugc.api.UgcIndexerUtil;
import com.adobe.cq.social.ugc.api.ValueConstraint;
import com.day.cq.commons.jcr.JcrConstants;
import com.day.cq.tagging.TagConstants;

/**
 * Query Filter Utility class uses to parse filter parameters. Filter query restricts the data returned in a query
 * request, which should take a form of
 * http://<host>:<port>//<path>.social.query.<startIndex>.<length>?<filter=<filterExpressoin>
 * <p>
 * A single filter uses the form: <code>property operator expression</code> where,
 * <ul>
 * <li><b>property</b> - the property name. @see MAP_INDEX_TYPE for the list of valid name.</li>
 * <li><b>operator</b> - defines the type of filter match to use.</li>
 * <li><b>expression</b> - states the values included in the results. There are couple of important rules for filter
 * expression.
 * <ul>
 * <li><b>URL-reversed characters</b> - Characters such as '&' must be url-encoded.</li>
 * <li><b>Reversed characters</b> - The comma and backslash must be backslash escaped when they appear in an
 * expression.</li>
 * </ul>
 * </li>
 * </ul>
 * </p>
 */
public class QueryFilterUtil {
    /**
     * General Query Filter Exception, thrown when there is an error while parsing a filter.
     */
    public static class QueryFilterException extends Exception {
        /**
         *
         */
        private static final long serialVersionUID = 1L;

        public QueryFilterException(final String msg) {
            super(msg);
        }

        public QueryFilterException(final String msg, final Exception cause) {
            super(msg, cause);
        }

        public QueryFilterException(final Exception cause) {
            super(cause);
        }
    }

    private static final String PROP_TAG = "tag";

    private static final String PROP_USER_DISPLAY_NAME = "author_display_name";

    // Definition of the indexed properties.
    private static Map<String, Integer> MAP_INDEXED_TYPE;
    static {
        // Define by AbstractUgcNodeIndexerExtension
        MAP_INDEXED_TYPE = new HashMap<String, Integer>();
        MAP_INDEXED_TYPE.put("jcr:primaryType", PropertyType.STRING);
        MAP_INDEXED_TYPE.put(":uuid", PropertyType.STRING);
        MAP_INDEXED_TYPE.put(":path", PropertyType.STRING);
        MAP_INDEXED_TYPE.put(":parent", PropertyType.STRING);
        MAP_INDEXED_TYPE.put(":name", PropertyType.STRING);
        // Define by ModerationUgcNodeIndexerExtension:
        MAP_INDEXED_TYPE.put(Comment.PROP_FLAGGED, PropertyType.BOOLEAN);
        MAP_INDEXED_TYPE.put(Comment.PROP_SPAM, PropertyType.BOOLEAN);
        MAP_INDEXED_TYPE.put("read", PropertyType.BOOLEAN);
        MAP_INDEXED_TYPE.put("influence", PropertyType.BOOLEAN);
        MAP_INDEXED_TYPE.put("attachments", PropertyType.BOOLEAN);
        MAP_INDEXED_TYPE.put("sentiment", PropertyType.LONG);
        MAP_INDEXED_TYPE.put("flagged", PropertyType.BOOLEAN);
        MAP_INDEXED_TYPE.put("added", PropertyType.DATE);
        MAP_INDEXED_TYPE.put("modifiedDate", PropertyType.DATE);
        MAP_INDEXED_TYPE.put("state", PropertyType.STRING);
        MAP_INDEXED_TYPE.put("userIdentifier", PropertyType.STRING);
        MAP_INDEXED_TYPE.put("parentPath", PropertyType.STRING);
        MAP_INDEXED_TYPE.put("parentTitle", PropertyType.STRING);
        MAP_INDEXED_TYPE.put("replies", PropertyType.LONG);
        MAP_INDEXED_TYPE.put(JcrConstants.JCR_TITLE, PropertyType.STRING);
        MAP_INDEXED_TYPE.put(JcrConstants.JCR_DESCRIPTION, PropertyType.STRING);
        MAP_INDEXED_TYPE.put(CommentSystem.NN_COMMENT_ATTACHMENTS, PropertyType.BOOLEAN);
        // Define by JournalUgcNodeIndexerExtension
        MAP_INDEXED_TYPE.put(SlingConstants.NAMESPACE_PREFIX + ":" + SlingConstants.PROPERTY_RESOURCE_TYPE,
            PropertyType.STRING);
        // Define by ForumSCIndexExtension
        // MAP_INDEXED_TYPE.put("replies", PropertyType.LONG);
        MAP_INDEXED_TYPE.put("parentPath", PropertyType.STRING);
        MAP_INDEXED_TYPE.put("parentTitle", PropertyType.STRING);
        MAP_INDEXED_TYPE.put("allowThreadedReply", PropertyType.BOOLEAN);
        // Define by QnaSCIndexExtension
        MAP_INDEXED_TYPE.put("answered", PropertyType.BOOLEAN);
        MAP_INDEXED_TYPE.put("chosenanswered", PropertyType.BOOLEAN);
        // Define by HBSIndexExtension
        MAP_INDEXED_TYPE.put(PROP_TAG, PropertyType.STRING);
        MAP_INDEXED_TYPE.put(TagConstants.PN_TAGS, PropertyType.STRING);
        MAP_INDEXED_TYPE.put(PROP_USER_DISPLAY_NAME, PropertyType.STRING);
    }

    /** default log. */
    private final static Logger LOG = LoggerFactory.getLogger(QueryFilterUtil.class);

    private static String TEXT_SEARCH_PROPERTY[] = {"jcr:title", "jcr:description"};

    // Filter index
    private static int NUMBER_FILTER_TOKENS = 3;
    private static int FILTER_PROP_INDEX = 0;
    private static int FILTER_OPERATOR_INDEX = 1;
    private static int FILTER_EXPRESSION_INDEX = 2;

    /**
     * Query Comparator types - map our URL comparator to UGC Comparator type.
     */
    public enum Comparator {
        EQ(ComparisonType.Equals), NE(ComparisonType.NotEquals), LT(ComparisonType.LessThan), LTE(
            ComparisonType.LessThanOrEqualTo), GT(ComparisonType.GreaterThan), GTE(
            ComparisonType.GreaterThanOrEqualTo), LIKE(null); // there is no comparison type for LIKE since it is a
        // FullTextConstraint

        ComparisonType comparator;

        Comparator(final ComparisonType comparator) {
            this.comparator = comparator;
        }

        ComparisonType getComparisonType() {
            return comparator;
        }

        public static Comparator fromString(final String text) throws QueryFilterException {
            if (text == null || text.length() == 0) {
                return null;
            }
            return Comparator.valueOf(text.toUpperCase());

        }
    }

    /**
     * Abstract of a filter clause, which consists of a property name, a comparator, and a value. Example filter=name
     * eq admin
     */
    public static class QueryFilter {
        private final Constraint constraint;
        private final String name;  // the name of the constraint property.

        @SuppressWarnings("unchecked")
        private QueryFilter(final String name, final Comparator comp, final String value, final Operator operator)
            throws QueryFilterException {
            if (StringUtils.isEmpty(name)) {
                throw new QueryFilterException("Name parameter can not be null.");
            }
            if (comp == null) {
                throw new QueryFilterException("Invalid comparator value");
            }
            if (StringUtils.isEmpty(value)) {
                throw new QueryFilterException("Value parameter can not be null.");
            }
            if (operator == null) {
                throw new QueryFilterException("Operator parameter can not be null.");
            }

            this.name = name;
            final String sValue = UgcFilter.escape(getPropertyValue(name, value));
            this.constraint =
                isTextSearchProperty(name, comp) ? new FullTextConstraint(sValue, name) : new ValueConstraint(name,
                    sValue, comp.getComparisonType(), operator);
        }

        public Constraint getConstraint() {
			return null;
		}

        public String getName() {
			return null;
		}

        public static QueryFilter parse(final String filter, final Operator operator) throws QueryFilterException {
			return null;
		}

        private boolean isTextSearchProperty(final String name, final Comparator comp) {
			return false;
		}

        private static String escapeQueryValue(final String value) {
			return null;
		}
    }

    /**
     * OR Logic Example: filter=name eq 'admin', name eq 'peter'
     */
    public static void parseOrFilters(final String expressions, final Map<String, ConstraintGroup> orFilters)
        throws QueryFilterException {
		}

    private static String getPropertyValue(final String name, final String input) throws QueryFilterException {
		return null;
	}

    /**
     * AND logic - AND logic is achieved using multiple filters. Example: filter=name eq 'admin'&filter=message eq
     * 'testing' Combining AND and OR filter=name eq 'admin',name eq 'peter'&filter=message eq testing
     */
    public static List<ConstraintGroup> parseFilter(final String[] filters) throws QueryFilterException {
		return null;
	}
}
