/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.dam.commons.handler;

import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.jcr.RepositoryException;

import org.apache.commons.imaging.ImageFormat;
import org.apache.commons.imaging.ImageInfo;
import org.apache.commons.imaging.ImageParser;
import org.apache.commons.imaging.ImageReadException;
import org.apache.commons.imaging.ImageWriteException;
import org.apache.commons.imaging.Imaging;
import org.apache.commons.imaging.common.IImageMetadata;
import org.apache.commons.imaging.common.ImageMetadata;
import org.apache.commons.imaging.common.bytesource.ByteSource;
import org.apache.commons.imaging.common.bytesource.ByteSourceInputStream;
import org.apache.commons.imaging.formats.jpeg.JpegImageMetadata;
import org.apache.commons.imaging.formats.jpeg.JpegPhotoshopMetadata;
import org.apache.commons.imaging.formats.tiff.TiffField;
import org.apache.commons.imaging.formats.tiff.TiffImageMetadata;
import org.apache.commons.imaging.formats.tiff.constants.TiffTagConstants;
import org.apache.commons.imaging.formats.tiff.fieldtypes.FieldType;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.felix.scr.annotations.Activate;
import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Property;
import org.apache.felix.scr.annotations.Service;
import org.apache.sling.commons.osgi.PropertiesUtil;
import org.osgi.service.component.ComponentContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.day.cq.dam.api.Asset;
import com.day.cq.dam.api.Rendition;
import com.day.cq.dam.api.metadata.ExtractedMetadata;
import com.day.image.Layer;

/**
 * The <code>StandardImageHandler</code> supports currently following image types: <ul> <li>gif <li>png <li>photoshop
 * <li>jpeg <li>tiff <li>bmp <ul>
 */
@Component(inherit = true, metatype = true)
@Service
public class StandardImageHandler extends AbstractAssetHandler {
    /**
     * the default logger
     */
    protected final Logger log = LoggerFactory.getLogger(StandardImageHandler.class);

    /**
     * Mime type
     */
    public static final String GIF_MIMETYPE = "image/gif";
    public static final String PNG1_MIMETYPE = "image/png";
    public static final String PNG2_MIMETYPE = "image/x-png";
    public static final String JPEG_MIMETYPE = "image/jpeg";
    public static final String PJPEG_MIMETYPE = "image/pjpeg";
    public static final String TIFF_MIMETYPE = "image/tiff";
    public static final String TIFF1_MIMETYPE = "image/x-tiff";
    public static final String BMP1_MIMETYPE = "image/x-ms-bmp";
    public static final String BMP2_MIMETYPE = "image/bmp";
    public static final String RAW1_MIMETYPE = "image/x-raw-adobe";
    public static final String RAW2_MIMETYPE = "image/x-raw-hasselblad";
    public static final String RAW3_MIMETYPE = "image/x-raw-fuji";
    public static final String RAW4_MIMETYPE = "image/x-raw-canon";
    public static final String RAW5_MIMETYPE = "image/x-raw-kodak";
    public static final String RAW6_MIMETYPE = "image/x-raw-minolta";
    public static final String RAW7_MIMETYPE = "image/x-raw-nikon";
    public static final String RAW8_MIMETYPE = "image/x-raw-olympus";
    public static final String RAW9_MIMETYPE = "image/x-raw-pentax";
    public static final String RAW10_MIMETYPE = "image/x-raw-sony";
    public static final String RAW11_MIMETYPE = "image/x-raw-sigma";
    public static final String RAW12_MIMETYPE = "image/x-raw-epson";
    public static final String RAW13_MIMETYPE = "image/x-raw-mamiya";
    public static final String RAW14_MIMETYPE = "image/x-raw-leaf";
    public static final String RAW15_MIMETYPE = "image/x-raw-panasonic";
    public static final String RAW16_MIMETYPE = "image/x-raw-phaseone";
    public static final String RAW17_MIMETYPE = "image/x-raw-red";
    public static final String RAW18_MIMETYPE = "image/x-raw-imacon";
    public static final String RAW19_MIMETYPE = "image/x-raw-logitech";
    public static final String RAW20_MIMETYPE = "image/x-raw-casio";
    public static final String RAW21_MIMETYPE = "image/x-raw-rawzor";
    public static final String DNG_MIMETYPE = "image/dng";

    protected static final String[] MIME_TYPES = {GIF_MIMETYPE,
            PNG1_MIMETYPE,
            PNG2_MIMETYPE,
            TIFF_MIMETYPE,
            TIFF1_MIMETYPE,
            BMP1_MIMETYPE,
            BMP2_MIMETYPE,
            RAW1_MIMETYPE,
            RAW2_MIMETYPE,
            RAW3_MIMETYPE,
            RAW4_MIMETYPE,
            RAW5_MIMETYPE,
            RAW6_MIMETYPE,
            RAW7_MIMETYPE,
            RAW8_MIMETYPE,
            RAW9_MIMETYPE,
            RAW10_MIMETYPE,
            RAW11_MIMETYPE,
            RAW12_MIMETYPE,
            RAW13_MIMETYPE,
            RAW14_MIMETYPE,
            RAW15_MIMETYPE,
            RAW16_MIMETYPE,
            RAW17_MIMETYPE,
            RAW18_MIMETYPE,
            RAW19_MIMETYPE,
            RAW20_MIMETYPE,
            RAW21_MIMETYPE,
            DNG_MIMETYPE};

    //Workaround for bug http://bugs.day.com/bugzilla/show_bug.cgi?id=33613
    //todo Must be corrected in Sanselan
    private static final Set<String> IGNORED_FROM_SANSELAN = new HashSet<String>();
    static {
        IGNORED_FROM_SANSELAN.add("Image Description");
        IGNORED_FROM_SANSELAN.add("Artist");
        IGNORED_FROM_SANSELAN.add("Copyright");
    }

    @Property(boolValue = false)
    protected static final String ENABLE_BINARY_META_EXTRACTION = "cq.dam.enable.ext.meta.extraction";

    protected boolean enableExtMetaExtraction = false;


    /**
     * @see com.day.cq.dam.api.handler.AssetHandler#getMimeTypes()
     */
    public String[] getMimeTypes() {
		return null;
	}

    public ExtractedMetadata extractMetadata(final Asset asset) {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    public BufferedImage getImage(final Rendition rendition) throws IOException {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    public BufferedImage getImage(final Rendition rendition, Dimension maxDimension) throws IOException {
		return null;
	}

    protected Map<String, Serializable> extractImageInfo(final Asset asset) {
		return null;
	}

    private Map<String, Serializable> extractImageInfo(final String path, ByteSource bsis, ImageParser ip) {
		return null;
	}

    protected void extractMetadata(Asset asset, ExtractedMetadata metadata) throws IOException, ImageReadException
    {
	}

    private void extractImageXMP(InputStream stream, ExtractedMetadata metadata) {
	}

    private void setPhotoshopItemValue(IImageMetadata.IImageMetadataItem item, ExtractedMetadata extractedMetadata) {
	}

    private ImageParser getImageParser(ByteSource byteSrc) throws ImageReadException, IOException {
		return null;
	}
    /**
     * Sanselan seems to add control chars at the end which needs to be removed
     * This method removes any ctrl chars if the TIFF field type is String
     * @param field, Tiff field which needs to be trimmed if String
     * @return Trimmed string or the original field value in case the field is not a String
     * @throws ImageReadException if unable to extract field value
     * */
    private Object trimValue(TiffField field) throws ImageReadException {
		return null;
	}

    private boolean isPng(final Rendition rendition) {
		return false;
	}

    private boolean isGif(final Rendition rendition) {
		return false;
	}

    private boolean isRaw(final Rendition rendition) {
		return false;
	}

    @Activate
    @SuppressWarnings("unused")
    protected void activate(final ComponentContext context) throws RepositoryException {
	}
}
