/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.dam.core.process;

import com.day.cq.dam.api.Asset;
import com.day.cq.dam.api.Rendition;
import com.day.cq.dam.api.handler.AssetHandler;
import com.day.cq.dam.api.thumbnail.ThumbnailConfig;
import com.day.cq.dam.commons.process.AbstractAssetWorkflowProcess;
import com.day.cq.dam.commons.thumbnail.ThumbnailGenerator;
import com.day.cq.workflow.WorkflowException;
import com.day.cq.workflow.WorkflowSession;
import com.day.cq.workflow.exec.WorkItem;
import com.day.cq.workflow.metadata.MetaDataMap;
import org.apache.commons.exec.CommandLine;
import org.apache.commons.exec.DefaultExecutor;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Property;
import org.apache.felix.scr.annotations.Service;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.jcr.RepositoryException;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Workflow process that calls a command-line program and uses each output file
 * that it produces as an additional rendition. Optionally creates thumbnails
 * based on those renditions.<br>
 * The dimensions given in the thumbnail specifications are the maximal size the
 * result must have. Aspect ratio is maintained for image resizing. As a
 * precondition, the payload of this Workflow process has to be an
 * {@link com.day.cq.dam.api.Asset Asset} or part of an Asset.
 * <p>
 * <p/>
 * Example with the following workflow step arguments:
 *
 * <pre>
 *    mime:application/postscript,
 *    tn:140:100,tn:48:48,
 *    cmd:/bin/convert ${directory}/${filename} ${directory}/${basename}.jpg
 * </pre>
 * <p/>
 * The process will call /bin/convert, pass it the full path of the asset being
 * processed (temporarily dumped to disk), and create thumbnails of size 140x100
 * and 48x48 based on the output created by /bin/convert. <br>
 * This will only happen for assets having the
 * <code>application/postscript</code> mime-type, others are ignored.
 * <p/>
 * <b>Arguments:</b>
 * <table>
 * <thead>
 * <tr>
 * <td>Name</td>
 * <td>Prefix</td>
 * <td>Description</td>
 * <td>Required</td>
 * <td>Multiple</td>
 * <td>Example</td>
 * </tr>
 * </thead>
 * <tr>
 * <td>Command</td>
 * <td>cmd:</td>
 * <td>Command as executed on the consle.<br>
 * The command can contain varibales which are replaced before execution. The
 * following variables are available:<br>
 * <i>filename</i>: name of the file as exported to disk.<br>
 * <i>file</i>: absolute path of the file exported.<br>
 * <i>directory</i>: absolute path of the directory the command is run and the
 * asset is exported to.<br>
 * <i>basename</i>: the assets name on the disk without possible file-extension<br>
 * <i>extension</i>: the assets file extension.<br>
 * </td>
 * <td>required</td>
 * <td>multiple</td>
 * <td>cmd:/bin/convert ${directory}/${filename} ${directory}/${basename}.jpg</td>
 * </tr>
 * <tr>
 * <td>Mimetype Filter</td>
 * <td>mime:</td>
 * <td>Mimetype the Asset must have. If the Asset is of a diffrent type, this
 * process will not be executed.</td>
 * <td>required</td>
 * <td>multiple</td>
 * <td>mime:application/postscript</td>
 * </tr>
 * <tr>
 * <td>Thumbnail Specification</td>
 * <td>tn:</td>
 * <td>Dimensions of the Thumbnails to be generated from the Asset.<br>
 * <i>width</i>:<i>height</i>[:<i>false</i>]<br>
 * <i>width</i>:Number the maximal width in Pixel the Thumbnail must not exceed
 * <i>height</i>:Number the maximal height in Pixel the Thumbnail must not
 * exceed. <i>center</i>:Boolean, optional flag to indicate that the resulting
 * Thumbnail must not be centered, true is default.</td>
 * <td></td>
 * <td>multiple</td>
 * <td>tn:140:100</td>
 * </tr>
 * </table>
 *
 * @see AbstractAssetWorkflowProcess
 */
@Component(metatype = false)
@Service
@Property(name = "process.label", value = "Command Line")
public class CommandLineProcess extends AbstractAssetWorkflowProcess {

    private final Logger log = LoggerFactory.getLogger(getClass());

    /**
     * The available arguments to this process implementation.
     */
    public enum Arguments {
        PROCESS_ARGS("PROCESS_ARGS"),

        /** Argument for the mime types to use */
        MIME_TYPES("mime"),

        /** Argument for thumb nail sizes */
        THUMBNAILS("tn"),

        /** Argument for command-line to execute */
        COMMANDS("cmd");

        private String argumentName;

        Arguments(String argumentName) {
            this.argumentName = argumentName;
        }

        public String getArgumentName() {
            return this.argumentName;
        }

        public String getArgumentPrefix() {
            return this.argumentName + ":";
        }

    }

    public void execute(WorkItem workItem, WorkflowSession wfsession, MetaDataMap args) throws WorkflowException {
	}

    protected void createThumbnails(final Asset asset, final Rendition rendition,
            final Collection<ThumbnailConfig> configs) throws Exception {
			}

    protected String recheck(String fileName) throws RepositoryException {
		return null;
	}

    public String[] buildArguments(MetaDataMap metaData) {
		return null;
	}

}