/*
 * Copyright 1997-2009 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.search;

import com.day.cq.search.eval.FulltextPredicateEvaluator;
import com.day.cq.search.eval.JcrPropertyPredicateEvaluator;
import com.day.cq.search.eval.PathPredicateEvaluator;
import com.day.cq.search.eval.RangePropertyPredicateEvaluator;
import com.day.cq.search.eval.TypePredicateEvaluator;
import com.day.cq.search.impl.builder.PredicateWalker;
import org.apache.jackrabbit.commons.query.GQL;
import org.apache.jackrabbit.util.Text;

import javax.jcr.RepositoryException;
import java.util.Comparator;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * {@linkplain PredicateConverter} provides mappings from the
 * predicate/predicate group data structure to others, such as a simple
 * key/value string map (eg. a request parameter map).
 * 
 * @since 5.2
 */
public abstract class PredicateConverter {

    /**
     * Prefix to separate predicate group parameters from child predicates (to
     * be exact: from parameters of child predicates). This also means that
     * there cannot be a predicate type named like this (ie.
     * <code>{@value}</code>). Examples to show the difference:
     * <ul>
     * <li><code>p.limit = true</code> (group parameter)</li>
     * <li><code>type = nt:file</code> (child predicate)</li>
     * <li><code>path.exact = true</code> (child predicate parameter)</li>
     * <li><code>group.type = nt:file</code> (child predicate group w/
     * predicate)</li>
     * <li><code>group.p.or = true</code> (group parameter of child group)</li>
     * </ul>
     */
    public static final String GROUP_PARAMETER_PREFIX = "p";
    
    /**
     * All parameters starting with "_" will be ignored. Typical examples
     * are "_charset_" or "_dc".
     */
    public static final String IGNORE_PARAMETER_PREFIX = "_";
    private static final String RANGE_DELIMITER = "..";

    private static final String ORDER = "order";

    private static final String DATERANGE_PRED = "daterange";
    private static final String RANGEPROP_PRED = "rangeproperty";

    private static final String DATE_REGEX = ".*[0-9]+-.*";
    private static final String RANGE_REGEX = "(.*)\\.{2,}(.*)";


    /**
     * Converts a map with predicates and their parameters into a predicate
     * tree. Accepts a map with strings as keys and either simple strings as
     * values or string arrays as values. In the array case, the first value
     * will be chosen.
     * 
     * <p>
     * Note that all parameters starting with a "_" (see
     * {@link #IGNORE_PARAMETER_PREFIX}) will be ignored. Typical examples are
     * "_charset_" or "_dc".
     */
    public static PredicateGroup createPredicates(Map predicateParameterMap) {
		return null;
	}
    
    /**
     * Converts a predicate tree into a parameter map, the inverse
     * transformation of {@link #createPredicates(Map)}.
     */
    public static Map<String, String> createMap(PredicateGroup root) {
		return null;
	}
    
    /**
     * Returns an URL query part containing the given group. This is the same
     * mapping as used in {@link #createMap(PredicateGroup)} and
     * {@link #createPredicates(Map)}. For example, the returned value could be:
     * <code>type=cq:Page&path=/content</code>. Note that this won't be a
     * complete URL, just a list of parameters for an URL query part. The keys
     * and values will be properly escaped for use in an URL.
     */
    public static String toURL(PredicateGroup group) {
		return null;
	}
    
    // -----------------------------------------------------< private >
    
    private static class CollectParameters extends PredicateWalker {
        
        private Map<String, String> result = new TreeMap<String, String>();
        
        @Override
        protected void visitInternal(Predicate predicate) {
		}
        
        public Map<String, String> getParameters() {
			return null;
		}
        
    }
    
    private static boolean isGroupParameter(String subParameter) {
		return false;
	}
    
    private static String getGroupParameterName(String subParameter) {
		return null;
	}

    /**
     * <code>PredicateTreeNode</code> is a helper data structure for parsing
     * key-value based predicateEvaluator parameter maps and turning them into a
     * "tree" of {@link Predicate Predicates} ({@link PredicateGroup} acts as
     * the folder "node"). It ensures the order given by the naming schema.
     * Parameters and their predicates can be inserted in any order (via
     * {@link #insert(String, String)}) and the tree-structured predicates are
     * generated at the end (via {@link #generatePredicateTree()}).
     * 
     * Schema Examples:
     *    predicateType = foobar
     *    2.predicateType = foobar
     *    2.predicateType.param = something
     *    group.1.group.2.predicateType = bla
     */
    private static class PredicateTreeNode {
        public Predicate predicate;
        
        /** Sorted map required to ensure order of keys */
        private SortedMap<Object, PredicateTreeNode> kids =
            new TreeMap<Object, PredicateTreeNode>(LongOrStringComparator.INSTANCE);
        
        public PredicateTreeNode(Predicate p) {
            this.predicate = p;
        }
        
        public void insert(final String name, final String value) {
		}
        
        public void generatePredicateTree() {
		}
    }

    /**
     * Custom comparator for the sorted map used in {@link PredicateTreeNode},
     * which contains either Longs or Strings.
     */
    private static class LongOrStringComparator implements Comparator<Object> {
        
        public static final LongOrStringComparator INSTANCE = new LongOrStringComparator();

        public int compare(Object o1, Object o2) {
			return 0;
		}
        
    }

    /**
     * Parse and converts GQL statement to QueryBuilder PredicateGroup.
     *
     * @param statement the statement to be processed for extracting conditions.
     * @return PredicateGroup containing all conditions formed from statement
     * @throws RepositoryException
     */
    public static PredicateGroup createPredicatesFromGQL(String statement) throws RepositoryException {
		return null;
	}

    /**
     * @param property      property name of the currently parsed expression.
     * @param value         value of the currently parsed expression.
     * @param optional      whether the previous token was the OR operator.
     * @param parentGroup   root group for GQL predicates.
     * @param conjointGroup group containing general predicates like path, order, type, limit etc which can't be ORed
     */
    private static void pushExpression(String property, String value, boolean optional, PredicateGroup parentGroup,
                                       PredicateGroup conjointGroup) {
									}
}
