/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.wcm.foundation;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.Value;
import javax.jcr.query.Query;
import javax.jcr.query.RowIterator;

import org.apache.jackrabbit.commons.query.GQL;
import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.resource.Resource;

import com.day.cq.commons.jcr.JcrConstants;
import com.day.cq.dam.api.DamConstants;
import com.day.cq.search.Predicate;
import com.day.cq.search.SimpleSearch;
import com.day.cq.search.Trends;
import com.day.cq.search.QueryBuilder;
import com.day.cq.search.facets.Facet;
import com.day.cq.search.result.ResultPage;
import com.day.cq.search.result.SearchResult;
import com.day.cq.wcm.api.NameConstants;
import com.day.cq.wcm.api.components.Component;
import com.day.cq.wcm.commons.WCMUtils;

/**
 * The <code>Search</code> class implements the search logic used in the
 * foundation search component and exposes the query result in a scripting
 * friendly object structure.
 * <p/>
 * This class does a fulltext query, which means wildcards like '*' and '?' will
 * be filtered out. Please use {@link QueryBuilder} or execute a query directly
 * on the JCR Session if wildcard support is needed.
 */
public final class Search {

    /**
     * The name for the query parameter.
     */
    private static final String QUERY_PARAM_NAME = "q";

    /**
     * The name for the start parameter.
     */
    private static final String START_PARAM_NAME = "start";

    /**
     * The name for the language facet parameter.
     */
    private static final String LANGUAGE_FACET_PARAM_NAME = "language";

 	  /**
     * The name for the charset parameter (usually set when posting).
     */
    private static final String CHARSET_PARAM_NAME = "_charset_";

    /**
     * The name for the tag facet parameter.
     */
    private static final String TAG_FACET_PARAM_NAME = "tag";

    /**
     * The name for the mime type facet parameter.
     */
    private static final String MIME_TYPE_FACET_PARAM_NAME = "mimeType";

    /**
     * The name for the last modified facet parameter, which specifies the
     * lower bound of the date range.
     */
    private static final String FROM_FACET_PARAM_NAME = "from";

    /**
     * The name for the last modified facet parameter, which specifies the
     * upper bound of the date range.
     */
    private static final String TO_FACET_PARAM_NAME = "to";

    /**
     * HTML extension.
     */
    private static final String HTML_EXT = "html";

    /**
     * The query template to check the spelling of a fulltext query statement.
     */
    private static final String SPELLCHECK_QUERY = "/jcr:root[rep:spellcheck('${query}')]/(rep:spellcheck())";

    /**
     * The current request.
     */
    private final SlingHttpServletRequest request;

    /**
     * The underlying search instance.
     */
    private final SimpleSearch search;

    /**
     * The current resource.
     */
    private final Resource resource;

    /**
     * The query result. Initially <code>null</code>.
     */
    private Result result;

    /**
     * The result pages.
     */
    private List<Page> resultPages;

    /**
     * The raw query as read from the request or manually set.
     */
    private String query;

    /**
     * Set to <code>true</code> if there was a tag predicate set in the request.
     */
    private boolean tagPredicateSet;

    /**
     * Creates a new search based on the given <code>request</code>.
     *
     * @param request the current request.
     */
    public Search(SlingHttpServletRequest request) {
        this.request = request;
        this.resource = request.getResource();
        this.search = resource.adaptTo(SimpleSearch.class);

        String charset = "ISO-8859-1"; //would be used by the GET requests

        if (request.getParameter(CHARSET_PARAM_NAME) != null) {
            charset = request.getParameter(CHARSET_PARAM_NAME);
        }

        if (request.getParameter(QUERY_PARAM_NAME) != null) {
            try {
                setQuery(new String(request.getParameter(
                        QUERY_PARAM_NAME).getBytes(charset), "UTF-8"));
            } catch (UnsupportedEncodingException e) {
                // ignore
            }
        }
        if (request.getParameter(START_PARAM_NAME) != null) {
            try {
                this.search.setStart(Long.parseLong(request.getParameter(START_PARAM_NAME)));
            } catch (NumberFormatException e) {
                // ignore
            }
        }

        // Note: the predicate names (first param to Predicate constructor) are the same
        // as in the facets returned and used in search.jsp
        Predicate languagePredicate = new Predicate("languages", "language");
        languagePredicate.set("language", request.getParameter(LANGUAGE_FACET_PARAM_NAME));
        this.search.addPredicate(languagePredicate);

        Predicate tagPredicate = new Predicate("tags", "tagid");
        tagPredicate.set("property", JcrConstants.JCR_CONTENT + "/" + NameConstants.PN_TAGS);
        tagPredicate.set("tagid", request.getParameter(TAG_FACET_PARAM_NAME));
        this.search.addPredicate(tagPredicate);
        this.tagPredicateSet = tagPredicate.get("tagid") != null;

        Predicate mimeTypePredicate = new Predicate("mimeTypes", "property");
        mimeTypePredicate.set("property", JcrConstants.JCR_CONTENT + "/" + JcrConstants.JCR_MIMETYPE);
        mimeTypePredicate.set("value", request.getParameter(MIME_TYPE_FACET_PARAM_NAME));
        this.search.addPredicate(mimeTypePredicate);

        Predicate lastModPredicate = new Predicate("lastModified", "daterange");
        lastModPredicate.set("property", JcrConstants.JCR_CONTENT + "/" + NameConstants.PN_PAGE_LAST_MOD);
        // TODO: consider lowerOperation/upperOperation (has to be additional request params)
        lastModPredicate.set("lowerBound", request.getParameter(FROM_FACET_PARAM_NAME));
        lastModPredicate.set("upperBound", request.getParameter(TO_FACET_PARAM_NAME));
        this.search.addPredicate(lastModPredicate);

        Predicate orderByScore = new Predicate("orderByScore", Predicate.ORDER_BY);
        orderByScore.set(Predicate.ORDER_BY, "@jcr:score");
        orderByScore.set(Predicate.PARAM_SORT, Predicate.SORT_DESCENDING);
        this.search.addPredicate(orderByScore);
    }

    /**
     * @return query trends (popular queries).
     */
    public Trends getTrends() {
		return null;
	}

    /**
     * @return the query result or <code>null</code> if there is neither a query
     *         parameter set nor a tag predicate.
     * @throws RepositoryException if an exception occurs while executing the
     *                             query.
     */
    public Result getResult() throws RepositoryException {
		return null;
	}

    /**
     * @return queries that are related to the current one.
     * @throws RepositoryException if an error occurs while reading from
     *                             the repository.
     */
    public List<String> getRelatedQueries() throws RepositoryException {
		return null;
	}

    /**
     * @return the query supplied by the user or an empty String if none is
     *         provided.
     */
    public String getQuery() {
		return null;
	}

    /**
     * Sets a new fulltext query that will be executed.
     *
     * @param query the fulltext query.
     */
    public void setQuery(String query) {
	}

    /**
     * @return the names of the properties that will be used in an excerpt.
     *
     * @deprecated since 5.2. Excerpt properties can now only be specified in
     *             the <a href="http://localhost:4502/system/console/configMgr/com.day.cq.search.impl.builder.QueryBuilderImpl"
     *             >configuration of the QueryBuilder interface</a>. For 5.3,
     *             when Jackrabbit 1.5 is used, the excerpt properties can be
     *             configured in the repository.
     *
     */
    @Deprecated
    public String getExcerptPropertyNames() {
		return null;
	}

    /**
     * @param properties
     *            comma separated names of the properties that will be used in
     *            an excerpt.
     *
     * @deprecated since 5.2. Excerpt properties can now only be specified in
     *             the <a href="http://localhost:4502/system/console/configMgr/com.day.cq.search.impl.builder.QueryBuilderImpl"
     *             >configuration of the QueryBuilder interface</a>. For 5.3,
     *             when Jackrabbit 1.5 is used, the excerpt properties can be
     *             configured in the repository.
     *
     */
    @Deprecated
    public void setExcerptPropertyNames(String properties) {
	}

    /**
     * @return the number of hits to display per page.
     */
    public long getHitsPerPage() {
		return 0L;
	}

    /**
     * @param num the number of hits to display on a page.
     */
    public void setHitsPerPage(long num) {
	}

    /**
     * @return the location where to search in.
     */
    public String getSearchIn() {
		return null;
	}

    /**
     * @param searchIn the location where to search in.
     */
    public void setSearchIn(String searchIn) {
	}

    /**
     * @return the names of the properties that will be searched.
     */
    public String getSearchProperties() {
		return null;
	}

    /**
     * @param properties comma separated names of the properties that will be searched.
     */
    public void setSearchProperties(String properties) {
	}

    /**
     * A search result.
     */
    public final class Result {

        /**
         * The underyling CQ search result instance.
         */
        private final SearchResult result;

        /**
         * The hits on the current result page.
         */
        private final List<Hit> hits;

        /**
         * The spell suggestion. Set only when requested. An empty string
         * indicates that a suggestion was returned by the spellcheck, but
         * does not return results for the current query.
         * See {@link #getSpellcheck()}.
         */
        private String spellSuggestion;

        /**
         * Creates a new result based on the given CQ search result.
         *
         * @param result the CQ search result.
         */
        private Result(SearchResult result) {
            this.result = result;
            this.hits = new ArrayList<Hit>();
            for (com.day.cq.search.result.Hit h : result.getHits()) {
                this.hits.add(new Hit(h));
            }
        }

        /**
         * @return a List of {@link Page}es to display the navigation through the
         *         result pages.
         * @throws RepositoryException if an error occurs while reading from
         *              the query result.
         */
        public List<Page> getResultPages() throws RepositoryException {
			return null;
		}

        /**
         * @return the page, which contains the information about the previous page.
         *         Returns <code>null</code> if there is no previous page (i.e. the
         *         current page is the first page).
         * @throws RepositoryException if an error occurs while reading from
         *              the query result.
         */
        public Page getPreviousPage() throws RepositoryException {
			return null;
		}

        /**
         * @return the page, which contains the information about the next page.
         *         Returns <code>null</code> if there is no next page (i.e. the
         *         current page is the last page).
         * @throws RepositoryException if an error occurs while reading from
         *              the query result.
         */
        public Page getNextPage() throws RepositoryException {
			return null;
		}

        /**
         * @return the script for query and result tracking.
         */
        public String getTrackerScript() {
			return null;
		}

        /**
         * @return the result of a spell check or <code>null</code> if spell
         *         checking is not supported or the repository thinks the spelling
         *         is correct.
         */
        public String getSpellcheck() {
			return null;
		}

        /**
         * @return the start index. i.e. from where to start to display the hits.
         */
        public long getStartIndex() {
			return 0L;
		}

        /**
         * @return the total number of matches.
         */
        public long getTotalMatches() {
			return 0L;
		}

        /**
         * Returns the execution time in fractions of a second.
         * <br/>
         * Example: 0.08 (means, the query took 80 milliseconds to execute).
         * @return the execution time of the query.
         */
        public String getExecutionTime() {
			return null;
		}

        /**
         * Returns the execution time in milliseconds.
         *
         * @return the execution time of the query.
         */
        public long getExecutionTimeMillis() {
			return 0L;
		}

        /**
         * Returns the facets for this search result.
         *
         * @return the facets for this search result.
         * @throws RepositoryException if an error occurs while executing the query
         *          or calculating the facets.
         */
        public Map<String, Facet> getFacets() throws RepositoryException {
			return null;
		}

        /**
         * @return a List of {@link Hit}s to display on the result page.
         */
        public List<Hit> getHits() {
			return null;
		}
    }

    /**
     * A hit within the search result.
     */
    public final class Hit {

        /**
         * The underlying CQ hit.
         */
        private final com.day.cq.search.result.Hit hit;

        /**
         * Creates a new hit based on the given CQ <code>hit</code>.
         *
         * @param hit the CQ hit to wrap.
         */
        private Hit(com.day.cq.search.result.Hit hit) {
            this.hit = hit;
        }

        /**
         * Returns the title for this hit. The returned string may contain
         * HTML tags, which means it must not be escaped when written to the
         * response.
         *
         * @return the title for this hit.
         * @throws RepositoryException if an error occurs while reading form the
         * repository.
          */
        public String getTitle() throws RepositoryException {
			return null;
		}

        /**
         * @return the default excerpt for this hit.
         * @throws RepositoryException if an error occurs while building the
         * excerpt.
         */
        public String getExcerpt() throws RepositoryException {
			return null;
		}

        /**
         * @return the url for this hit.
         * @throws RepositoryException if an error occurs while reading from the
         *              query result.
         */
        public String getURL() throws RepositoryException {
			return null;
		}

        /**
         * @return the url that displays similar pages for this hit.
         * @throws RepositoryException if an error occurs while reading from
         * the query result.
         */
        public String getSimilarURL() throws RepositoryException {
			return null;
		}

        /**
         * Returns an icon image for this hit.
         *
         * @return an icon image for this hit.
         * @throws RepositoryException if an error occurs while reading from
         * the repository.
         */
        public String getIcon() throws RepositoryException {
			return null;
		}

        /**
         * Returns the extension of the url of this hit.
         * @return the extension
         * @throws RepositoryException if an error occurs
         */
        public String getExtension() throws RepositoryException {
			return null;
		}

        /**
         * @return the content properties on this hit.
         * @throws RepositoryException if an error occurs while reading from the
         * repository.
         */
        public Map getProperties() throws RepositoryException {
			return null;
		}

        /**
         * Returns <code>true</code> if the given node is either of type
         * <code>cq:Page</code>, <code>cq:PseudoPage</code>or
         * <code>dam:Asset</code>.
         *
         * @param n the node to check.
         * @return <code>true</code> if the node represents either a page or an
         *         asset; <code>false</code> otherwise.
         * @throws RepositoryException if an error occurs during the check.
         */
        private boolean isPageOrAsset(Node n) throws RepositoryException {
			return false;
		}

        /**
         * Returns <code>true</code> if the given node is either of type
         * <code>cq:Page</code> or <code>cq:PseudoPage</code>.
         *
         * @param n the node to check.
         * @return <code>true</code> if the node represents a page;
         *         <code>false</code> otherwise.
         * @throws RepositoryException if an error occurs during the check.
         */
        private boolean isPage(Node n) throws RepositoryException {
			return false;
		}

        /**
         * Returns the page or asset node that contains the current hit node.
         *
         * @return the page or asset node that contains the current hit node.
         * @throws RepositoryException if an error occurs while reading from the
         *                             repository.
         */
        private Node getPageOrAsset() throws RepositoryException {
			return null;
		}
    }

    /**
     * A result page.
     */
    public final class Page {

        /**
         * The underlying CQ result page.
         */
        private final ResultPage rp;

        /**
         * Creates a new page with the given CQ result page.
         *
         * @param rp the CQ result page.
         */
        private Page(ResultPage rp) {
            this.rp = rp;
        }

        /**
         * @return whether this page is currently displayed.
         */
        public boolean isCurrentPage() {
			return false;
		}

        /**
         * @return zero based index of this result page.
         */
        public long getIndex() {
			return 0L;
		}

        /**
         * @return the URL to this search result page.
         */
        public String getURL() {
			return null;
		}
    }

    /**
     * Encodes the given <code>url</code> using {@link URLEncoder#encode(String, String)}
     * with a <code>UTF-8</code> encoding.
     *
     * @param url the url to encode.
     * @return the encoded url.
     */
    private static String encodeURL(String url) {
		return null;
	}

    /**
     * Returns the icon path for the given extension or <code>null</code> if
     * none is found for the given extension.
     *
     * @param extension the extension.
     * @return the icon path for the given extension or <code>null</code>.
     */
    private String getIconPath(String extension) {
		return null;
	}
}
