/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xerces" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

// Sep 14, 2000:
//  Fixed problem with namespace handling. Contributed by
//  David Blondeau <blondeau@intalio.com>
// Sep 14, 2000:
//  Fixed serializer to report IO exception directly, instead at
//  the end of document processing.
//  Reported by Patrick Higgins <phiggins@transzap.com>
// Aug 21, 2000:
//  Fixed bug in startDocument not calling prepare.
//  Reported by Mikael Staldal <d96-mst-ingen-reklam@d.kth.se>
// Aug 21, 2000:
//  Added ability to omit DOCTYPE declaration.


package com.day.jcr.vault.util.xml.serialize;


import java.io.IOException;
import java.io.OutputStream;
import java.io.Writer;

import org.w3c.dom.Attr;
import org.w3c.dom.DOMError;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.xml.sax.AttributeList;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

import com.day.jcr.vault.util.xml.xerces.dom.DOMMessageFormatter;
import com.day.jcr.vault.util.xml.xerces.util.NamespaceSupport;
import com.day.jcr.vault.util.xml.xerces.util.SymbolTable;
import com.day.jcr.vault.util.xml.xerces.util.XMLChar;
import com.day.jcr.vault.util.xml.xerces.util.XMLSymbols;
import com.day.jcr.vault.util.xml.xerces.xni.NamespaceContext;

/**
 * <b>Day Note<b>: This class and dependencies were copied from the sun jdk1.5
 * source base. The original serializer is extended by a new OutputFormat
 * setting that allows to break the line for each attribute. additionally,
 * all attribute names are sorted alphabetically.
 * Some Features were removed to limit the number of dependent classes:
 * <ul>
 * <li> dom filter support
 * <li> all text nodes as CDATA feature
 * <li> skip attribute default values feature
 * <li> entity node reference feature
 * </ul>
 * <p/>
 *  
 * Implements an XML serializer supporting both DOM and SAX pretty
 * serializing. For usage instructions see {@link Serializer}.
 * <p/>
 * If an output stream is used, the encoding is taken from the
 * output format (defaults to <tt>UTF-8</tt>). If a writer is
 * used, make sure the writer uses the same encoding (if applies)
 * as specified in the output format.
 * <p/>
 * The serializer supports both DOM and SAX. SAX serializing is done by firing
 * SAX events and using the serializer as a document handler. DOM serializing is done
 * by calling {@link #serialize(Document)} or by using DOM Level 3
 * <p/>
 * If an I/O exception occurs while serializing, the serializer
 * will not throw an exception directly, but only throw it
 * at the end of serializing (either DOM or SAX's {@link
 * org.xml.sax.DocumentHandler#endDocument}.
 * <p/>
 * For elements that are not specified as whitespace preserving,
 * the serializer will potentially break long text lines at space
 * boundaries, indent lines, and serialize elements on separate
 * lines. Line terminators will be regarded as spaces, and
 * spaces at beginning of line will be stripped.
 *
 * @author <a href="mailto:arkin@intalio.com">Assaf Arkin</a>
 * @author <a href="mailto:rahul.srivastava@sun.com">Rahul Srivastava</a>
 * @author Elena Litani IBM
 * @see Serializer
 */
public class XMLSerializer extends BaseMarkupSerializer {

    //
    // constants
    //

    protected static final boolean DEBUG = false;

    //
    // data
    //

    //
    // DOM Level 3 implementation: variables initialized in DOMSerializerImpl
    //

    /**
     * stores namespaces in scope
     */
    protected NamespaceSupport fNSBinder;

    /**
     * stores all namespace bindings on the current element
     */
    protected NamespaceSupport fLocalNSBinder;

    /**
     * symbol table for serialization
     */
    protected SymbolTable fSymbolTable;

    protected final static String PREFIX = "NS";

    /**
     * Controls whether namespace fixup should be performed during
     * the serialization.
     * NOTE: if this field is set to true the following
     * fields need to be initialized: fNSBinder, fLocalNSBinder, fSymbolTable,
     * XMLSymbols.EMPTY_STRING, fXmlSymbol, fXmlnsSymbol
     */
    protected boolean fNamespaces = false;


    private boolean fPreserveSpace;


    /**
     * Constructs a new serializer. The serializer cannot be used without
     * calling {@link #setOutputCharStream} or {@link #setOutputByteStream}
     * first.
     */
    public XMLSerializer() {
        super(new OutputFormat(Method.XML, null, false));
    }


    /**
     * Constructs a new serializer. The serializer cannot be used without
     * calling {@link #setOutputCharStream} or {@link #setOutputByteStream}
     * first.
     */
    public XMLSerializer(OutputFormat format) {
        super(format != null ? format : new OutputFormat(Method.XML, null, false));
        _format.setMethod(Method.XML);
    }


    /**
     * Constructs a new serializer that writes to the specified writer
     * using the specified output format. If <tt>format</tt> is null,
     * will use a default output format.
     *
     * @param writer The writer to use
     * @param format The output format to use, null for the default
     */
    public XMLSerializer(Writer writer, OutputFormat format) {
        super(format != null ? format : new OutputFormat(Method.XML, null, false));
        _format.setMethod(Method.XML);
        setOutputCharStream(writer);
    }


    /**
     * Constructs a new serializer that writes to the specified output
     * stream using the specified output format. If <tt>format</tt>
     * is null, will use a default output format.
     *
     * @param output The output stream to use
     * @param format The output format to use, null for the default
     */
    public XMLSerializer(OutputStream output, OutputFormat format) {
        super(format != null ? format : new OutputFormat(Method.XML, null, false));
        _format.setMethod(Method.XML);
        setOutputByteStream(output);
    }


    public void setOutputFormat(OutputFormat format) {
	}


    /**
     * This methods turns on namespace fixup algorithm during
     * DOM serialization.
     *
     * @param namespaces
     */
    public void setNamespaces(boolean namespaces) {
	}

    //-----------------------------------------//
    // SAX content handler serializing methods //
    //-----------------------------------------//


    public void startElement(String namespaceURI, String localName,
                             String rawName, Attributes attrs)
            throws SAXException {
			}


    public void endElement(String namespaceURI, String localName,
                           String rawName)
            throws SAXException {
			}


    public void endElementIO(String namespaceURI, String localName,
                             String rawName)
            throws IOException {
			}

    //------------------------------------------//
    // SAX document handler serializing methods //
    //------------------------------------------//


    public void startElement(String tagName, AttributeList attrs)
            throws SAXException {
			}


    public void endElement(String tagName)
            throws SAXException {
			}

    //------------------------------------------//
    // Generic node serializing methods methods //
    //------------------------------------------//


    /**
     * Called to serialize the document's DOCTYPE by the root element.
     * The document type declaration must name the root element,
     * but the root element is only known when that element is serialized,
     * and not at the start of the document.
     * <p/>
     * This method will check if it has not been called before ({@link #_started}),
     * will serialize the document type declaration, and will serialize all
     * pre-root comments and PIs that were accumulated in the document
     * (see {@link #serializePreRoot}). Pre-root will be serialized even if
     * this is not the first root element of the document.
     */
    protected void startDocument(String rootTagName)
            throws IOException {
			}


    /**
     * Called to serialize a DOM element. Equivalent to calling {@link
     * #startElement}, {@link #endElement} and serializing everything
     * inbetween, but better optimized.
     */
    protected void serializeElement(Element elem)
            throws IOException {
			}


    /**
     * Serializes a namespace attribute with the given prefix and value for URI.
     * In case prefix is empty will serialize default namespace declaration.
     *
     * @param prefix
     * @param uri
     * @throws IOException
     */

    private void printNamespaceAttr(String prefix, String uri) throws IOException {
	}


    /**
     * Prints attribute.
     * NOTE: xml:space attribute modifies output format
     *
     * @param name
     * @param value
     * @param isSpecified
     * @throws IOException
     */
    private void printAttribute(String name, String value, boolean isSpecified, Attr attr) throws IOException {
	}

    protected String getEntityRef(int ch) {
		return null;
	}


    /**
     * Retrieve and remove the namespaces declarations from the list of attributes.
     */
    private Attributes extractNamespaces(Attributes attrs)
            throws SAXException {
				return null;
			}

    //
    // Printing attribute value
    //
    protected void printEscaped(String source) throws IOException {
	}

    /**
     * print text data
     */
    protected void printXMLChar(int ch) throws IOException {
	}

    protected void printText(String text, boolean preserveSpace, boolean unescaped)
            throws IOException {
			}


    protected void printText(char[] chars, int start, int length,
                             boolean preserveSpace, boolean unescaped) throws IOException {
							}


    /**
     * DOM Level 3:
     * Check a node to determine if it contains unbound namespace prefixes.
     *
     * @param node The node to check for unbound namespace prefices
     */
    protected void checkUnboundNamespacePrefixedNode(Node node) throws IOException {
	}

    public boolean reset() {
		return false;
	}

}




