/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */

package com.day.text;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.BitSet;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;
import java.util.Properties;
import java.util.TimeZone;
import java.util.Vector;

/**
 * This class holds a collection of useful string operations that are not
 * available in java.
 */
public final class Text {

    /**
     * The default format pattern used in strftime() if no pattern
     * parameter has been supplied. This is the default format used to format
     * dates in Communiqu&eacute; 2
     */
    public static final String DEFAULT_DATE_FORMAT_PATTERN = "dd.MM.yyyy HH:mm:ss";

    /**
     * Common used DateFormat implementation. When the supplied formatting
     * pattern has been translated it is applied to this formatter and then
     * executed. Both steps occur in a synchronized section, such that no
     * two threads disturb each other.
     * <ol>
     * <li>A single instance of the formatter is used to prevent the object
     * creation overhead. But then how much is this ?
     * <li>Using one static formatter for each thread may prove to give even
     * more overhead given all those synchronized blocks waiting for each
     * other during real formatting. But then how knows ?
     * </ol>
     * <p>
     * This formatter must always be used synchronized as follows :
     * <pre>
            synchronized (dateFormatter) {
                dateFormatter.applyPattern( ... );
                dateFormatter.setTimezone( ... );
                String result = dateFormatter.format( ... );
            }
     *	</pre>
     * <p>
     * To parse date and time strings , the formatter is used as follows :
     * <pre>
            synchronized (dateFormatter) {
                dateFormatter.applyPattern( ... );
                dateFormatter.setTimezone( ... );
                try {
                    Date result = dateFormatter.parse(dateString);
                } catch (ParseException pe) {
                    // handle exception
                }
            }
     *	</pre>
     *
     */
    private static final SimpleDateFormat dateFormatter = new SimpleDateFormat();

    /**
     * The UTC timezone
     */
    public static final TimeZone TIMEZONE_UTC = TimeZone.getTimeZone("UTC");

    /** format for RFC 1123 date string -- "Sun, 06 Nov 1994 08:49:37 GMT" */
    private final static SimpleDateFormat rfc1123Format =
        new SimpleDateFormat("EEE, dd MMM yyyyy HH:mm:ss z", Locale.US);

    static {
        rfc1123Format.setTimeZone(TIMEZONE_UTC);
    }

    /**
     * The local timezone
     */
    public static final TimeZone TIMEZONE_LOCAL = TimeZone.getDefault();

    /**
     * Empty result
     */
    private final static String[] empty = new String[0];

    /**
     * MD4 hash algorithm name.
     */
    private static final String MD4_HASH = "md4";

    /**
     * NTLM hash that requires UnicodeLittleUnmarked encoding.
     */
    private static final String NTLM_HASH = "ntlm";

    /**
     * avoid instantiation
     */
    private Text() {
    }

    /**
     * returns an array of strings decomposed of the original string, split at
     * every occurance of 'ch'. if 2 'ch' follow each other with no intermediate
     * characters, empty "" entries are avoided.
     *
     * @param str the string to decompose
     * @param ch the character to use a split pattern
     * @return an array of strings
     *
     * @deprecated use {@link #explode(String, int)}
     */
    public static String[] split(String str, int ch) {
		return null;
	}

    /**
     * returns an array of strings decomposed of the original string, split at
     * every occurance of 'ch'.
     * @param str the string to decompose
     * @param ch the character to use a split pattern
     * @param respectEmpty if <code>true</code>, empty elements are generated
     * @return an array of strings
     *
     * @deprecated use {@link #explode(String, int, boolean)}
     */
    public static String[] split(String str, int ch, boolean respectEmpty) {
		return null;
	}

    /**
     * returns an array of strings decomposed of the original string, split at
     * every occurance of 'ch'. if 2 'ch' follow each other with no intermediate
     * characters, empty "" entries are avoided.
     *
     * @param str the string to decompose
     * @param ch the character to use a split pattern
     * @return an array of strings
     */
    public static String[] explode(String str, int ch) {
		return null;
	}

    /**
     * returns an array of strings decomposed of the original string, split at
     * every occurance of 'ch'.
     * @param str the string to decompose
     * @param ch the character to use a split pattern
     * @param respectEmpty if <code>true</code>, empty elements are generated
     * @return an array of strings
     */
    public static String[] explode(String str, int ch, boolean respectEmpty) {
		return null;
	}

    /**
     * Concatenates all strings in the string array using the specified delimiter.
     * @param arr
     * @param delim
     * @return the concatenated string
     */
    public static String implode(String[] arr, String delim) {
		return null;
	}

    /**
     * Replaces all occurences of <code>oldString</code> in <code>text</code>
     * with <code>newString</code>.
     *
     * @param text
     * @param oldString old substring to be replaced with <code>newString</code>
     * @param newString new substring to replace occurences of <code>oldString</code>
     * @return a string
     */
    public static String replace(String text, String oldString, String newString) {
		return null;
	}

        /**
     * Returns the name part of the path
     *
     * @param path the path
     * @return the name part
     */
        public static String getName(String path) {
			return null;
		}

    /**
     * Returns the name part of the path, delimited by the given <code>delim</code>
     *
     * @param path the path
     * @param delim the delimiter
     * @return the name part
     */
    public static String getName(String path, char delim) {
		return null;
	}

    /**
     * Same as {@link #getName(String)} but adding the possibility
     * to pass paths that end with a trailing '/'
     *
     * @param path the path
     * @param ignoreTrailingSlash if <code>true</code> a trailing slash is ignored
     * @return the name part
     *
     * @see #getName(String)
     */
    public static String getName(String path, boolean ignoreTrailingSlash) {
		return null;
	}

    /**
     * Returns the namespace prefix of the given <code>qname</code>. If the
     * prefix is missing, an empty string is returned. Please note, that this
     * method does not validate the name or prefix.
     * </p>
     * the qname has the format: qname := [prefix ':'] local;
     *
     * @param qname a qualified name
     * @return the prefix of the name or "".
     *
     * @see #getLocalName(String)
     *
     * @throws NullPointerException if <code>qname</code> is <code>null</code>
     */
    public static String getNamespacePrefix(String qname) {
		return null;
	}

    /**
     * Returns the local name of the given <code>qname</code>. Please note, that
     * this method does not validate the name.
     * </p>
     * the qname has the format: qname := [prefix ':'] local;
     *
     * @param qname a qualified name
     * @return the localname
     *
     * @see #getNamespacePrefix(String)
     *
     * @throws NullPointerException if <code>qname</code> is <code>null</code>
     */
    public static String getLocalName(String qname) {
		return null;
	}

    /**
     * compares to handles lexigographically with one exception: the '/'
     * character is always considered smaller than all other chars. this results
     * in a ordering, in which the parent pages come first (it's about 6 times
     * slower than the string impl. of compareTo).
     * <ul>example (normal string compare):
     * <li>/foo
     * <li>/foo-bar
     * <li>/foo/bar</li>
     * </ul>
     * <ul>example (this handle compare):
     * <li>/foo
     * <li>/foo/bar</li>
     * <li>/foo-bar
     * </ul>
     *
     * @param h1 the first handle
     * @param h2 the second handle
     * @return the return is positive, if the first handle is bigger than the
     *         second; negative, if the first handle is smaller than the second;
     *         and zero, if the two handles are equal.
     */
    public static int compareHandles(String h1, String h2) {
		return 0;
	}

    /**
     * this method does the similar than the {@link #fullPath(String, String)}
     * but it considers the <i>parent</i> as parent directory rather than a base
     * handle. if further respects full qualified uri's.
     * <br>examples:
     * <xmp>
     * parent    | path     | result
     * ----------+----------+------------
     * ""        | ""       | /
     * /foo      | ""       | /foo
     * ""        | /foo     | /foo
     * "."       | foo      | foo
     * /foo/bar  | bla      | /foo/bar/bla
     * /foo/bar  | /bla     | /bla
     * /foo/bar  | ../bla   | /foo/bla
     * /foo/bar  | ./bla    | /foo/bar/bla
     * foo       | bar      | foo/bar
     * c:/bla    | gurk     | c:/bla/gurk
     * /foo      | c:/bar   | c:/bar
     * </xmp>
     *
     * @param parent the base handle
     * @param path the path
     */
    public static String fullFilePath(String parent, String path) {
		return null;
	}

    /**
     * returns a full path.
     * if base is empty, '/' is assumed
     * if base and path are relative, a relative path will be generated.
     * <br>examples:
     * <pre>
     * base      | path     | result
     * ----------+----------+------------
     * ""        | ""       | /
     * /foo      | ""       | /foo
     * ""        | /foo     | /foo
     * "."       | foo      | foo
     * /foo/bar  | bla      | /foo/bla
     * /foo/bar  | /bla     | /bla
     * /foo/bar  | ../bla   | /bla
     * /foo/bar  | ./bla    | /foo/bla
     * foo       | bar      | bar
     * </pre>
     *
     * @param base the base handle
     * @param path the path
     */
    public static String fullPath(String base, String path) {
		return null;
	}

    /**
     * Make a path canonical. This is a shortcut for
     * <code>
     * Text.makeCanonicalPath(new StringBuffer(path));
     * </code>
     * @param path path to make canonical
     */
    public static String makeCanonicalPath(String path) {
		return null;
	}

    /**
     * make a cannonical path. removes all /./ and /../ and multiple slashes.
     * @param source the input source
     * @return a string containing the cannonical path
     *
     * @deprecated use {@link #makeCanonicalPath(char[])} instead
     */
    public static String makeCanonicalPath(StringBuffer source) {
		return null;
	}

    /**
     * make a cannonical path. removes all /./ and /../ and multiple slashes.
     * @param source the input source
     * @return a string containing the cannonical path
     */
    public static String makeCanonicalPath(char[] source) {
		return null;
	}

    /**
     * Determines, if two handles are sister-pages, that meens, if they
     * represent the same hierarchic level and share the same parent page.
     * @param h1 first handle
     * @param h2 second handle
     * @return true if on same level, false otherwise
     */
     public static boolean isSibling(String h1, String h2) {
		return false;
	}

    /**
     * Determines if the <code>descendant</code> handle is hierarchical a
     * descendant of <code>handle</code>.
     * <xmp>
     * /content/playground/en  isDescendantOf     /content/playground
     * /content/playground/en  isDescendantOf     /content
     * /content/playground/en  isNOTDescendantOf  /content/designground
     * /content/playground/en  isNOTDescendantOf  /content/playground/en
     * </xmp>
     *
     * @param handle the current handle
     * @param descendant the potential descendant
     * @return <code>true</code> if the <code>descendant</code> is a descendant;
     *         <code>false</code> otherwise.
     * @since gumbear
     */
    public static boolean isDescendant(String handle, String descendant) {
		return false;
	}

    /**
     * Determines if the <code>descendant</code> handle is hierarchical a
     * descendant of <code>handle</code> or equal to it.
     * <xmp>
     * /content/playground/en  isDescendantOrEqualOf     /content/playground
     * /content/playground/en  isDescendantOrEqualOf     /content
     * /content/playground/en  isDescendantOrEqualOf  /content/playground/en
     * /content/playground/en  isNOTDescendantOrEqualOf  /content/designground
     * </xmp>
     *
     * @param handle the current handle
     * @param descendant the potential descendant
     * @return <code>true</code> if the <code>descendant</code> is a descendant
     *         or equal; <code>false</code> otherwise.
     * @since gumbear
     */
    public static boolean isDescendantOrEqual(String handle, String descendant) {
		return false;
	}

    /**
     * Returns the label of a handle
     * @param handle the handle
     * @return the label
     * @deprecated use {@link #getName(String)}
     */
    public static String getLabel(String handle) {
		return null;
	}

    /**
     * Returns the label of a string
     * @param handle the string
     * @param delim the delimiter
     * @return the label
     *
     * @deprecated use {@link #getName(String, char)}
     */
    public static String getLabel(String handle, char delim) {
		return null;
	}

    /**
     * used for the md5
     */
    public final static char[] hexTable="0123456789abcdef".toCharArray();

    /**
     * Calculate an MD5 hash of the string given.
     * @param data the data to encode
     * @param enc the character encoding to use
     * @return a hex encoded string of the md5 digested input
     * @throws UnsupportedEncodingException if the given encoding is not supported
     */
    public static String md5(String data, String enc)
            throws UnsupportedEncodingException {
				return null;
			}

    /**
     * Calculate an MD5 hash of the string given.
     * @param data the data to encode
     * @return a hex encoded string of the md5 digested input
     *
     * @deprecated {@link #md5(String, String)}
     */
    public static String md5(String data) {
		return null;
	}

    /**
     * Digest the plain string using the given algorithm.
     *
     * @param algorithm The alogrithm for the digest. This algorithm must be
     *        supported by the MessageDigest class.
     * @param data The plain text String to be digested.
     * @param enc The character encoding to use
     *
     * @return The digested plain text String represented as Hex digits.
     *
     * @throws NoSuchAlgorithmException if the desired algorithm is not supported by
     *         the MessageDigest class.
     * @throws UnsupportedEncodingException if the encoding is not supported
     */
    public static String digest(String algorithm, String data, String enc)
            throws NoSuchAlgorithmException, UnsupportedEncodingException {
				return null;
			}

    /**
     * Digest the plain string using the given algorithm.
     *
     * @param algorithm The alogrithm for the digest. This algorithm must be
     *        supported by the MessageDigest class.
     * @param data The plain text String to be digested.
     *
     * @return The digested plain text String represented as Hex digits.
     *
     * @throws NoSuchAlgorithmException if the desired algorithm is not supported by
     *         the MessageDigest class.
     *
     * @deprecated use {@link #digest(String, String, String)}
     */
    public static String digest(String algorithm, String data)
            throws NoSuchAlgorithmException {
				return null;
			}

    /**
     * Digest the plain string using the given algorithm.
     *
     * @param algorithm The alogrithm for the digest. This algorithm must be
     *        supported by the MessageDigest class.
     * @param data the data to digest with the given algorithm
     *
     * @return The digested plain text String represented as Hex digits.
     *
     * @throws NoSuchAlgorithmException if the desired algorithm is not supported by
     *         the MessageDigest class.
     */
    public static String digest(String algorithm, byte[] data)
            throws NoSuchAlgorithmException {
				return null;
			}

    /**
     * Returns the n<sup>th</sup> relative parent of the handle, where n=level.
     * <p>Example:<br>
     * <code>
     * Text.getRelativeParent("/en/home/index/about", 1) == "/en/home/index"
     * </code>
     *
     * @param handle the handle of the page
     * @param level the level of the parent
     */
    public static String getRelativeParent(String handle, int level) {
		return null;
	}

    /**
     * Returns the n<sup>th</sup> absolute parent of the handle, where n=level.
     * <p>Example:<br>
     * <code>
     * Text.getAbsoluteParent("/en/home/index/about", 1) == "/en/home"
     * </code>
     *
     * @param handle the handle of the page
     * @param level the level of the parent
     */
    public static String getAbsoluteParent(String handle, int level) {
		return null;
	}

    /**
     * The list of characters that are not encoded by the <code>escape()</code>
     * and <code>unescape()</code> methods. They contains the characters as
     * defined 'unreserved' in section 2.3 of the RFC 2396 'URI genric syntax':
     *
     * <pre>
     * unreserved  = alphanum | mark
     * mark        = "-" | "_" | "." | "!" | "~" | "*" | "'" | "(" | ")"
     * </pre>
     */
    public static BitSet URISave;

    /**
     * Same as {@link #URISave} but also contains the '/'
     */
    public static BitSet URISaveEx;

    static {
        URISave = new BitSet(256);
        int i;
        for (i = 'a'; i <= 'z'; i++) {
            URISave.set(i);
        }
        for (i = 'A'; i <= 'Z'; i++) {
            URISave.set(i);
        }
        for (i = '0'; i <= '9'; i++) {
            URISave.set(i);
        }
        URISave.set('-');
        URISave.set('_');
        URISave.set('.');
        URISave.set('!');
        URISave.set('~');
        URISave.set('*');
        URISave.set('\'');
        URISave.set('(');
        URISave.set(')');

        URISaveEx = (BitSet) URISave.clone();
        URISaveEx.set('/');
    }

    /**
     * Does an URL encoding of the <code>string</code> using the
     * <code>escape</code> character. The characters that don't need encoding
     * are those defined 'unreserved' in section 2.3 of the 'URI genric syntax'
     * RFC 2396, but without the escape character.
     *
     * @param string the string to encode.
     * @param escape the escape character.
     * @return the escaped string
     *
     * @throws NullPointerException if <code>string</code> is <code>null</code>.
     */
    public static String escape(String string, char escape) {
		return null;
	}

    /**
     * Does an URL encoding of the <code>string</code> using the
     * <code>escape</code> character. The characters that don't need encoding
     * are those defined 'unreserved' in section 2.3 of the 'URI genric syntax'
     * RFC 2396, but without the escape character. If <code>isPath</code> is
     * <code>true</code>, additionally the slash '/' is ignored, too.
     *
     * @param string the string to encode.
     * @param escape the escape character.
     * @param isPath if <code>true</code>, the string is treated as path
     *
     * @return the escaped string
     *
     * @throws NullPointerException if <code>string</code> is <code>null</code>.
     */
    public static String escape(String string, char escape, boolean isPath) {
		return null;
	}

    /**
     * Escapes all character that have specific meaning in XMl/HTML, namely
     * '&lt;', '&gt;', '"', '&amp;' by their isolating encoding aequivalents
     *  "&amp;lt;", "&amp;gt", "&amp;quot;", "&amp;amp;"
     *
     * @param string the string to escape
     * @return the escaped string or <code>null</code> if the input string was
     *         <code>null</code>
     */
    public static String escapeXml(String string) {
		return null;
	}

    /**
     * Does a URL encoding of the <code>string</code>. The characters that
     * don't need encoding are those defined 'unreserved' in section 2.3 of
     * the 'URI genric syntax' RFC 2396.
     *
     * @param string the string to encode
     * @return the escaped string
     *
     * @throws NullPointerException if <code>string</code> is <code>null</code>.
     */
    public static String escape(String string) {
		return null;
	}

    /**
     * Does a URL encoding of the <code>path</code>. The characters that
     * don't need encoding are those defined 'unreserved' in section 2.3 of
     * the 'URI genric syntax' RFC 2396. In contrast to the
     * {@link #escape(String)} method, not the entire path string is escaped,
     * but every individual part (i.e. the slashes are not escaped).
     *
     * @param path the path to encode
     * @return the escaped path
     *
     * @throws NullPointerException if <code>path</code> is <code>null</code>.
     */
    public static String escapePath(String path) {
		return null;
	}

    /**
     * Does a URL decoding of the <code>string</code> using the
     * <code>escape</code> character. Please note that in opposite to the
     * {@link java.net.URLDecoder} it does not transform the + into spaces.
     * @param string the string to decode
     * @param escape the escape character
     * @return the decoded string
     *
     * @throws NullPointerException if <code>string</code> is <code>null</code>.
     * @throws ArrayIndexOutOfBoundsException if not enough character follow an
     *         escape character
     * @throws IllegalArgumentException if the 2 characters following the escape
     *         character do not represent a hex-number.
     */
    public static String unescape(String string, char escape) {
		return null;
	}

    /**
     * Does a URL decoding of the <code>string</code>. Please note that in
     * opposite to the {@link java.net.URLDecoder} it does not transform the +
     * into spaces.
     *
     * @param string the string to decode
     * @return the decoded string
     *
     * @throws NullPointerException if <code>string</code> is <code>null</code>.
     * @throws ArrayIndexOutOfBoundsException if not enough character follow an
     *         escape character
     * @throws IllegalArgumentException if the 2 characters following the escape
     *         character do not represent a hex-number.
     */
    public static String unescape(String string) {
		return null;
	}

    /**
     * Returns a stringified date accoring to the date format specified in
     * RTF1123. this is of the form: "Sun, 06 Nov 1994 08:49:37 GMT"
     */
    public static String dateToRfc1123String(Date date) {
		return null;
	}

    /**
     * Implements a date formatting routine supporting (a subset) of the POSIX
     * <code>strftime()</code> function.
     *
     * @param date The date value to be formatted
     * @param formatPattern The pattern used to format the date. This pattern
     * 		supports a subset of the pattern characters of the POSIX
     * 		<code>strftime()</code> function. If this pattern is empty or
     * 		<code>null</code> the default pattern
     * 		<code>dd.MM.yyyy HH:mm:ss</code> is used.
     * @param zone Defines for which time zone the date should be outputted. If
     *      this parameter is <code>null</code>, then the local time zone is taken.
     *
     * @return the formatted date as a String.
     */
    public static String strftime(Date date, String formatPattern, TimeZone zone) {
		return null;
	}

    /**
     * Implements a date formatting routine supporting (a subset) of the POSIX
     * <code>strftime()</code> function.
     *
     * @param date The date value to be formatted
     * @param formatPattern The pattern used to format the date. This pattern
     * 		supports a subset of the pattern characters of the POSIX
     * 		<code>strftime()</code> function. If this pattern is empty or
     * 		<code>null</code> the default pattern
     * 		<code>dd.MM.yyyy HH:mm:ss</code> is used.
     * @param   asUTC Defines whether to interpret the date as belong to the UTC
     * 		time zone or the local time zone.
     */
    public static String strftime(Date date, String formatPattern, boolean asUTC) {
		return null;
	}

    /**
     * Implements a date formatting routine supporting (a subset) of the POSIX
     * <code>strftime()</code> function. The default pattern
     * <code>dd.MM.yyyy HH:mm:ss</code> is used to format the date.
     *
     * @param date The date value to be formatted
     * @param asUTC Defines whether to interpret the date as belong to the UTC
     * 		time zone or the local time zone.
     */
    public static String strftime(Date date, boolean asUTC) {
		return null;
	}

    /**
     * Implements a date formatting routine supporting (a subset) of the POSIX
     * <code>strftime()</code> function. The default pattern
     * <code>dd.MM.yyyy HH:mm:ss</code> is used to format the date, which is
     * interpreted to be in the local time zone.
     *
     * @param date The date value to be formatted
     */
    public static String strftime(Date date) {
		return null;
	}

    /**
     * Parses the date string based on the format pattern which is in the
     * format used by the Java platfrom SimpleDateFormat class.
     *
     * @param dateString The date string to be parsed
     * @param formatPattern the pattern to use for parsing. If <code>null</code>
     * 		or empty, the same default pattern is used as with
     * 		{@link #strftime(Date, String, boolean)}, namely
     * 		<code>dd.MM.yyyy HH:mm:ss</code>.
     *
     * @throws ParseException if the date string cannot be parsed accordinung
     * 		to the format pattern.
     */
    public static Date parseDate(String dateString, String formatPattern)
            throws ParseException {
				return null;
			}

    /**
     * Parses the date string based on the format pattern which is in the
     * format used by the Java platfrom SimpleDateFormat class.
     *
     * @param dateString The date string to be parsed
     * @param formatPattern the pattern to use for parsing. If <code>null</code>
     * 		or empty, the same default pattern is used as with
     * 		{@link #strftime(Date, String, boolean)}, namely
     * 		<code>dd.MM.yyyy HH:mm:ss</code>.
     * @param isUTC if <code>true</code> the date string is considered in UTC,
     *        otherwise the default timezone of the host is used.
     *
     * @throws ParseException if the date string cannot be parsed accordinung
     * 		to the format pattern.
     */
    public static Date parseDate(String dateString, String formatPattern,
                                 boolean isUTC)
            throws ParseException {
				return null;
			}

    /**
     * Parses the date string based on the format pattern which is in the
     * default format <code>dd.MM.yyyy HH:mm:ss</code>.
     *
     * @param dateString The date string to be parsed
     *
     * @throws ParseException if the date string cannot be parsed accordinung
     * 		to the format pattern.
     */
    public static Date parseDate(String dateString) throws ParseException {
		return null;
	}

    /**
     * Parses the date string based on the format pattern which is in the
     * default format <code>dd.MM.yyyy HH:mm:ss</code>.
     *
     * @param dateString The date string to be parsed
     * @param isUTC if <code>true</code> the date string is considered in UTC,
     *        otherwise the default timezone of the host is used.
     *
     *
     * @throws ParseException if the date string cannot be parsed accordinung
     * 		to the format pattern.
     */
    public static Date parseDate(String dateString, boolean isUTC)
            throws ParseException {
				return null;
			}

    //--------- sprintf() formatting constants ---------------------------------

    /** left justified - '-' flag */
    private static final int FLAG_LJ = 1;

    /** always show sign - '+' flag */
    private static final int FLAG_SI = 2;

    /** space placeholder for omitted plus sign - ' ' flag, ignore if SI */
    private static final int FLAG_SP = 3;

    /** zero padded if right aligned - '0' flag, ignore if LJ */
    private static final int FLAG_ZE = 4;

    /**
     * alternate format - '#' flag :
     * 	SI  - incr. precision to have zero as first char
     * 	x   - prefix '0x'
     * 	X   - prefix '0X'
     *  eEf - always show decimal point, omit trailing zeroes
     *  gG  - always show decimal point, show trailing zeroes
     */
    private static final int FLAG_AL = 5;

    /** interpret ints as short - 'h' size */
    private static final int FLAG_SHORT = 8;

    /** interpret ints as long - 'l' size */
    private static final int FLAG_LONG = 9;

    /** waiting for format */
    private static final int PARSE_STATE_NONE = 0;

    /** parsing flags */
    private static final int PARSE_STATE_FLAGS = 1;

    /** parsing wdth */
    private static final int PARSE_STATE_WIDTH = 2;

    /** parsing precision */
    private static final int PARSE_STATE_PRECISION = 3;

    /** parsing size */
    private static final int PARSE_STATE_SIZE = 4;

    /** parsing type */
    private static final int PARSE_STATE_TYPE = 5;

    /** parsing finished */
    private static final int PARSE_STATE_END = 6;

    /** incomplete pattern at end of format string, throw error */
    private static final int PARSE_STATE_ABORT = 7;

    /** end of format string during plain text */
    private static final int PARSE_STATE_TERM = 8;

    /**
     * This method implements the famous and ubiquituous <code>sprintf</code>
     * formatter in Java. The arguments to the method are the formatting string
     * and the list of arguments defined by the formatting instructions
     * contained in the formatting string.
     * <p>
     * Each element in the argument array is either a <code>Number</code> object
     * or any other <code>Object</code> type. Whenever the formatting string
     * stipulates the corresponding argument to be numeric, it is assumed this
     * array element to be a <code>Number</code>. If this is not the case an
     * <code>IllegalArgumentException</code> is thrown. If a <code>String</code>
     * argument is stipulated by the formatting string, a simple call to the
     * <code>toString()</code> method of the object yields the
     * <code>String</code> required.
     * <p>
     * <b>SPECIFICATION</b>
     * <p>
     * <code>sprintf</code> accepts a series of arguments, applies to each a
     * format specifier from <code>format</code>, and stores the formatted data
     * to the result <code>Strint</code>. The method throws an
     * <code>IllegalArgumentException</code> if either the <code>format</code>
     * is incorrect, there are not enough arguments for the <code>format</code>
     * or if any of the argument's type is wrong. <code>sprintf</code> returns
     * when it  reaches the end of the format string. If there are more
     * arguments than the format requires, excess arguments are ignored.
     * <p>
     * <code>format</code> is a <code>String</code> containing two types of
     * objects: ordinary characters (other than <code><i>%</i></code>), which
     * are copied unchanged to the output, and conversion  specifications,
     * each of which is introduced by <code><i>%</i></code>. (To include
     * <code><i>%</i></code> in the output, use <code>%%</code> in the format
     * string.) A conversion specification has the following form:
     * <blockquote>
     *   <code>
     *     [ <i>flags</i> ] [ <i>width</i> ] [ "." <i>prec</i> ] [ <i>size</i> ]
     *     <i>type</i>
     *   </code>
     * </blockquote>
     * <p>
     * The  fields  of the conversion specification have the following meanings:
     *
     * <dl>
     * <dt><i>flags</i>
     * <dd>an optional sequence of characters  which  control  output
     * justification, numeric  signs,  decimal  points, trailing zeroes, and
     * octal and hex prefixes. The  flag  characters are minus (<code>-</code>),
     * plus (<code>+</code>), space ("<code> </code>"), zero (<code>0</code>),
     * and sharp (<code>#</code> ). They can appear in any combination.
     *
     * <table>
     *   <tr>
     *     <td><code>-</code></td>
     *     <td>The result of the conversion is left justified, and the right
     *         is  padded with blanks. If you do not use this flag, the result
     *         is right justified,  and  padded  on  the left.</td>
     *   </tr>
     *   <tr>
     *   <td><code>+</code></td>
     *   <td>The  result of a signed conversion (as determined by <i>type</i>)
     *       will always begin with a plus or minus sign. (If you do not use
     *       this flag, positive values do not begin with a plus sign.)</td>
     *   </tr>
     *   <tr>
     *     <td>"<code> </code>" (space)</td>
     *     <td>If the first character of a  signed conversion  specification is
     *         not a sign, or if a signed conversion results in no characters,
     *         the result will begin  with a space. If the space
     *         (<code> </code>) flag and the plus (<code>+</code>) flag both
     *         appear, the space flag is ignored.</td>
     *   </tr>
     *   <tr>
     *     <td><code>0</code></td>
     *     <td>If the <i>type</i> is <code>d</code>, <code>i</code>,
     *         <code>o</code>, <code>u</code>, <code>x</code>, <code>X</code>,
     *         <code>e</code>, <code>E</code>, <code>f</code>, <code>g</code>,
     *         or <code>G</code>:  leading  zeroes,  are  used to pad the field
     *         width (following any indication of sign or base); no spaces
     *         are used  for  padding.  If the zero (<code>0</code>) and minus
     *         (<code>-</code>) flags both appear, the zero (<code>0</code>)
     *         flag will be ignored. For <code>d</code>, <code>i</code>,
     *         <code>o</code>, <code>u</code>, <code>x</code>, <code>X</code>
     *         conversions, if a precision <i>prec</i> is specified, the zero
     *         (<code>0</code>) flag is ignored.
     *         <br>
     *         Note that <i>0</i> is interpreted as a flag, not as the
     *         beginning of a field width.</td>
     *   </tr>
     *   <tr>
     *     <td><code>#</code></td>
     *     <td>The result is to be converted to an alternative form, according
     *         to the <i>type</i> character:
     *         <dl>
     *         <dt><code>o</code>
     *         <dd>Increases precision to force the first digit of the result
     *             to be a zero.
     *
     *         <dt><code>x</code>
     *         <dl>A non-zero result will have a <code>0x</code> prefix.
     *
     *         <dt><code>X</code>
     *         <dl>A non-zero result will have a <code>0X</code> prefix.
     *
     *         <dt><code>e</code>, <code>E</code>, or <code>f</code>
     *         <dl>The result will always contain a decimal point even if no
     *             digits follow the point. (Normally, a decimal point appears
     *             only if a digit follows it.) Trailing zeroes are removed.
     *
     *         <dt><code>g</code> or <code>G</code>
     *         <dl>Same as <code>e</code> or <code>E</code>, but trailing
     *             zeroes arenot removed.
     *
     *         <dt>all others
     *         <dd>Undefined.
     *         </dl></td>
     *   </tr>
     * </table>
     *
     * <dt><i>width</i>
     * <dd><i>width</i> is  an optional minimum field width. You can either
     * specify it directly as a decimal integer, or indirectly by using instead
     * an asterisk (<code>*</code>), in which case an integral numeric argument
     * is used as the field width. Negative field widths are not supported; if
     * you attempt to specify a negative field width, it is interpreted as a
     * minus (<code>i</code>) flag followed  by a positive field width.
     *
     * <dt><i>prec</i>
     * <dd>an  optional field; if present, it is introduced with
     * `<code>.</code>' (a period). This field gives the maximum number of
     * characters  to print in a conversion; the minimum number of digits of an
     * integer to print, for conversions with <i>type</i> <code>d</code>,
     * <code>i</code>, <code>o</code>, <code>u</code>, <code>x</code>, and
     * <code>X</code>; the maximum number of significant digits, for the
     * <code>g</code> and <code>G</code> conversions; or the number of digits
     * to  print after the decimal point, for <code>e</code>, <code>E</code>,
     * and <code>f</code> conversions. You can specify the precision either
     * directly as  a decimal  integer  or indirectly by using an asterisk
     * (<code>*</code>), in which case an integral numeric argument is used as
     * the  precision. Supplying  a negative precision is equivalent to
     * omitting the precision.  If only a period is specified  the  precision
     * is zero.  If a precision appears with any other conversion <i>type</i>
     * than those listed here, the behavior is undefined.
     *
     * <dt><i>size</i>
     * <dd><code>h</code>, <code>l</code>, and <code>L</code> are optional size
     * characters which override the default way that <code>sprintf</code>
     * interprets the  data  type  of the  corresponding argument.
     * <code>h</code> forces the following <code>d</code>, <code>i</code>,
     * <code>o</code>, <code>u</code>, <code>x</code>, or <code>X</code>
     * conversion <i>type</i> to apply to a <code>short</code> or <code>unsigned
     * short</code>. Similarily, an <code>l</code> forces the following
     * <code>d</code>, <code>i</code>, <code>o</code>, <code>u</code>,
     * <code>x</code>, or <code>X</code> conversion  <i>type</i> to apply  to
     * a <code>long</code> or <code>unsigned long</code>. If an <code>h</code>
     * or an <code>l</code> appears with another conversion specifier, the
     * behavior  is undefined. <code>L</code> forces  a following
     * <code>e</code>, <code>E</code>, <code>f</code>, <code>g</code>, or
     * <code>G</code> conversion <i>type</i> to apply to a <code>long
     * double</code> argument. If <code>L</code> appears  with any other
     * conversion <i>type</i>, the behavior is undefined.
     *
     * <dt><i>type</i>
     * <dd><i>type</i> specifies  what  kind of conversion <code>sprintf</code>
     * performs. Here is a table of these:
     *
     *   <dl>
     *   <dt><code>%</code>
     *   <dd>prints the percent character (<code>%</code>)
     *
     *   <dt><code>c</code>
     *   <dd>prints <i>arg</i> as single character. That is the argument is
     *       converted to a <code>String</code> of which only the first
     *       character is printed.
     *
     *   <dt><code>s</code>
     *   <dd>Prints characters until precision is reached or the
     *       <code>String</code> ends; takes any <code>Object</code> whose
     *       <code>toString()</code> method is called to get the
     *       <code>String</code> to print.
     *
     *   <dt><code>d</code>
     *   <dd>prints a signed decimal integer; takes a <code>Number</code> (same
     *       as <code>i</code>)
     *
     *   <dt><code>d</code>
     *   <dd>prints a signed decimal integer; takes a <code>Number</code> (same
     *       as <code>d</code>)
     *
     *   <dt><code>d</code>
     *   <dd>prints a signed octal integer; takes a <code>Number</code>
     *
     *   <dt><code>u</code>
     *   <dd>prints a unsigned decimal integer; takes a <code>Number</code>.
     *       This conversion is not supported correctly by the Java
     *       implementation and is really the same as <code>i</code>.
     *
     *   <dt><code>x</code>
     *   <dd>prints an unsigned hexadecimal integer (using <i>abcdef</i> as
     *       digits beyond <i>9</i>; takes a <code>Number</code>
     *
     *   <dt><code>X</code>
     *   <dd>prints an unsigned hexadecimal integer (using <i>ABCDEF</i> as
     *       digits beyond <i>9</i>; takes a <code>Number</code>
     *
     *   <dt><code>f</code>
     *   <dd>prints a signed value of the form <i>[-]9999.9999</i>; takes a
     *       <code>Number</code>
     *
     *   <dt><code>e</code>
     *   <dd>prints a signed value of the form <i>[-]9.9999e[+|-]999</i>; takes
     *       a <code>Number</code>
     *
     *   <dt><code>E</code>
     *   <dd>prints the same way as <code>e</code>, but using <i>E</i> to
     *       introduce the exponent; takes a <code>Number</code>
     *
     *   <dt><code>g</code>
     *   <dd>prints a signed value in either <code>f</code> or <code>e</code>
     *       form, based  on given value and precision &emdash; trailing zeros
     *       and the decimal point are printed only if necessary; takes a
     *       <code>Number</code>
     *
     *   <dt><code>G</code>
     *   <dd>prints the same way as <code>g</code>, but using <code>E</code>
     *       for the exponent if an exponent is needed; takes a
     *       <code>Number</code>
     *
     *   <dt><code>n</code>
     *   <dd>Not supported in the Java implementation, throws an
     *       <code>IllegalArgumentException</code> if used.
     *
     *   <dt><code>p</code>
     *   <dd>Not supported in the Java implementation, throws an
     *       <code>IllegalArgumentException</code> if used.
     *   </dl>
     *
     * </dl>
     *
     * <p>
     * <b>IMPLEMENTATION NOTES</b>
     * <p>
     * Due to the nature of the Java programming language, neither pointer
     * conversions, nor <code>unsigned</code> and <code>long double</code>
     * conversions are supported.
     * <p>
     * Also the Java implementation only distinguishes between
     * <code>Number</code> and other <code>Object</code> arguments. If a
     * numeric argument is expected as per the <code>format</code>
     * <code>String</code>, the current argument must be a <code>Number</code>
     * object that is converted to a basic type as expected. If a
     * <code>String</code> or <code>char</code> argument is expected, any
     * Object is valid, whose <code>toString()</code> method is used to convert
     * to a <code>String</code>.
     *
     * @param format The format string as known from the POSIX sprintf()
     * 		C function.
     * @param args The list of arguments to accomodate the format string. This
     * 		argument list is supposed to contain at least as much entries as
     * 		there are formatting options in the format string. If for a
     * 		numeric option, the entry is not a number an
     * 		<code>IllegalArgumentException</code> is thrown.
     *
     * @return The formatted <code>String</code>. An empty <code>String</code>
     * 		is only returned if the <code>format</code> <code>String</code>
     * 		is empty. A <code>null</code> value is never returned.
     *
     * @throws NullPointerException if the formatting string or any of the
     * 		argument values is <code>null</code>.
     * @throws IllegalArgumentException if the formatting string has wrong
     *         	format tags, if the formatting string has an incomplete
     * 		formatting pattern at the end of the string, if the argument
     * 		vector has not enough values to satisfy the formatting string
     * 		or if an argument's type does not match the requirements of the
     * 		format string.
     *
     */
    public static String sprintf(String format, Object[] args) {
		return null;
	}

    /**
     * Formats a string according to format and argument. This method only
     * supports string formats. See {@link #sprintf(String, Object[])} for details.
     *
     * @param format The format string
     * @param a0 The single parameter
     *
     * @return the result from <code>sprintf(format, new Object[]{ a0 })</code>.
     *
     * @throws IllegalArgumentException from {@link #sprintf(String, Object[])}.
     */
    public static String sprintf(String format, Object a0) {
		return null;
	}

    /**
     * Formats a string according to format and argument. This method only
     * supports string formats. See {@link #sprintf(String, Object[])} for details.
     *
     * @param format The format string
     * @param a0 The first parameter
     * @param a1 The second parameter
     *
     * @return the result from <code>sprintf(format, new Object[]{ ... })</code>.
     *
     * @throws IllegalArgumentException from {@link #sprintf(String, Object[])}.
     */
    public static String sprintf(String format, Object a0, Object a1) {
		return null;
	}

    /**
     * Formats a string according to format and argument. This method only
     * supports string formats. See {@link #sprintf(String, Object[])} for details.
     *
     * @param format The format string
     * @param a0 The first parameter
     * @param a1 The second parameter
     * @param a2 The thrid parameter
     *
     * @return the result from <code>sprintf(format, new Object[]{ ... })</code>.
     *
     * @throws IllegalArgumentException from {@link #sprintf(String, Object[])}.
     */
    public static String sprintf(String format, Object a0, Object a1,
                                 Object a2) {
									return null;
								}

    /**
     * Formats a string according to format and argument. This method only
     * supports string formats. See {@link #sprintf(String, Object[])} for details.
     *
     * @param format The format string
     * @param a0 The first parameter
     * @param a1 The second parameter
     * @param a2 The thrid parameter
     * @param a3 The fourth parameter
     *
     * @return the result from <code>sprintf(format, new Object[]{ ... })</code>.
     *
     * @throws IllegalArgumentException from {@link #sprintf(String, Object[])}.
     */
    public static String sprintf(String format, Object a0, Object a1,
                                 Object a2, Object a3) {
									return null;
								}

    /**
     * Formats a string according to format and argument. This method only
     * supports string formats. See {@link #sprintf(String, Object[])} for details.
     *
     * @param format The format string
     * @param a0 The first parameter
     * @param a1 The second parameter
     * @param a2 The thrid parameter
     * @param a3 The fourth parameter
     * @param a4 The fifth parameter
     *
     * @return the result from <code>sprintf(format, new Object[]{ ... })</code>.
     *
     * @throws IllegalArgumentException from {@link #sprintf(String, Object[])}.
     */
    public static String sprintf(String format, Object a0, Object a1,
                                 Object a2, Object a3, Object a4) {
									return null;
								}

    /** The list of characters that valid label characters */
    private final static String[] labelCharMapping = new String[] {
        "_","_","_","_","_","_","_","_","_","_","_","_","_","_","_","_",
        "_","_","_","_","_","_","_","_","_","_","_","_","_","_","_","_",
        "_","_","_","_","_","_","_","_","_","_","_","_","_","-","_","_",
        "0","1","2","3","4","5","6","7","8","9","_","_","_","_","_","_",
        "_","a","b","c","d","e","f","g","h","i","j","k","l","m","n","o",
        "p","q","r","s","t","u","v","w","x","y","z","_","_","_","_","_",
        "_","a","b","c","d","e","f","g","h","i","j","k","l","m","n","o",
        "p","q","r","s","t","u","v","w","x","y","z","_","_","_","_","_",
        "_","f","_","_","_","fi","fi","_","_","_","_","_","_","_","_","_",
        "_","_","_","_","_","_","_","_","_","_","_","_","y","_","_","_",
        "_","i","c","p","o","v","_","s","_","_","_","_","_","_","_","_",
        "_","_","_","_","_","_","_","_","_","_","_","_","_","_","_","_",
        "a","a","a","a","ae","a","ae","c","e","e","e","e","i","i","i","i",
        "d","n","o","o","o","o","oe","x","o","u","u","u","ue","y","b","ss",
        "a","a","a","a","ae","a","ae","c","e","e","e","e","i","i","i","i",
        "o","n","o","o","o","o","oe","_","o","u","u","u","ue","y","b","y"
    };

    /**
     * Non-valid handle characters: / \ : * ? " < > | and carriage return.<br>
     * (That's slash, backslash, colon, asterisk, question mark, quotation mark,
     * less than symbol, greater than symbol, pipe, and carriage return.)
     */
    public final static String NON_VALID_CHARS = "/\\:*?\"<>|\n";

    /**
     * Create a valid label out of an arbitrary string.
     * @param labelhint  the given label string that we will examine
     *                   and convert any illegal character if necessary
     * @return a valid label string
     */
    public static String createValidLabel(String labelhint) {
		return null;
	}

    /**
     * Checks if the label is not empty and contains all valid label chars.
     * @param labelhint the labelhint to check
     * @throws IllegalArgumentException if the label does not contain all valid
     *         chars.
     */
    public static void validateLabel(String labelhint) {
	}


    //---------- internal ------------------------------------------------------

    /**
     * Convert a Date formatting string in POSIX strftime() format to the
     * pattern format used by the Java SimpleDateFormat class.
     *
     * These are the symbols used in SimpleDateFormat to which we convert
     * our strftime() symbols.
     *
     * <table>
     * <tr><th>Symbol<th>Meaning<th>Presentation<th>Example</tr>
     * <tr><td>G<td>era designator      <td>(Text)     <td>AD</tr>
     * <tr><td>y<td>year                <td>(Number)   <td>1996</tr>
     * <tr><td>M<td>month in year       <td>(Text & Number)<td>July & 07</tr>
     * <tr><td>d<td>day in month        <td>(Number)   <td>10</tr>
     * <tr><td>h<td>hour in am/pm (1~12)<td>(Number)   <td>12</tr>
     * <tr><td>H<td>hour in day (0~23)  <td>(Number)   <td>0</tr>
     * <tr><td>m<td>minute in hour      <td>(Number)   <td>30</tr>
     * <tr><td>s<td>second in minute    <td>(Number)   <td>55</tr>
     * <tr><td>S<td>millisecond         <td>(Number)   <td>978</tr>
     * <tr><td>E<td>day in week         <td>(Text)     <td>Tuesday</tr>
     * <tr><td>D<td>day in year         <td>(Number)   <td>189</tr>
     * <tr><td>F<td>day of week in month<td>(Number)   <td>2 (2nd Wed in July)</tr>
     * <tr><td>w<td>week in year        <td>(Number)   <td>27</tr>
     * <tr><td>W<td>week in month       <td>(Number)   <td>2</tr>
     * <tr><td>a<td>am/pm marker        <td>(Text)     <td>PM</tr>
     * <tr><td>k<td>hour in day (1~24)  <td>(Number)   <td>24</tr>
     * <tr><td>K<td>hour in am/pm (0~11)<td>(Number)   <td>0</tr>
     * <tr><td>z<td>time zone           <td>(Text)     <td>Pacific Standard Time</tr>
     * <tr><td>'<td>escape for text     <td>(Delimiter)<td>&nbsp;</tr>
     * <tr><td>''<td>single quote       <td>(Literal)  <td>'</tr>
     * </table>
     *
     * @param posixFormat The formatting pattern in POSIX strftime() format.
     *
     * @return The Date formatting pattern in SimpleDateFormat pattern format.
     *
     * todo: Check for the more or less complete support of all pattern tags.
     */
    private static String convertFormat(String posixFormat) {
		return null;
	}

    /**
     * Implements the integral number formatting part of the
     * <code>sprintf()</code> method, that is, this method handles d, i, o, u,
     * x, and X formatting characters.
     *
     * @param buf The formatted number is appended to this string buffer
     * @param num The number object to format
     * @param fmt The format character defining the radix of the number
     * @param width The minimum field width for the number
     * @param precision The minimum number of digits to print for the number,
     * 		this does not include any signs or prefix characters
     * @param flags The flags governing the formatting. This is a combination
     * 		of the FLAG_* constants above.
     *
     * @return The formatted string
     *
     * @see <a href="sprintf.html"><code>sprintf()</code></a>
     */
    private static StringBuffer format(StringBuffer buf, Number num,
                                       char fmt, int width, int precision, BitSet flags) {
										return null;
									}

    /**
     * Implements the floating point number formatting part of the
     * <code>sprintf()</code> method, that is, this method handles f, e, E, g,
     * and G formatting characters.
     *
     * @param buf The formatted number is appended to this string buffer
     * @param num The numeric value to format
     * @param fmt The format character defining the floating point format
     * @param width The minimum field width for the number
     * @param precision Depending on <code>fmt</code> either the number of
     * 		digits after the decimal point or the number of significant
     * 		digits.
     * @param flags The flags governing the formatting. This is a combination
     * 		of the FLAG_* constants above.
     *
     * @return The formatted string
     *
     * @see <a href="sprintf.html"><code>sprintf()</code></a>
     */
    private static StringBuffer format(StringBuffer buf, double num,
                                       char fmt, int width, int precision, BitSet flags) {
										return null;
									}

    /**
     * Formats the <code>String</code> appending to the
     * <code>StringBuffer</code> at least the <code>String</code> and
     * justifying according to the flags.
     * <p>
     * The flags will be interpreted as follows :<br>
     * * if {@link #FLAG_LJ} is set, append blanks for left justification<br>
     * * else if {@link #FLAG_ZE} is set, insert zeroes between prefStr and str
     * * else prepend blanks for right justification
     *
     * @param buf The <code>StringBuffer</code> to append the formatted result
     * 		to.
     * @param str The <code>String</code> to be appended with surrounding
     * 		blanks, zeroes and <code>prefStr</code> depending on the
     * 		<code>flags</code>. This is usually the real string to print
     * 		like "ape" or "4.5E99".
     * @param prefStr An optional prefix <code>String</code> to be appended
     * 		in front of the <code>String</code>. This is usually the prefix
     * 		string for numeric <code>str</code> values, for example
     * 		"-", "0x", or "+". The reason for this separation is that the
     * 		{@link #FLAG_ZE} flag will insert zeroes between the prefix and
     * 		the string itself to fill the field to the width.
     * @param width The minimal field width. If the field width is larger than
     * 		the sum of the lengths of <code>str</code> and
     * 		<code>prefStr</code>, blanks or zeroes will be prepended or
     * 		appended according to the flags value.
     * @param flags The flags indicating where blanks or zeroes will be filled.
     * 		See above for the interpretation of flags.
     *
     * @throws NullPointerException if any of <code>buf</code>,
     * 		<code>str</code>, <code>prefStr</code> or <code>flags</code>
     * 		is <code>null</code>.
     */
    private static StringBuffer format(StringBuffer buf, String str,
                                       String prefStr, int width, BitSet flags) {
										return null;
									}

    /**
     * Join two paths or handles, fixing the slashes.
     * All these will go to "/a/b":
     *     /a/ + /b/   (trailing slashes are removed)
     * 	   /a  + /b
     *     /a/ + b
     *     /a/ + b/
     *     /a// + /////b  (i am paranoid)
     *
     * Note that leading slashes are untouched, so variations
     * on the above without the leading slash on 'a' would go
     * to "a/b".
     * @deprecated use {@link #makeCanonicalPath(String)}
     */
    public static String joinFixSlash(String first, String second) {
		return null;
	}

    /**
     * Performs variable replacement on the given string value.
     * Each <code>${...}</code> sequence within the given value is replaced
     * with the value of the named parser variable. If a variable is not found
     * in the properties an IllegalArgumentException is thrown unless
     * <code>ignoreMissing</code> is <code>true</code>. In the later case, the
     * missing variable is replaced by the empty string.
     *
     * @param value         the original value
     * @param ignoreMissing if <code>true</code>, missing variables are replaced
     *                      by the empty string.
     * @return value after variable replacements
     * @throws IllegalArgumentException if the replacement of a referenced
     *                                  variable is not found
     */
    public static String replaceVariables(Properties variables, String value,
                                          boolean ignoreMissing)
            throws IllegalArgumentException {
				return null;
			}

    /**
     * Parses an ISO8601-compliant date/time string.
     * (see <a href="http://www.w3.org/TR/NOTE-datetime">ISO 8601</a>).
     * <p/>
     * The currently supported format is:
     * <pre>
     *   &plusmn;YYYY-MM-DDThh:mm:ss.SSSTZD
     * </pre>
     * where:
     * <pre>
     *   &plusmn;YYYY = four-digit year with optional sign where values <= 0 are
     *           denoting years BCE and values > 0 are denoting years CE,
     *           e.g. -0001 denotes the year 2 BCE, 0000 denotes the year 1 BCE,
     *           0001 denotes the year 1 CE, and so on...
     *   MM    = two-digit month (01=January, etc.)
     *   DD    = two-digit day of month (01 through 31)
     *   hh    = two digits of hour (00 through 23) (am/pm NOT allowed)
     *   mm    = two digits of minute (00 through 59)
     *   ss    = two digits of second (00 through 59)
     *   SSS   = three digits of milliseconds (000 through 999)
     *   TZD   = time zone designator, Z for Zulu (i.e. UTC) or an offset from UTC
     *           in the form of +hh:mm or -hh:mm
     * </pre>
     *
     * @param text the date/time string to be parsed
     * @return a <code>Calendar</code>, or <code>null</code> if the input could
     *         not be parsed
     * @throws IllegalArgumentException if a <code>null</code> argument is passed
     */
    public static Calendar parseISO8601(String text) {
		return null;
	}

    /**
     * Formats a <code>Calendar</code> value into an ISO8601-compliant
     * date/time string.
     *
     * @param cal the time value to be formatted into a date/time string.
     * @return the formatted date/time string.
     * @throws IllegalArgumentException if a <code>null</code> argument is passed
     */
    public static String formatISO8601(Calendar cal) {
		return null;
	}

    private static void pad0(StringBuffer buf, int len, int nr) {
	}
}
