/*
 * $URL: http://svn.day.com/repos/wiki/trunk/wiki-parser/src/main/java/com/day/wiki/parser/util/diff/Diff.java $
 * $Id: Diff.java 29931 2007-08-07 13:17:24Z tripod $
 *
 * Copyright 1997-2006 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.util.diff;

import java.util.HashMap;

/**
 * A class to compare vectors of objects.  The result of comparison
 * is a list of <code>change</code> objects which form an
 * edit script.  The objects compared are traditionally lines
 * of text from two files.  Comparison options such as "ignore
 * whitespace" are implemented by modifying the <code>equals</code>
 * and <code>hashcode</code> methods for the objects compared.
 * <p/>
 * The basic algorithm is described in: </br>
 * "An O(ND) Difference Algorithm and its Variations", Eugene Myers,
 * Algorithmica Vol. 1 No. 2, 1986, p 251.
 * <p/>
 * This class outputs different results from GNU diff 1.15 on some
 * inputs.  Our results are actually better (smaller change list, smaller
 * total size of changes), but it would be nice to know why.  Perhaps
 * there is a memory overwrite bug in GNU diff 1.15.
 *
 * @author Stuart D. Gathman, translated from GNU diff 1.15
 *         Copyright (C) 2000  Business Management Systems, Inc.
 *         <p/>
 *         This program is free software; you can redistribute it and/or modify
 *         it under the terms of the GNU General Public License as published by
 *         the Free Software Foundation; either version 1, or (at your option)
 *         any later version.
 *         <p/>
 *         This program is distributed in the hope that it will be useful,
 *         but WITHOUT ANY WARRANTY; without even the implied warranty of
 *         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *         GNU General Public License for more details.
 *         <p/>
 *         You should have received a copy of the <a href=COPYING.txt>
 *         GNU General Public License</a>
 *         along with this program; if not, write to the Free Software
 *         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * @version $Rev: 29931 $, $Date: 2007-08-07 15:17:24 +0200 (Tue, 07 Aug 2007) $
 */
public class Diff {

    /**
     * The CVS/SVN id
     */
    static final String CVS_ID = "$URL: http://svn.day.com/repos/wiki/trunk/wiki-parser/src/main/java/com/day/wiki/parser/util/diff/Diff.java $ $Rev: 29931 $ $Date: 2007-08-07 15:17:24 +0200 (Tue, 07 Aug 2007) $";

    /**
     * Prepare to find differences between two arrays.  Each element of
     * the arrays is translated to an "equivalence number" based on
     * the result of <code>equals</code>.  The original Object arrays
     * are no longer needed for computing the differences.  They will
     * be needed again later to print the results of the comparison as
     * an edit script, if desired.
     */
    public Diff(Object[] a, Object[] b) {
        HashMap h = new HashMap(a.length + b.length);
        filevec[0] = new FileData(a, h);
        filevec[1] = new FileData(b, h);
    }

    /**
     * 1 more than the maximum equivalence value used for this or its
     * sibling file.
     */
    private int equiv_max = 1;

    /**
     * When set to true, the comparison uses a heuristic to speed it up.
     * With this heuristic, for files with a constant small density
     * of changes, the algorithm is linear in the file size.
     */
    public boolean heuristic = false;

    /**
     * When set to true, the algorithm returns a guarranteed minimal
     * set of changes.  This makes things slower, sometimes much slower.
     */
    public boolean no_discards = false;

    private int[] xvec, yvec;    /* Vectors being compared. */
    private int[] fdiag;         /* Vector, indexed by diagonal, containing
				   the X coordinate of the point furthest
				   along the given diagonal in the forward
				   search of the edit matrix. */
    private int[] bdiag;        /* Vector, indexed by diagonal, containing
				   the X coordinate of the point furthest
				   along the given diagonal in the backward
				   search of the edit matrix. */
    private int fdiagoff, bdiagoff;
    private final FileData[] filevec = new FileData[2];
    private int cost;

    /**
     * Find the midpoint of the shortest edit script for a specified
     * portion of the two files.
     * <p/>
     * We scan from the beginnings of the files, and simultaneously from the ends,
     * doing a breadth-first search through the space of edit-sequence.
     * When the two searches meet, we have found the midpoint of the shortest
     * edit sequence.
     * <p/>
     * The value returned is the number of the diagonal on which the midpoint lies.
     * The diagonal number equals the number of inserted lines minus the number
     * of deleted lines (counting only lines before the midpoint).
     * The edit cost is stored into COST; this is the total number of
     * lines inserted or deleted (counting only lines before the midpoint).
     * <p/>
     * This function assumes that the first lines of the specified portions
     * of the two files do not match, and likewise that the last lines do not
     * match.  The caller must trim matching lines from the beginning and end
     * of the portions it is going to specify.
     * <p/>
     * Note that if we return the "wrong" diagonal value, or if
     * the value of bdiag at that diagonal is "wrong",
     * the worst this can do is cause suboptimal diff output.
     * It cannot cause incorrect diff output.
     */

    private int diag(int xoff, int xlim, int yoff, int ylim) {
		return 0;
	}

    /**
     * Compare in detail contiguous subsequences of the two files
     * which are known, as a whole, to match each other.
     * <p/>
     * The results are recorded in the vectors filevec[N].changed_flag, by
     * storing a 1 in the element for each line that is an insertion or deletion.
     * <p/>
     * The subsequence of file 0 is [XOFF, XLIM) and likewise for file 1.
     * <p/>
     * Note that XLIM, YLIM are exclusive bounds.
     * All line numbers are origin-0 and discarded lines are not counted.
     */
    private void compareseq(int xoff, int xlim, int yoff, int ylim) {
	}

    /**
     * Discard lines from one file that have no matches in the other file.
     */
    private void discard_confusing_lines() {
	}

    private boolean inhibit = false;

    /**
     * Adjust inserts/deletes of blank lines to join changes
     * as much as possible.
     */
    private void shift_boundaries() {
	}

    /**
     * Scan the tables of which lines are inserted and deleted,
     * producing an edit script in reverse order.
     */
    private Change build_reverse_script() {
		return null;
	}

    /**
     * Scan the tables of which lines are inserted and deleted,
     * producing an edit script in forward order.
     */
    private Change build_script() {
		return null;
	}

    /**
     * Report the differences of two files.  DEPTH is the current directory depth.
     */
    public Change diff_2(final boolean reverse) {
		return null;
	}

    /**
     * The result of comparison is an "edit script": a chain of change objects.
     * Each change represents one place where some lines are deleted
     * and some are inserted.
     * <p/>
     * LINE0 and LINE1 are the first affected lines in the two files (origin 0).
     * DELETED is the number of lines deleted here from file 0.
     * INSERTED is the number of lines inserted here in file 1.
     * <p/>
     * If DELETED is 0 then LINE0 is the number of the line before
     * which the insertion was done; vice versa for INSERTED and LINE1.
     */
    public static class Change {
        /**
         * Previous or next edit command.
         */
        public Change nextChange;
        /**
         * # lines of file 1 changed here.
         */
        public final int inserted;
        /**
         * # lines of file 0 changed here.
         */
        public final int deleted;
        /**
         * Line number of 1st deleted line.
         */
        public final int line0;
        /**
         * Line number of 1st inserted line.
         */
        public final int line1;

        /**
         * Cons an additional entry onto the front of an edit script OLD.
         * LINE0 and LINE1 are the first affected lines in the two files (origin 0).
         * DELETED is the number of lines deleted here from file 0.
         * INSERTED is the number of lines inserted here in file 1.
         * <p/>
         * If DELETED is 0 then LINE0 is the number of the line before
         * which the insertion was done; vice versa for INSERTED and LINE1.
         */
        Change(int line0, int line1, int deleted, int inserted, Change old) {
            this.line0 = line0;
            this.line1 = line1;
            this.inserted = inserted;
            this.deleted = deleted;
            this.nextChange = old;
        }
    }

    /**
     * Data on one input file being compared.
     */
    class FileData {

        /**
         * Allocate changed array for the results of comparison.
         */
        void clear() {
		}

        /**
         * Return equiv_count[I] as the number of lines in this file
         * that fall in equivalence class I.
         *
         * @return the array of equivalence class counts.
         */
        int[] equivCount() {
			return null;
		}

        /**
         * Discard lines that have no matches in another file.
         * <p/>
         * A line which is discarded will not be considered by the actual
         * comparison algorithm; it will be as if that line were not in the file.
         * The file's `realindexes' table maps virtual line numbers
         * (which don't count the discarded lines) into real line numbers;
         * this is how the actual comparison algorithm produces results
         * that are comprehensible when the discarded lines are counted.
         * <p/>
         * When we discard a line, we also mark it as a deletion or insertion
         * so that it will be printed in the output.
         *
         * @param f the other file
         */
        void discard_confusing_lines(FileData f) {
		}

        /**
         * Mark to be discarded each line that matches no line of another file.
         * If a line matches many lines, mark it as provisionally discardable.
         *
         * @param counts The count of each equivalence number for the other file.
         * @return 0=nondiscardable, 1=discardable or 2=provisionally discardable
         *         for each line
         */

        private byte[] discardable(final int[] counts) {
			return null;
		}

        /**
         * Don't really discard the provisional lines except when they occur
         * in a run of discardables, with nonprovisionals at the beginning
         * and end.
         */

        private void filterDiscards(final byte[] discards) {
		}

        /**
         * Actually discard the lines.
         *
         * @param discards flags lines to be discarded
         */
        private void discard(final byte[] discards) {
		}

        FileData(Object[] data, HashMap h) {
            buffered_lines = data.length;

            equivs = new int[buffered_lines];
            undiscarded = new int[buffered_lines];
            realindexes = new int[buffered_lines];

            for (int i = 0; i < data.length; ++i) {
                Integer ir = (Integer) h.get(data[i]);
                if (ir == null)
                    h.put(data[i], new Integer(equivs[i] = equiv_max++));
                else
                    equivs[i] = ir.intValue();
            }
        }

        /**
         * Adjust inserts/deletes of blank lines to join changes
         * as much as possible.
         * <p/>
         * We do something when a run of changed lines include a blank
         * line at one end and have an excluded blank line at the other.
         * We are free to choose which blank line is included.
         * `compareseq' always chooses the one at the beginning,
         * but usually it is cleaner to consider the following blank line
         * to be the "change".  The only exception is if the preceding blank line
         * would join this change to other changes.
         *
         * @param f the file being compared against
         */

        void shift_boundaries(FileData f) {
		}

        /**
         * Number of elements (lines) in this file.
         */
        final int buffered_lines;

        /**
         * Vector, indexed by line number, containing an equivalence code for
         * each line.  It is this vector that is actually compared with that
         * of another file to generate differences.
         */
        private final int[] equivs;

        /**
         * Vector, like the previous one except that
         * the elements for discarded lines have been squeezed out.
         */
        final int[] undiscarded;

        /**
         * Vector mapping virtual line numbers (not counting discarded lines)
         * to real ones (counting those lines).  Both are origin-0.
         */
        final int[] realindexes;

        /**
         * Total number of nondiscarded lines.
         */
        int nondiscarded_lines;

        /**
         * Array, indexed by real origin-1 line number,
         * containing true for a line that is an insertion or a deletion.
         * The results of comparison are stored here.
         */
        boolean[] changed_flag;

    }
}
