/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.oak.kernel;

import static com.google.common.collect.Lists.newLinkedList;
import static com.google.common.collect.Maps.newConcurrentMap;
import static com.google.common.collect.Maps.newLinkedHashMap;
import static org.apache.jackrabbit.oak.commons.PathUtils.getName;
import static org.apache.jackrabbit.oak.commons.PathUtils.getParentPath;
import static org.apache.jackrabbit.oak.plugins.memory.PropertyStates.createProperty;

import java.io.IOException;
import java.io.InputStream;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.zip.CheckedInputStream;
import java.util.zip.Checksum;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import javax.jcr.PropertyType;

import com.google.common.collect.Lists;
import com.google.common.hash.HashCode;
import com.google.common.hash.Hasher;
import com.google.common.hash.Hashing;
import com.google.common.io.ByteStreams;
import org.apache.jackrabbit.mk.api.MicroKernel;
import org.apache.jackrabbit.mk.api.MicroKernelException;
import org.apache.jackrabbit.oak.api.Blob;
import org.apache.jackrabbit.oak.api.CommitFailedException;
import org.apache.jackrabbit.oak.api.PropertyState;
import org.apache.jackrabbit.oak.api.Type;
import org.apache.jackrabbit.oak.cache.StringCache;
import org.apache.jackrabbit.oak.commons.PathUtils;
import org.apache.jackrabbit.oak.commons.json.JsopBuilder;
import org.apache.jackrabbit.oak.commons.json.JsopReader;
import org.apache.jackrabbit.oak.commons.json.JsopTokenizer;
import org.apache.jackrabbit.oak.json.BlobSerializer;
import org.apache.jackrabbit.oak.json.JsonSerializer;
import org.apache.jackrabbit.oak.json.JsopDiff;
import org.apache.jackrabbit.oak.json.TypeCodes;
import org.apache.jackrabbit.oak.plugins.memory.AbstractBlob;
import org.apache.jackrabbit.oak.plugins.value.Conversions;
import org.apache.jackrabbit.oak.spi.commit.CommitHook;
import org.apache.jackrabbit.oak.spi.commit.CommitInfo;
import org.apache.jackrabbit.oak.spi.commit.DefaultValidator;
import org.apache.jackrabbit.oak.spi.commit.EditorHook;
import org.apache.jackrabbit.oak.spi.commit.Validator;
import org.apache.jackrabbit.oak.spi.commit.ValidatorProvider;
import org.apache.jackrabbit.oak.spi.state.NodeBuilder;
import org.apache.jackrabbit.oak.spi.state.NodeState;
import org.apache.jackrabbit.oak.spi.state.NodeStore;

/**
 * This is a simple {@link NodeStore}-based {@link MicroKernel} implementation.
 */
public class NodeStoreKernel implements MicroKernel {

    private static final CommitHook CONFLICT_HOOK =
            new EditorHook(new ValidatorProvider() {
                @Override
                protected Validator getRootValidator(
                        NodeState before, NodeState after, CommitInfo info) {
                    return new DefaultValidator() {
                        @Override
                        public Validator childNodeAdded(
                                String name, NodeState after)
                                throws CommitFailedException {
                            if (name.equals(MicroKernel.CONFLICT)) {
                                throw new CommitFailedException(
                                        CommitFailedException.STATE, 0, "Conflict");
                            }
                            return null;
                        }
                        @Override
                        public Validator childNodeChanged(
                                String name, NodeState before, NodeState after)
                                throws CommitFailedException {
                            return this;
                        }
                    };
                }
            });

    private final NodeStore store;

    private final Map<String, Revision> revisions = newLinkedHashMap();

    private final Map<String, Blob> blobs = newConcurrentMap();

    private final BlobSerializer blobSerializer = new BlobSerializer() {
        @Override
        public String serialize(Blob blob) {
			return null;
		}
    };

    private Revision head;

    public NodeStoreKernel(NodeStore store) {
        this.store = store;
        this.head = new Revision(store.getRoot());
        revisions.put(head.id, head);
    }

    @Nonnull
    private synchronized Revision getRevision(@CheckForNull String id) {
		return null;
	}

    private NodeState getRoot(String id) throws MicroKernelException {
		return null;
	}

    private NodeState getNode(String revision, String path)
            throws MicroKernelException {
				return null;
			}

    private void applyJsop(NodeBuilder builder, String path, String jsonDiff)
            throws MicroKernelException {
			}

    private void applyJsop(NodeBuilder builder, String jsonDiff) {
	}

    private NodeBuilder getNode(NodeBuilder builder, String path) {
		return null;
	}

    private void addNode(NodeBuilder builder, JsopTokenizer tokenizer)
            throws MicroKernelException {
			}

    /**
     * Read a multi valued {@code PropertyState} from a {@link JsopReader}
     * @param name  The name of the property state
     * @param reader  The reader
     * @return new property state
     */
    private static PropertyState readArrayProperty(String name, JsopReader reader) {
		return null;
	}

    @Override
    public synchronized String getHeadRevision() {
		return null;
	}

    @Override
    public String checkpoint(long lifetime) {
		return null;
	}

    @Override
    public String getRevisionHistory(
            long since, int maxEntries, String path)
            throws MicroKernelException {
				return null;
			}

    @Override
    public synchronized String waitForCommit(
            String oldHeadRevisionId, long timeout)
            throws MicroKernelException, InterruptedException {
				return null;
			}

    @Override
    public String getJournal(
            String fromRevisionId, String toRevisionId, String path)
            throws MicroKernelException {
				return null;
			}

    @Override
    public String diff(
            String fromRevisionId, String toRevisionId, String path, int depth)
            throws MicroKernelException {
				return null;
			}

    @Override
    public boolean nodeExists(String path, String revisionId)
            throws MicroKernelException {
				return false;
			}

    @Override
    public long getChildNodeCount(String path, String revisionId)
            throws MicroKernelException {
				return 0L;
			}

    @Override
    public String getNodes(
            String path, String revisionId, int depth,
            long offset, int maxChildNodes, String filter)
            throws MicroKernelException {
				return null;
			}

    @Override
    public synchronized String commit(
            String path, String jsonDiff, String revisionId, String message)
            throws MicroKernelException {
				return null;
			}

    @Override
    public synchronized String branch(String trunkRevisionId)
            throws MicroKernelException {
				return null;
			}

    @Override
    public synchronized String merge(String branchRevisionId, String message)
            throws MicroKernelException {
				return null;
			}

    @Override
    public String rebase(String branchRevisionId, String newBaseRevisionId)
            throws MicroKernelException {
				return null;
			}

    @Nonnull
    @Override
    public synchronized String reset(@Nonnull String branchRevisionId,
                                     @Nonnull String ancestorRevisionId)
            throws MicroKernelException {
				return null;
			}

    @Override
    public long getLength(String blobId) throws MicroKernelException {
		return 0L;
	}

    @Override
    public int read(String blobId, long pos, byte[] buff, int off, int length)
            throws MicroKernelException {
				return 0;
			}

    @Override
    public String write(InputStream in) throws MicroKernelException {
		return null;
	}

    private static class Revision {

        private final Revision base;

        private final NodeBuilder branch;

        private final String id = UUID.randomUUID().toString();

        private final NodeState root;

        private final String message;

        private final long timestamp;

        Revision(NodeState root) {
            this.base = null;
            this.branch = null;
            this.root = root;
            this.message = "start";
            this.timestamp = 0;
        }

        Revision(Revision base) {
            this.base = base;
            this.branch = base.root.builder();
            this.root = base.root;
            this.message = "branch";
            this.timestamp = System.currentTimeMillis();
        }

        public Revision(Revision base, NodeState root, String message) {
            this.base = base;
            this.branch = base.branch;
            this.root = root;
            this.message = message;
            this.timestamp = System.currentTimeMillis();
        }

        boolean hasPathChanged(String path) {
			return false;
		}

        String getPathChanges(String path, BlobSerializer blobs) {
			return null;
		}

    }

    private static NodeState getNode(NodeState node, String path) {
		return null;
	}

}
