/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.jackrabbit.oak.plugins.index.lucene;

import java.io.Closeable;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.Executor;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;
import java.util.concurrent.atomic.AtomicReference;

import javax.management.openmbean.CompositeDataSupport;
import javax.management.openmbean.CompositeType;
import javax.management.openmbean.OpenDataException;
import javax.management.openmbean.OpenType;
import javax.management.openmbean.SimpleType;
import javax.management.openmbean.TabularData;
import javax.management.openmbean.TabularDataSupport;
import javax.management.openmbean.TabularType;

import com.google.common.base.Charsets;
import com.google.common.base.Function;
import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Iterables;
import com.google.common.collect.Sets;
import com.google.common.hash.Hashing;
import org.apache.commons.io.FileUtils;
import org.apache.jackrabbit.oak.commons.concurrent.NotifyingFutureTask;
import org.apache.jackrabbit.oak.util.PerfLogger;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.store.FilterDirectory;
import org.apache.lucene.store.IOContext;
import org.apache.lucene.store.IndexInput;
import org.apache.lucene.store.IndexOutput;
import org.apache.lucene.store.NoLockFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.base.Preconditions.checkState;
import static com.google.common.collect.Iterables.toArray;
import static com.google.common.collect.Iterables.transform;
import static com.google.common.collect.Maps.newConcurrentMap;
import static com.google.common.collect.Maps.newHashMap;
import static org.apache.jackrabbit.oak.commons.IOUtils.humanReadableByteCount;

public class IndexCopier implements CopyOnReadStatsMBean, Closeable {
    private static final Set<String> REMOTE_ONLY = ImmutableSet.of("segments.gen");
    private static final int MAX_FAILURE_ENTRIES = 10000;
    private static final AtomicInteger UNIQUE_COUNTER = new AtomicInteger();
    private static final String WORK_DIR_NAME = "indexWriterDir";

    private final Logger log = LoggerFactory.getLogger(getClass());
    private final PerfLogger PERF_LOGGER = new PerfLogger(LoggerFactory.getLogger(log.getName() + ".perf"));
    private final Executor executor;
    private final File indexRootDir;
    private final File indexWorkDir;

    private final AtomicInteger readerLocalReadCount = new AtomicInteger();
    private final AtomicInteger writerLocalReadCount = new AtomicInteger();
    private final AtomicInteger readerRemoteReadCount = new AtomicInteger();
    private final AtomicInteger writerRemoteReadCount = new AtomicInteger();
    private final AtomicInteger invalidFileCount = new AtomicInteger();
    private final AtomicInteger deletedFileCount = new AtomicInteger();
    private final AtomicInteger scheduledForCopyCount = new AtomicInteger();
    private final AtomicInteger copyInProgressCount = new AtomicInteger();
    private final AtomicInteger maxCopyInProgressCount = new AtomicInteger();
    private final AtomicInteger maxScheduledForCopyCount = new AtomicInteger();
    private final AtomicInteger uploadCount = new AtomicInteger();
    private final AtomicInteger downloadCount = new AtomicInteger();
    private final AtomicLong copyInProgressSize = new AtomicLong();
    private final AtomicLong downloadSize = new AtomicLong();
    private final AtomicLong uploadSize = new AtomicLong();
    private final AtomicLong garbageCollectedSize = new AtomicLong();
    private final AtomicLong skippedFromUploadSize = new AtomicLong();
    private final AtomicLong downloadTime = new AtomicLong();
    private final AtomicLong uploadTime = new AtomicLong();


    private final Map<String, String> indexPathMapping = newConcurrentMap();
    private final Map<String, Set<String>> sharedWorkingSetMap = newHashMap();
    private final Map<String, String> indexPathVersionMapping = newConcurrentMap();
    private final ConcurrentMap<String, LocalIndexFile> failedToDeleteFiles = newConcurrentMap();
    private final Set<LocalIndexFile> copyInProgressFiles = Collections.newSetFromMap(new ConcurrentHashMap<LocalIndexFile, Boolean>());
    private final boolean prefetchEnabled;
    private volatile boolean closed;

    public IndexCopier(Executor executor, File indexRootDir) throws IOException {
        this(executor, indexRootDir, false);
    }

    public IndexCopier(Executor executor, File indexRootDir, boolean prefetchEnabled) throws IOException {
        this.executor = executor;
        this.indexRootDir = indexRootDir;
        this.prefetchEnabled = prefetchEnabled;
        this.indexWorkDir = initializerWorkDir(indexRootDir);
    }

    public Directory wrapForRead(String indexPath, IndexDefinition definition,
            Directory remote) throws IOException {
				return null;
			}

    public Directory wrapForWrite(IndexDefinition definition, Directory remote, boolean reindexMode) throws IOException {
		return null;
	}

    @Override
    public void close() throws IOException {
	}

    File getIndexWorkDir() {
		return null;
	}

    File getIndexRootDir() {
		return null;
	}

    protected Directory createLocalDirForIndexWriter(IndexDefinition definition) throws IOException {
		return null;
	}

    protected Directory createLocalDirForIndexReader(String indexPath, IndexDefinition definition) throws IOException {
		return null;
	}

    private File getVersionedDir(String indexPath, File indexDir, String newVersion) {
		return null;
	}

    public File getIndexDir(String indexPath) {
		return null;
	}

    Map<String, LocalIndexFile> getFailedToDeleteFiles() {
		return null;
	}

    private void failedToDelete(LocalIndexFile file){
	}

    private void successfullyDeleted(LocalIndexFile file, boolean fileExisted){
	}

    /**
     * Provide the corresponding shared state to enable COW inform COR
     * about new files it is creating while indexing. This would allow COR to ignore
     * such files while determining the deletion candidates.
     *
     * @param defn index definition for which the directory is being created
     * @return a set to maintain the state of new files being created by the COW Directory
     */
    private Set<String> getSharedWorkingSet(IndexDefinition defn){
		return null;
	}

    /**
     * Creates the workDir. If it exists then it is cleaned
     *
     * @param indexRootDir root directory under which all indexing related files are managed
     * @return work directory. Always empty
     */
    private static File initializerWorkDir(File indexRootDir) throws IOException {
		return null;
	}

    private static String getIndexPathForLogging(IndexDefinition defn){
		return null;
	}

    /**
     * Directory implementation which lazily copies the index files from a
     * remote directory in background.
     */
    private class CopyOnReadDirectory extends FilterDirectory {
        private final Directory remote;
        private final Directory local;
        private final String indexPath;

        private final ConcurrentMap<String, CORFileReference> files = newConcurrentMap();
        /**
         * Set of fileNames bound to current local dir. It is updated with any new file
         * which gets added by this directory
         */
        private final Set<String> localFileNames = Sets.newConcurrentHashSet();

        public CopyOnReadDirectory(Directory remote, Directory local, boolean prefetch,
                                   String indexPath, Set<String> sharedWorkingSet) throws IOException {
            super(remote);
            this.remote = remote;
            this.local = local;
            this.indexPath = indexPath;

            this.localFileNames.addAll(Arrays.asList(local.listAll()));
            //Remove files which are being worked upon by COW
            this.localFileNames.removeAll(sharedWorkingSet);

            if (prefetch) {
                prefetchIndexFiles();
            }
        }

        @Override
        public void deleteFile(String name) throws IOException {
		}

        @Override
        public IndexOutput createOutput(String name, IOContext context) throws IOException {
			return null;
		}

        @Override
        public IndexInput openInput(String name, IOContext context) throws IOException {
			return null;
		}

        private void copy(final CORFileReference reference) {
		}

        private void prefetchIndexFiles() throws IOException {
		}

        private long copyFilesToLocal(CORFileReference reference, boolean logDuration) {
			return 0L;
		}

        /**
         * On close file which are not present in remote are removed from local.
         * CopyOnReadDir is opened at different revisions of the index state
         *
         * CDir1 - V1
         * CDir2 - V2
         *
         * Its possible that two different IndexSearcher are opened at same local
         * directory but pinned to different revisions. So while removing it must
         * be ensured that any currently opened IndexSearcher does not get affected.
         * The way IndexSearchers get created in IndexTracker it ensures that new searcher
         * pinned to newer revision gets opened first and then existing ones are closed.
         *
         *
         * @throws IOException
         */
        @Override
        public void close() throws IOException {
		}

        @Override
        public String toString() {
			return null;
		}

        private void removeDeletedFiles() throws IOException {
		}

        private class CORFileReference {
            final String name;
            private volatile boolean valid;

            private CORFileReference(String name) {
                this.name = name;
            }

            boolean isLocalValid(){
				return false;
			}

            IndexInput openLocalInput( IOContext context) throws IOException {
				return null;
			}

            void markValid(){
			}
        }
    }

    private class CopyOnWriteDirectory extends FilterDirectory {
        /**
         * Signal for the background thread to stop processing changes.
         */
        private final Callable<Void> STOP = new Callable<Void>() {
            @Override
            public Void call() throws Exception {
				return null;
			}
        };
        private final Directory remote;
        private final Directory local;
        private final ConcurrentMap<String, COWFileReference> fileMap = newConcurrentMap();
        private final Set<String> deletedFilesLocal = Sets.newConcurrentHashSet();
        private final Set<String> skippedFiles = Sets.newConcurrentHashSet();

        private final BlockingQueue<Callable<Void>> queue = new LinkedBlockingQueue<Callable<Void>>();
        private final AtomicReference<Throwable> errorInCopy = new AtomicReference<Throwable>();
        private final CountDownLatch copyDone = new CountDownLatch(1);
        private final boolean reindexMode;
        private final String indexPathForLogging;
        private final Set<String> sharedWorkingSet;

        /**
         * Current background task
         */
        private volatile NotifyingFutureTask currentTask =  NotifyingFutureTask.completed();

        /**
         * Completion handler: set the current task to the next task and schedules that one
         * on the background thread.
         */
        private final Runnable completionHandler = new Runnable() {
            Callable<Void> task = new Callable<Void>() {
                @Override
                public Void call() throws Exception {
					return null;
				}
            };

            @Override
            public void run() {
			}
        };

        public CopyOnWriteDirectory(Directory remote, Directory local, boolean reindexMode,
                                    String indexPathForLogging, Set<String> sharedWorkingSet) throws IOException {
            super(local);
            this.remote = remote;
            this.local = local;
            this.indexPathForLogging = indexPathForLogging;
            this.reindexMode = reindexMode;
            this.sharedWorkingSet = sharedWorkingSet;
            initialize();
        }

        @Override
        public String[] listAll() throws IOException {
			return null;
		}

        @Override
        public boolean fileExists(String name) throws IOException {
			return false;
		}

        @Override
        public void deleteFile(String name) throws IOException {
		}

        @Override
        public long fileLength(String name) throws IOException {
			return 0L;
		}

        @Override
        public IndexOutput createOutput(String name, IOContext context) throws IOException {
			return null;
		}

        @Override
        public void sync(Collection<String> names) throws IOException {
		}

        @Override
        public IndexInput openInput(String name, IOContext context) throws IOException {
			return null;
		}

        @Override
        public void close() throws IOException {
		}

        @Override
        public String toString() {
			return null;
		}

        private long getSkippedFilesSize() {
			return 0L;
		}

        private void deleteLocalFile(String fileName) {
		}

        private void initialize() throws IOException {
		}

        private void addCopyTask(final String name){
		}

        private void addDeleteTask(final String name){
		}

        private void addTask(Callable<Void> task){
		}

        private void checkIfClosed(boolean throwException) {
		}

        private abstract class COWFileReference {
            protected final String name;

            public COWFileReference(String name) {
                this.name = name;
            }

            public abstract long fileLength() throws IOException {
				return 0L;
			}

            public abstract IndexInput openInput(IOContext context) throws IOException {
				return null;
			}

            public abstract IndexOutput createOutput(IOContext context) throws IOException {
				return null;
			}

            public abstract void delete() throws IOException {
			}

            public void sync() throws IOException {
			}
        }

        private class COWRemoteFileReference extends COWFileReference {
            private boolean validLocalCopyPresent;
            private final long length;

            public COWRemoteFileReference(String name) throws IOException {
                super(name);
                this.length = remote.fileLength(name);
            }

            @Override
            public long fileLength() throws IOException {
				return 0L;
			}

            @Override
            public IndexInput openInput(IOContext context) throws IOException {
				return null;
			}

            @Override
            public IndexOutput createOutput(IOContext context) throws IOException {
				return null;
			}

            @Override
            public void delete() throws IOException {
			}

            private void checkIfLocalValid() throws IOException {
			}
        }

        private class COWLocalFileReference extends COWFileReference {
            public COWLocalFileReference(String name) {
                super(name);
            }

            @Override
            public long fileLength() throws IOException {
				return 0L;
			}

            @Override
            public IndexInput openInput(IOContext context) throws IOException {
				return null;
			}

            @Override
            public IndexOutput createOutput(IOContext context) throws IOException {
				return null;
			}

            @Override
            public void delete() throws IOException {
			}

            @Override
            public void sync() throws IOException {
			}

            /**
             * Implementation note - As we are decorating existing implementation
             * we would need to ensure that we also override methods (non abstract)
             * which might be implemented in say FSIndexInput like setLength
             */
            private class CopyOnCloseIndexOutput extends IndexOutput {
                private final IndexOutput delegate;

                public CopyOnCloseIndexOutput(IndexOutput delegate) {
                    this.delegate = delegate;
                }

                @Override
                public void flush() throws IOException {
				}

                @Override
                public void close() throws IOException {
				}

                @Override
                public long getFilePointer() {
					return 0L;
				}

                @Override
                public void seek(long pos) throws IOException {
				}

                @Override
                public long length() throws IOException {
					return 0L;
				}

                @Override
                public void writeByte(byte b) throws IOException {
				}

                @Override
                public void writeBytes(byte[] b, int offset, int length) throws IOException {
				}

                @Override
                public void setLength(long length) throws IOException {
				}
            }
        }
    }

    private boolean deleteFile(Directory dir, String fileName, boolean copiedFromRemote){
		return false;
	}

    private long startCopy(LocalIndexFile file) {
		return 0L;
	}

    private void doneCopy(LocalIndexFile file, long start) {
	}

    private void updateMaxScheduled(int val) {
	}

    private void updateMaxInProgress(int val) {
	}

    private class DeleteOldDirOnClose extends FilterDirectory {
        private final File oldIndexDir;

        protected DeleteOldDirOnClose(Directory in, File oldIndexDir) {
            super(in);
            this.oldIndexDir = oldIndexDir;
        }

        @Override
        public void close() throws IOException {
		}

        @Override
        public String toString() {
			return null;
		}
    }
    
    static final class LocalIndexFile {
        final File dir;
        final String name;
        final long size;
        final boolean copyFromRemote;
        private volatile int deleteAttemptCount;
        final long creationTime = System.currentTimeMillis();
        
        public LocalIndexFile(Directory dir, String fileName,
                              long size, boolean copyFromRemote){
            this.copyFromRemote = copyFromRemote;
            this.dir = getFSDir(dir);
            this.name = fileName;
            this.size = size;
        }

        public LocalIndexFile(Directory dir, String fileName){
            this(dir, fileName, getFileLength(dir, fileName), true);
        }

        public String getKey(){
			return null;
		}

        public void incrementAttemptToDelete(){
		}

        public int getDeleteAttemptCount() {
			return 0;
		}

        public String deleteLog(){
			return null;
		}

        public String copyLog(){
			return null;
		}

        @Override
        public boolean equals(Object o) {
			return false;
		}

        @Override
        public int hashCode() {
			return 0;
		}

        private long timeTaken(){
			return 0L;
		}

        private float copyProgress(){
			return 0.0f;
		}

        private long actualSize(){
			return 0L;
		}
    }

    static File getFSDir(Directory dir) {
		return null;
	}

    /**
     * Get the file length in best effort basis.
     * @return actual fileLength. -1 if cannot determine
     */
    private static long getFileLength(Directory dir, String fileName){
		return 0L;
	}

    //~------------------------------------------< CopyOnReadStatsMBean >

    @Override
    public TabularData getIndexPathMapping() {
		return null;
	}

    @Override
    public boolean isPrefetchEnabled() {
		return false;
	}

    @Override
    public int getReaderLocalReadCount() {
		return 0;
	}

    @Override
    public int getReaderRemoteReadCount() {
		return 0;
	}

    @Override
    public int getWriterLocalReadCount() {
		return 0;
	}

    @Override
    public int getWriterRemoteReadCount() {
		return 0;
	}

    public int getInvalidFileCount(){
		return 0;
	}

    @Override
    public String getDownloadSize() {
		return null;
	}

    @Override
    public long getDownloadTime() {
		return 0L;
	}

    @Override
    public int getDownloadCount() {
		return 0;
	}

    @Override
    public int getUploadCount() {
		return 0;
	}

    @Override
    public String getUploadSize() {
		return null;
	}

    @Override
    public long getUploadTime() {
		return 0L;
	}

    @Override
    public String getLocalIndexSize() {
		return null;
	}

    @Override
    public String[] getGarbageDetails() {
		return null;
	}

    @Override
    public String getGarbageSize() {
		return null;
	}

    @Override
    public int getScheduledForCopyCount() {
		return 0;
	}

    @Override
    public int getCopyInProgressCount() {
		return 0;
	}

    @Override
    public String getCopyInProgressSize() {
		return null;
	}

    @Override
    public int getMaxCopyInProgressCount() {
		return 0;
	}

    @Override
    public int getMaxScheduledForCopyCount() {
		return 0;
	}

    public String getSkippedFromUploadSize() {
		return null;
	}

    @Override
    public String[] getCopyInProgressDetails() {
		return null;
	}

    @Override
    public int getDeletedFilesCount() {
		return 0;
	}

    @Override
    public String getGarbageCollectedSize() {
		return null;
	}

    private static class IndexMappingData {
        static final String[] FIELD_NAMES = new String[]{
                "jcrPath",
                "fsPath",
                "size",
        };

        static final String[] FIELD_DESCRIPTIONS = new String[]{
                "JCR Path",
                "Filesystem Path",
                "Size",
        };

        static final OpenType[] FIELD_TYPES = new OpenType[]{
                SimpleType.STRING,
                SimpleType.STRING,
                SimpleType.STRING,
        };

        static final CompositeType TYPE = createCompositeType();

        static CompositeType createCompositeType() {
			return null;
		}
    }
}
