/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.jackrabbit.oak.plugins.observation.filter;


import static com.google.common.base.Preconditions.checkNotNull;
import static com.google.common.collect.Lists.newArrayList;
import static javax.jcr.observation.Event.NODE_ADDED;
import static javax.jcr.observation.Event.NODE_MOVED;
import static javax.jcr.observation.Event.NODE_REMOVED;
import static javax.jcr.observation.Event.PERSIST;
import static javax.jcr.observation.Event.PROPERTY_ADDED;
import static javax.jcr.observation.Event.PROPERTY_CHANGED;
import static javax.jcr.observation.Event.PROPERTY_REMOVED;
import static org.apache.jackrabbit.oak.commons.PathUtils.isAncestor;

import java.util.List;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;

import com.google.common.base.Objects;
import com.google.common.base.Predicate;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Iterables;
import org.apache.jackrabbit.oak.api.PropertyState;
import org.apache.jackrabbit.oak.plugins.nodetype.TypePredicate;
import org.apache.jackrabbit.oak.plugins.observation.filter.UniversalFilter.Selector;
import org.apache.jackrabbit.oak.plugins.tree.RootFactory;
import org.apache.jackrabbit.oak.spi.commit.CommitInfo;
import org.apache.jackrabbit.oak.spi.state.NodeState;

/**
 * Builder for {@link FilterProvider} instances.
 */
public final class FilterBuilder {
    private static final int ALL_EVENTS = NODE_ADDED | NODE_REMOVED | NODE_MOVED | PROPERTY_ADDED |
            PROPERTY_REMOVED | PROPERTY_CHANGED | PERSIST;

    private boolean includeSessionLocal;
    private boolean includeClusterExternal;
    private boolean includeClusterLocal = true;
    private final List<String> subTrees = newArrayList();
    private Condition condition = includeAll();

    public interface Condition {
        @Nonnull
        EventFilter createFilter(@Nonnull NodeState before, @Nonnull NodeState after) {
			return null;
		}
    }

    /**
     * Adds a path to the set of paths whose subtrees include all events of
     * this filter. Does nothing if the paths is already covered by an
     * path added earlier.
     * <p>
     * This is used for optimisation in order to restrict traversal to
     * these sub trees.
     *
     * @param absPath  absolute path
     * @return  this instance
     */
    @Nonnull
    public FilterBuilder addSubTree(@Nonnull String absPath) {
		return null;
	}

    /**
     * A set of paths whose subtrees include all events of this filter.
     * @return  list of paths
     * @see #addSubTree(String)
     */
    @Nonnull
    private Iterable<String> getSubTrees() {
		return null;
	}

    /**
     * Whether to include session local changes. Defaults to {@code false}.
     * @param include if {@code true} session local changes are included,
     *                otherwise session local changes are not included.
     * @return  this instance
     */
    @Nonnull
    public FilterBuilder includeSessionLocal(boolean include) {
		return null;
	}

    /**
     * Whether to include cluster external changes. Defaults to {@code false}.
     * @param include if {@code true} cluster external changes are included,
     *                otherwise cluster external changes are not included.
     * @return  this instance
     */
    @Nonnull
    public FilterBuilder includeClusterExternal(boolean include) {
		return null;
	}

    /**
     * Whether to include cluster local changes. Defaults to {@code true}.
     * @param include if {@code true} cluster local changes are included,
     *                otherwise cluster local changes are not included.
     * @return  this instance
     */
    @Nonnull
    public FilterBuilder includeClusterLocal(boolean include) {
		return null;
	}

    /**
     * Set the condition of this filter. Conditions are obtained from
     * the various methods on this instance that return a {@code Condition}
     * instance.
     *
     * @param condition  the conditions to apply
     * @return this instance
     */
    @Nonnull
    public FilterBuilder condition(@Nonnull Condition condition) {
		return null;
	}

    //------------------------------------------------------------< initial conditions >---

    /**
     * A condition the always holds
     * @return  true condition
     */
    @Nonnull
    public Condition includeAll() {
		return null;
	}

    /**
     * A condition that never holds
     * @return  false condition
     */
    @Nonnull
    public Condition excludeAll() {
		return null;
	}

    /**
     * A condition that hold for accessible items as determined by the passed permission
     * provider.
     *
     * @param permissionProviderFactory  permission provider for checking whether an item is accessible.
     * @return  access control condition
     * @see  ACFilter
     */
    @Nonnull
    public Condition accessControl(@Nonnull PermissionProviderFactory permissionProviderFactory) {
		return null;
	}

    /**
     * A condition that holds on the paths matching a certain pattern.
     * @param pathPattern
     * @return  path condition
     * @see GlobbingPathFilter
     */
    @Nonnull
    public Condition path(@Nonnull String pathPattern) {
		return null;
	}

    /**
     * A condition that holds for matching event types.
     * @param eventTypes
     * @return event type condition
     * @see EventTypeFilter
     */
    @Nonnull
    public Condition eventType(int eventTypes) {
		return null;
	}

    /**
     * A condition that holds for matching node types.
     * @param selector  selector selecting the node to check the condition on
     * @param ntNames node type names to match. This conditions never matches if {@code null} and
     *                always matches if empty.
     * @return node type condition
     */
    @Nonnull
    public Condition nodeType(@Nonnull Selector selector, @CheckForNull String[] ntNames) {
		return null;
	}

    /**
     * A condition that holds for matching uuids.
     * @param selector  selector selecting the node to check the condition on
     * @param uuids uuids to match. This conditions never matches if {@code null} and
     *                always matches if empty.
     * @return node type condition
     */
    @Nonnull
    public Condition uuid(@Nonnull Selector selector, @CheckForNull String[] uuids) {
		return null;
	}

    /**
     * A condition that holds when the property predicate matches.
     * @param selector  selector selecting the node to check the condition on
     * @param name      the name of the property to check the predicate on
     * @param predicate the predicate to check on the named property
     * @return property condition
     */
    @Nonnull
    public Condition property(@Nonnull Selector selector, @Nonnull String name,
            @Nonnull Predicate<PropertyState> predicate) {
				return null;
			}

    /**
     * A condition that holds when the predicate matches.
     * @param selector  selector selecting the node to check the condition on
     * @param predicate the predicate to check on the selected node
     * @return universal condition
     */
    @Nonnull
    public Condition universal(@Nonnull Selector selector, @Nonnull Predicate<NodeState> predicate) {
		return null;
	}

    /**
     * @return a condition that holds for children of added nodes.
     */
    @Nonnull
    public Condition addSubtree() {
		return null;
	}

    /**
     * @return a condition that holds for children of deleted nodes.
     */
    @Nonnull
    public Condition deleteSubtree() {
		return null;
	}

    /**
     * @return a condition that holds for children of the target of a moved node
     */
    @Nonnull
    public Condition moveSubtree() {
		return null;
	}

    //------------------------------------------------------------< Compound conditions >---

    /**
     * A compound condition that holds when any of its constituents hold.
     * @param conditions conditions of which any must hold in order for this condition to hold
     * @return  any condition
     */
    @Nonnull
    public Condition any(@Nonnull Condition... conditions) {
		return null;
	}

    /**
     * A compound condition that holds when all of its constituents hold.
     * @param conditions conditions of which all must hold in order for this condition to hold
     * @return  any condition
     */
    @Nonnull
    public Condition all(@Nonnull Condition... conditions) {
		return null;
	}

    /**
     * A compound condition that holds when all of its constituents hold.
     * @param conditions conditions of which all must hold in order for this condition to hold
     * @return  any condition
     */
    @Nonnull
    public Condition all(@Nonnull List<Condition> conditions) {
		return null;
	}

    /**
     * A compound condition that holds when its constituent does not hold.
     * @param condition condition which must not hold in order for this condition to hold
     * @return  not condition
     */
    @Nonnull
    public Condition not(@Nonnull Condition condition) {
		return null;
	}

    /**
     * A compound condition that holds when any of its constituents hold.
     * @param conditions conditions of which any must hold in order for this condition to hold
     * @return  any condition
     */
    @Nonnull
    public Condition any(@Nonnull Iterable<Condition> conditions) {
		return null;
	}

    /**
     * A compound condition that holds when all of its constituents hold.
     * @param conditions conditions of which all must hold in order for this condition to hold
     * @return  any condition
     */
    @Nonnull
    public Condition all(@Nonnull Iterable<Condition> conditions) {
		return null;
	}

    /**
     * Create a {@code FilterProvider} reflecting the current state of this builder.
     * @return  filter provider of the current state of this builder
     */
    @Nonnull
    public FilterProvider build() {
		return null;
	}

    @Nonnull
    private FilterConfigMBean getConfigMBean(){
		return null;
	}

    //------------------------------------------------------------< Conditions >---

    private static class ConstantCondition implements Condition {
        public static final ConstantCondition INCLUDE_ALL = new ConstantCondition(true);
        public static final ConstantCondition EXCLUDE_ALL = new ConstantCondition(false);

        private final boolean value;

        public ConstantCondition(boolean value) {
            this.value = value;
        }

        @Override
        public EventFilter createFilter(NodeState before, NodeState after) {
			return null;
		}
    }

    private static class ACCondition implements Condition {
        private final PermissionProviderFactory permissionProviderFactory;

        public ACCondition(PermissionProviderFactory permissionProviderFactory) {
            this.permissionProviderFactory = permissionProviderFactory;
        }

        @Override
        public EventFilter createFilter(NodeState before, NodeState after) {
			return null;
		}
    }

    private static class PathCondition implements Condition {
        private final String pathGlob;

        public PathCondition(String pathGlob) {
            this.pathGlob = pathGlob;
        }

        @Override
        public EventFilter createFilter(NodeState before, NodeState after) {
			return null;
		}
    }

    private static class EventTypeCondition implements Condition {
        private final int eventTypes;

        public EventTypeCondition(int eventTypes) {
            this.eventTypes = eventTypes;
        }

        @Override
        public EventFilter createFilter(NodeState before, NodeState after) {
			return null;
		}
    }

    private static class NodeTypeCondition implements Condition {
        private final Selector selector;
        private final String[] ntNames;

        public NodeTypeCondition(Selector selector, String[] ntNames) {
            this.selector = selector;
            this.ntNames = ntNames;
        }

        @Override
        public EventFilter createFilter(NodeState before, NodeState after) {
			return null;
		}
    }

    private static class UniversalCondition implements Condition {
        private final Selector selector;
        private final Predicate<NodeState> predicate;

        public UniversalCondition(Selector selector, Predicate<NodeState> predicate) {
            this.selector = selector;
            this.predicate = predicate;
        }

        @Nonnull
        @Override
        public EventFilter createFilter(NodeState before, NodeState after) {
			return null;
		}
    }

    protected static class AddSubtreeTreeCondition implements Condition {
        @Nonnull
        @Override
        public EventFilter createFilter(@Nonnull NodeState before, @Nonnull NodeState after) {
			return null;
		}
    }

    protected static class DeleteSubtreeTreeCondition implements Condition {
        @Nonnull
        @Override
        public EventFilter createFilter(@Nonnull NodeState before, @Nonnull NodeState after) {
			return null;
		}
    }

    protected static class MoveCondition implements Condition {
        @Nonnull
        @Override
        public EventFilter createFilter(@Nonnull NodeState before, @Nonnull NodeState after) {
			return null;
		}
    }

    private static class AnyCondition implements Condition {
        private final Iterable<Condition> conditions;

        public AnyCondition(Iterable<Condition> conditions) {
            this.conditions = conditions;
        }

        public AnyCondition(Condition... conditions) {
            this(newArrayList(conditions));
        }

        @Override
        public EventFilter createFilter(NodeState before, NodeState after) {
			return null;
		}
    }

    private static class AllCondition implements Condition {
        private final Iterable<Condition> conditions;

        public AllCondition(Iterable<Condition> conditions) {
            this.conditions = conditions;
        }

        public AllCondition(Condition... conditions) {
            this(newArrayList(conditions));
        }

        @Override
        public EventFilter createFilter(NodeState before, NodeState after) {
			return null;
		}
    }

    private static class NotCondition implements Condition {
        private final Condition condition;

        public NotCondition(Condition condition) {
            this.condition = condition;
        }

        @Nonnull
        @Override
        public EventFilter createFilter(NodeState before, NodeState after) {
			return null;
		}
    }

}
