/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.oak.plugins.segment;

import static com.google.common.collect.Lists.newArrayList;
import static com.google.common.collect.Maps.newHashMapWithExpectedSize;
import static java.util.Collections.nCopies;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.apache.jackrabbit.oak.plugins.segment.compaction.CompactionStrategy;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Hash table of weak references to segment identifiers.
 */
public class SegmentIdTable {

    /**
     * The list of weak references to segment identifiers that are currently
     * being accessed. This represents a hash table that uses open addressing
     * with linear probing. It is not a hash map, to speed up read access.
     * <p>
     * The size of the table is always a power of two, so that we can use
     * bitwise "and" instead of modulo.
     * <p>
     * The table is indexed by the random identifier bits, which guarantees
     * uniform distribution of entries.
     * <p>
     * Open addressing with linear probing is used. Each table entry is either
     * null (when there are no matching identifiers), a weak references to the
     * matching identifier, or a weak reference to another identifier.
     * There are no tombstone entries as there is no explicit remove operation,
     * but a referent can become null if the entry is garbage collected.
     * <p>
     * The array is not sorted (we could; lookup might be faster, but adding
     * entries would be slower).
     */
    private final ArrayList<WeakReference<SegmentId>> references =
            newArrayList(nCopies(1024, (WeakReference<SegmentId>) null));

    private final SegmentTracker tracker;
    
    private static final Logger LOG = LoggerFactory.getLogger(SegmentIdTable.class);

    
    /**
     * The refresh count (for diagnostics and testing).
     */
    private int rebuildCount;
    
    /**
     * The number of used entries (WeakReferences) in this table.
     */
    private int entryCount;

    SegmentIdTable(SegmentTracker tracker) {
        this.tracker = tracker;
    }

    /**
     * Get the segment id, and reference it in the weak references map.
     * 
     * @param msb
     * @param lsb
     * @return the segment id
     */
    synchronized SegmentId getSegmentId(long msb, long lsb) {
		return null;
	}

    /**
     * Returns all segment identifiers that are currently referenced in memory.
     *
     * @param ids referenced segment identifiers
     */
    void collectReferencedIds(Collection<SegmentId> ids) {
	}

    private synchronized Collection<SegmentId> refresh() {
		return null;
	}

    private int getIndex(SegmentId id) {
		return 0;
	}

    private int getIndex(long lsb) {
		return 0;
	}

    synchronized void clearSegmentIdTables(CompactionStrategy strategy) {
	}
    
    /**
     * Get the number of map rebuild operations (used for testing and diagnostics).
     * 
     * @return the rebuild count
     */
    int getMapRebuildCount() {
		return 0;
	}
    
    /**
     * Get the entry count (used for testing and diagnostics).
     * 
     * @return the entry count
     */
    int getEntryCount() {
		return 0;
	}
    
    /**
     * Get the size of the internal map (used for testing and diagnostics).
     * 
     * @return the map size
     */
    int getMapSize() {
		return 0;
	}
    
    /**
     * Get the raw list of segment ids (used for testing).
     * 
     * @return the raw list
     */
    List<SegmentId> getRawSegmentIdList() {
		return null;
	}

}
