/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.oak.plugins.segment.file;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.base.Preconditions.checkNotNull;
import static com.google.common.base.Preconditions.checkState;
import static com.google.common.collect.Lists.newArrayList;
import static com.google.common.collect.Lists.newArrayListWithCapacity;
import static com.google.common.collect.Lists.newLinkedList;
import static com.google.common.collect.Maps.newHashMap;
import static com.google.common.collect.Sets.newHashSet;
import static java.lang.String.format;
import static java.util.Collections.emptyMap;
import static java.util.Collections.singletonMap;
import static org.apache.jackrabbit.oak.commons.IOUtils.humanReadableByteCount;
import static org.apache.jackrabbit.oak.plugins.memory.EmptyNodeState.EMPTY_NODE;
import static org.apache.jackrabbit.oak.plugins.segment.compaction.CompactionStrategy.NO_COMPACTION;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.channels.FileLock;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;
import java.util.concurrent.Callable;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicReference;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.annotation.Nonnull;

import com.google.common.base.Stopwatch;
import com.google.common.collect.Maps;
import org.apache.jackrabbit.oak.api.Blob;
import org.apache.jackrabbit.oak.plugins.blob.BlobStoreBlob;
import org.apache.jackrabbit.oak.plugins.segment.CompactionMap;
import org.apache.jackrabbit.oak.plugins.segment.Compactor;
import org.apache.jackrabbit.oak.plugins.segment.RecordId;
import org.apache.jackrabbit.oak.plugins.segment.Segment;
import org.apache.jackrabbit.oak.plugins.segment.SegmentId;
import org.apache.jackrabbit.oak.plugins.segment.SegmentNodeState;
import org.apache.jackrabbit.oak.plugins.segment.SegmentNodeStore;
import org.apache.jackrabbit.oak.plugins.segment.SegmentNotFoundException;
import org.apache.jackrabbit.oak.plugins.segment.SegmentStore;
import org.apache.jackrabbit.oak.plugins.segment.SegmentTracker;
import org.apache.jackrabbit.oak.plugins.segment.SegmentVersion;
import org.apache.jackrabbit.oak.plugins.segment.SegmentWriter;
import org.apache.jackrabbit.oak.plugins.segment.compaction.CompactionStrategy;
import org.apache.jackrabbit.oak.spi.blob.BlobStore;
import org.apache.jackrabbit.oak.spi.gc.GCMonitor;
import org.apache.jackrabbit.oak.spi.state.NodeBuilder;
import org.apache.jackrabbit.oak.spi.state.NodeState;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * The storage implementation for tar files.
 */
public class FileStore implements SegmentStore {

    /** Logger instance */
    private static final Logger log = LoggerFactory.getLogger(FileStore.class);

    private static final int MB = 1024 * 1024;

    private static final Pattern FILE_NAME_PATTERN =
            Pattern.compile("(data|bulk)((0|[1-9][0-9]*)[0-9]{4})([a-z])?.tar");

    private static final String FILE_NAME_FORMAT = "data%05d%s.tar";

    private static final String JOURNAL_FILE_NAME = "journal.log";

    private static final String LOCK_FILE_NAME = "repo.lock";

    static final boolean MEMORY_MAPPING_DEFAULT =
            "64".equals(System.getProperty("sun.arch.data.model", "32"));

    private final SegmentTracker tracker;

    private final File directory;

    private final BlobStore blobStore;

    private final int maxFileSize;

    private final boolean memoryMapping;

    private volatile List<TarReader> readers;

    private int writeNumber;

    private File writeFile;

    private TarWriter writer;

    private final RandomAccessFile journalFile;

    private final RandomAccessFile lockFile;

    private final FileLock lock;

    /**
     * The latest head state.
     */
    private final AtomicReference<RecordId> head;

    /**
     * The persisted head of the root journal, used to determine whether the
     * latest {@link #head} value should be written to the disk.
     */
    private final AtomicReference<RecordId> persistedHead;

    /**
     * The background flush thread. Automatically flushes the TarMK state
     * once every five seconds.
     */
    private final BackgroundThread flushThread;

    /**
     * The background compaction thread. Compacts the TarMK contents whenever
     * triggered by the {@link #gc()} method.
     */
    private final BackgroundThread compactionThread;

    private CompactionStrategy compactionStrategy = NO_COMPACTION;

    /**
     * Flag to request revision cleanup during the next flush.
     */
    private final AtomicBoolean cleanupNeeded = new AtomicBoolean(false);

    /**
     * List of old tar file generations that are waiting to be removed. They can
     * not be removed immediately, because they first need to be closed, and the
     * JVM needs to release the memory mapped file references.
     */
    private final LinkedList<File> toBeRemoved = newLinkedList();

    /**
     * Version of the segment storage format.
     */
    private final SegmentVersion version = SegmentVersion.V_11;

    /**
     * {@code GCMonitor} monitoring this instance's gc progress
     */
    private final GCMonitor gcMonitor;

    /**
     * Create a new instance of a {@link Builder} for a file store.
     * @param directory  directory where the tar files are stored
     * @return a new {@link Builder} instance.
     */
    @Nonnull
    public static Builder newFileStore(@Nonnull File directory) {
		return null;
	}

    /**
     * Builder for creating {@link FileStore} instances.
     */
    public static class Builder {
        private final File directory;
        private BlobStore blobStore;   // null ->  store blobs inline
        private NodeState root = EMPTY_NODE;
        private int maxFileSize = 256;
        private int cacheSize;   // 0 -> DEFAULT_MEMORY_CACHE_SIZE
        private boolean memoryMapping;
        private final LoggingGCMonitor gcMonitor = new LoggingGCMonitor();

        private Builder(File directory) {
            this.directory = directory;
        }

        /**
         * Specify the {@link BlobStore}.
         * @param blobStore
         * @return this instance
         */
        @Nonnull
        public Builder withBlobStore(@Nonnull BlobStore blobStore) {
			return null;
		}

        /**
         * Specify the initial root node state for the file store
         * @param root
         * @return this instance
         */
        @Nonnull
        public Builder withRoot(@Nonnull NodeState root) {
			return null;
		}

        /**
         * Maximal size of the generated tar files in MB.
         * @param maxFileSize
         * @return this instance
         */
        @Nonnull
        public Builder withMaxFileSize(int maxFileSize) {
			return null;
		}

        /**
         * Size of the cache in MB.
         * @param cacheSize
         * @return this instance
         */
        @Nonnull
        public Builder withCacheSize(int cacheSize) {
			return null;
		}

        /**
         * Turn caching off
         * @return this instance
         */
        @Nonnull
        public Builder withNoCache() {
			return null;
		}

        /**
         * Turn memory mapping on or off
         * @param memoryMapping
         * @return this instance
         */
        @Nonnull
        public Builder withMemoryMapping(boolean memoryMapping) {
			return null;
		}

        /**
         * {@link GCMonitor} for monitoring this files store's gc process.
         * @param gcMonitor
         * @return this instance
         */
        @Nonnull
        public Builder withGCMonitor(@Nonnull GCMonitor gcMonitor) {
			return null;
		}

        /**
         * Create a new {@link FileStore} instance with the settings specified in this
         * builder. If none of the {@code with} methods have been called before calling
         * this method, a file store with the following default settings is returned:
         * <ul>
         * <li>blob store: inline</li>
         * <li>root: empty node</li>
         * <li>max file size: 256MB</li>
         * <li>cache size: 256MB</li>
         * <li>memory mapping: on for 64 bit JVMs off otherwise</li>
         * <li>whiteboard: none. No {@link GCMonitor} tracking</li>
         * </ul>
         *
         * @return a new file store instance
         * @throws IOException
         */
        @Nonnull
        public FileStore create() throws IOException {
			return null;
		}
    }

    @Deprecated
    public FileStore(BlobStore blobStore, File directory, int maxFileSizeMB, boolean memoryMapping)
            throws IOException {
        this(blobStore, directory, EMPTY_NODE, maxFileSizeMB, 0, memoryMapping, GCMonitor.EMPTY);
    }

    @Deprecated
    public FileStore(File directory, int maxFileSizeMB, boolean memoryMapping)
            throws IOException {
        this(null, directory, maxFileSizeMB, memoryMapping);
    }

    @Deprecated
    public FileStore(File directory, int maxFileSizeMB)
            throws IOException {
        this(null, directory, maxFileSizeMB, MEMORY_MAPPING_DEFAULT);
    }

    @Deprecated
    public FileStore(File directory, int maxFileSizeMB, int cacheSizeMB,
            boolean memoryMapping) throws IOException {
        this(null, directory, EMPTY_NODE, maxFileSizeMB, cacheSizeMB, memoryMapping, GCMonitor.EMPTY);
    }

    @Deprecated
    FileStore(File directory, NodeState initial, int maxFileSize) throws IOException {
        this(null, directory, initial, maxFileSize, -1, MEMORY_MAPPING_DEFAULT, GCMonitor.EMPTY);
    }

    @Deprecated
    public FileStore(
            BlobStore blobStore, final File directory, NodeState initial, int maxFileSizeMB,
            int cacheSizeMB, boolean memoryMapping) throws IOException {
        this(blobStore, directory, initial, maxFileSizeMB, cacheSizeMB, memoryMapping, GCMonitor.EMPTY);
    }

    private FileStore(
            BlobStore blobStore, final File directory, NodeState initial, int maxFileSizeMB,
            int cacheSizeMB, boolean memoryMapping, GCMonitor gcMonitor)
            throws IOException {
        checkNotNull(directory).mkdirs();
        if (cacheSizeMB < 0) {
            this.tracker = new SegmentTracker(this, 0, getVersion());
        } else if (cacheSizeMB > 0) {
            this.tracker = new SegmentTracker(this, cacheSizeMB, getVersion());
        } else {
            this.tracker = new SegmentTracker(this, getVersion());
        }
        this.blobStore = blobStore;
        this.directory = directory;
        this.maxFileSize = maxFileSizeMB * MB;
        this.memoryMapping = memoryMapping;
        this.gcMonitor = gcMonitor;

        journalFile = new RandomAccessFile(new File(directory, JOURNAL_FILE_NAME), "rw");
        lockFile = new RandomAccessFile(new File(directory, LOCK_FILE_NAME), "rw");

        Map<Integer, Map<Character, File>> map = collectFiles(directory);
        this.readers = newArrayListWithCapacity(map.size());
        Integer[] indices = map.keySet().toArray(new Integer[map.size()]);
        Arrays.sort(indices);
        for (int i = indices.length - 1; i >= 0; i--) {
            readers.add(TarReader.open(map.get(indices[i]), memoryMapping));
        }

        if (indices.length > 0) {
            this.writeNumber = indices[indices.length - 1] + 1;
        } else {
            this.writeNumber = 0;
        }
        this.writeFile = new File(
                directory,
                String.format(FILE_NAME_FORMAT, writeNumber, "a"));
        this.writer = new TarWriter(writeFile);

        RecordId id = null;
        JournalReader journalReader = new JournalReader(new File(directory, JOURNAL_FILE_NAME));
        try {
            Iterator<String> heads = journalReader.iterator();
            while (id == null && heads.hasNext()) {
                RecordId last = RecordId.fromString(tracker, heads.next());
                SegmentId segmentId = last.getSegmentId();
                if (containsSegment(
                        segmentId.getMostSignificantBits(),
                        segmentId.getLeastSignificantBits())) {
                    id = last;
                } else {
                    log.warn("Unable to access revision {}, rewinding...", last);
                }
            }
        } finally {
            journalReader.close();
        }

        journalFile.seek(journalFile.length());
        lock = lockFile.getChannel().lock();

        if (id != null) {
            head = new AtomicReference<RecordId>(id);
            persistedHead = new AtomicReference<RecordId>(id);
        } else {
            NodeBuilder builder = EMPTY_NODE.builder();
            builder.setChildNode("root", initial);
            head = new AtomicReference<RecordId>(tracker.getWriter().writeNode(
                    builder.getNodeState()).getRecordId());
            persistedHead = new AtomicReference<RecordId>(null);
        }

        this.flushThread = new BackgroundThread(
                "TarMK flush thread [" + directory + "]", 5000, // 5s interval
                new Runnable() {
                    @Override
                    public void run() {
                        try {
                            flush();
                        } catch (IOException e) {
                            log.warn("Failed to flush the TarMK at" +
                                    directory, e);
                        }
                    }
                });
        this.compactionThread = new BackgroundThread(
                "TarMK compaction thread [" + directory + "]", -1,
                new Runnable() {
                    @Override
                    public void run() {
                        maybeCompact(true);
                    }
                });

        log.info("TarMK opened: {} (mmap={})", directory, memoryMapping);
    }

    public boolean maybeCompact(boolean cleanup) {
		return false;
	}

    static Map<Integer, Map<Character, File>> collectFiles(File directory)
            throws IOException {
				return null;
			}

    public synchronized long size() throws IOException {
		return 0L;
	}

    /**
     * Returns the number of segments in this TarMK instance.
     *
     * @return number of segments
     */
    private synchronized int count() {
		return 0;
	}

    CompactionGainEstimate estimateCompactionGain() {
		return null;
	}

    public void flush() throws IOException {
	}

    /**
     * Runs garbage collection on the segment level, which could write new
     * generations of tar files. It checks which segments are still reachable,
     * and throws away those that are not.
     * <p>
     * A new generation of a tar file is created (and segments are only
     * discarded) if doing so releases more than 25% of the space in a tar file.
     */
    public synchronized void cleanup() throws IOException {
	}

    /**
     * Copy every referenced record in data (non-bulk) segments. Bulk segments
     * are fully kept (they are only removed in cleanup, if there is no
     * reference to them).
     */
    public void compact() {
	}

    public synchronized Iterable<SegmentId> getSegmentIds() {
		return null;
	}

    @Override
    public SegmentTracker getTracker() {
		return null;
	}

    @Override
    public SegmentNodeState getHead() {
		return null;
	}

    @Override
    public boolean setHead(SegmentNodeState base, SegmentNodeState head) {
		return false;
	}

    @Override
    public void close() {
	}

    @Override
    public boolean containsSegment(SegmentId id) {
		return false;
	}

    private boolean containsSegment(long msb, long lsb) {
		return false;
	}

    @Override
    public Segment readSegment(SegmentId id) {
		return null;
	}

    @Override
    public synchronized void writeSegment(
            SegmentId id, byte[] data, int offset, int length) {
			}

    private void newWriter() throws IOException {
	}

    @Override
    public Blob readBlob(String blobId) {
		return null;
	}

    @Override
    public BlobStore getBlobStore() {
		return null;
	}

    @Override
    public void gc() {
	}

    public Map<String, Set<UUID>> getTarReaderIndex() {
		return null;
	}

    public Map<UUID, List<UUID>> getTarGraph(String fileName) throws IOException {
		return null;
	}

    public FileStore setCompactionStrategy(CompactionStrategy strategy) {
		return null;
	}

    private synchronized void setRevision(String rootRevision) {
	}

    /**
     * A read only {@link FileStore} implementation that supports
     * going back to old revisions.
     * <p>
     * All write methods are no-ops.
     */
    public static class ReadOnlyStore extends FileStore {
        public ReadOnlyStore(File directory) throws IOException {
            super(directory, 266);
            super.flushThread.close();
            super.compactionThread.close();
        }

        /**
         * Go to the specified {@code revision}
         *
         * @param revision
         */
        public synchronized void setRevision(String revision) {
		}

        /**
         * @return false
         */
        @Override
        public boolean setHead(SegmentNodeState base, SegmentNodeState head) {
			return false;
		}

        /**
         * no-op
         */
        @Override
        public synchronized void writeSegment(SegmentId id, byte[] data, int offset, int length) {
		}

        /**
         * no-op
         */
        @Override
        public void flush() {
		}

        /**
         * no-op
         */
        @Override
        public synchronized void cleanup() {
		}

        /**
         * no-op
         */
        @Override
        public void gc() {
		}

    }

    private class SetHead implements Callable<Boolean> {
        private final SegmentNodeState before;
        private final SegmentNodeState after;
        private final Compactor compactor;

        public SetHead(SegmentNodeState before, SegmentNodeState after, Compactor compactor) {
            this.before = before;
            this.after = after;
            this.compactor = compactor;
        }

        @Override
        public Boolean call() throws Exception {
			return null;
		}
    }

    public SegmentVersion getVersion() {
		return null;
	}

    private static class LoggingGCMonitor implements GCMonitor {
        public GCMonitor delegatee = GCMonitor.EMPTY;

        @Override
        public void info(String message, Object... arguments) {
		}

        @Override
        public void warn(String message, Object... arguments) {
		}

        @Override
        public void error(String message, Exception exception) {
		}

        @Override
        public void skipped(String reason, Object... arguments) {
		}

        @Override
        public void compacted() {
		}

        @Override
        public void cleaned(long reclaimedSize, long currentSize) {
		}
    }
}
