/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.jackrabbit.oak.plugins.version;

import static com.google.common.base.Preconditions.checkNotNull;
import static javax.jcr.version.OnParentVersionAction.ABORT;
import static javax.jcr.version.OnParentVersionAction.COMPUTE;
import static javax.jcr.version.OnParentVersionAction.COPY;
import static javax.jcr.version.OnParentVersionAction.IGNORE;
import static javax.jcr.version.OnParentVersionAction.INITIALIZE;
import static javax.jcr.version.OnParentVersionAction.VERSION;
import static org.apache.jackrabbit.JcrConstants.JCR_BASEVERSION;
import static org.apache.jackrabbit.JcrConstants.JCR_CREATED;
import static org.apache.jackrabbit.JcrConstants.JCR_FROZENMIXINTYPES;
import static org.apache.jackrabbit.JcrConstants.JCR_FROZENNODE;
import static org.apache.jackrabbit.JcrConstants.JCR_FROZENPRIMARYTYPE;
import static org.apache.jackrabbit.JcrConstants.JCR_FROZENUUID;
import static org.apache.jackrabbit.JcrConstants.JCR_ISCHECKEDOUT;
import static org.apache.jackrabbit.JcrConstants.JCR_MIXINTYPES;
import static org.apache.jackrabbit.JcrConstants.JCR_PREDECESSORS;
import static org.apache.jackrabbit.JcrConstants.JCR_PRIMARYTYPE;
import static org.apache.jackrabbit.JcrConstants.JCR_UUID;
import static org.apache.jackrabbit.JcrConstants.JCR_VERSIONHISTORY;
import static org.apache.jackrabbit.JcrConstants.MIX_VERSIONABLE;
import static org.apache.jackrabbit.JcrConstants.NT_FROZENNODE;
import static org.apache.jackrabbit.JcrConstants.NT_VERSIONEDCHILD;
import static org.apache.jackrabbit.oak.plugins.version.Utils.primaryTypeOf;
import static org.apache.jackrabbit.oak.plugins.version.Utils.uuidFromNode;

import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.nodetype.PropertyDefinition;
import javax.jcr.version.OnParentVersionAction;

import com.google.common.collect.Lists;
import org.apache.jackrabbit.oak.api.CommitFailedException;
import org.apache.jackrabbit.oak.api.PropertyState;
import org.apache.jackrabbit.oak.api.Tree;
import org.apache.jackrabbit.oak.api.Type;
import org.apache.jackrabbit.oak.plugins.identifier.IdentifierManager;
import org.apache.jackrabbit.oak.plugins.memory.EmptyNodeState;
import org.apache.jackrabbit.oak.plugins.memory.MemoryNodeBuilder;
import org.apache.jackrabbit.oak.plugins.memory.PropertyStates;
import org.apache.jackrabbit.oak.plugins.nodetype.ReadOnlyNodeTypeManager;
import org.apache.jackrabbit.oak.plugins.tree.TreeFactory;
import org.apache.jackrabbit.oak.plugins.tree.impl.ImmutableTree;
import org.apache.jackrabbit.oak.plugins.tree.impl.TreeConstants;
import org.apache.jackrabbit.oak.spi.state.NodeBuilder;
import org.apache.jackrabbit.oak.spi.state.NodeState;
import org.apache.jackrabbit.oak.spi.state.NodeStateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * {@code VersionableState} provides methods to create a versionable state
 * for a version based on a versionable node.
 * <p>
 * The restore implementation of this class does not handle the removeExisting
 * flag. It is expected that this is handled on a higher level. If this is not
 * done the uniqueness constraint on the jcr:uuid will kick in and fail the
 * commit.
 * </p>
 */
class VersionableState {

    private static final Logger log = LoggerFactory.getLogger(VersionableState.class);

    private static final String JCR_CHILDVERSIONHISTORY = "jcr:childVersionHistory";
    private static final Set<String> BASIC_PROPERTIES = new HashSet<String>();
    private static final Set<String> BASIC_FROZEN_PROPERTIES = new HashSet<String>();

    static {
        BASIC_PROPERTIES.add(JCR_PRIMARYTYPE);
        BASIC_PROPERTIES.add(JCR_UUID);
        BASIC_PROPERTIES.add(JCR_MIXINTYPES);
        BASIC_FROZEN_PROPERTIES.addAll(BASIC_PROPERTIES);
        BASIC_FROZEN_PROPERTIES.add(JCR_FROZENPRIMARYTYPE);
        BASIC_FROZEN_PROPERTIES.add(JCR_FROZENUUID);
        BASIC_FROZEN_PROPERTIES.add(JCR_FROZENMIXINTYPES);
    }

    private final NodeBuilder version;
    private final NodeBuilder history;
    private final NodeBuilder frozenNode;
    private final NodeBuilder versionable;
    private final ReadWriteVersionManager vMgr;
    private final ReadOnlyNodeTypeManager ntMgr;

    private VersionableState(@Nonnull NodeBuilder version,
                             @Nonnull NodeBuilder history,
                             @Nonnull NodeBuilder versionable,
                             @Nonnull ReadWriteVersionManager vMgr,
                             @Nonnull ReadOnlyNodeTypeManager ntMgr) {
        this.version = checkNotNull(version);
        this.history = checkNotNull(history);
        this.frozenNode = version.child(JCR_FROZENNODE);
        this.versionable = checkNotNull(versionable);
        this.vMgr = checkNotNull(vMgr);
        this.ntMgr = checkNotNull(ntMgr);
    }

    /**
     * Creates a frozen node under the version and initializes it with the basic
     * frozen properties (jcr:frozenPrimaryType, jcr:frozenMixinTypes and
     * jcr:frozenUuid) from the given versionable node.
     *
     * @param version the parent node of the frozen node.
     * @param history the history node of the version.
     * @param versionable the versionable node.
     * @param vMgr the version manager.
     * @param ntMgr the node type manager.
     * @return a versionable state
     */
    @Nonnull
    static VersionableState fromVersion(@Nonnull NodeBuilder version,
                                        @Nonnull NodeBuilder history,
                                        @Nonnull NodeBuilder versionable,
                                        @Nonnull ReadWriteVersionManager vMgr,
                                        @Nonnull ReadOnlyNodeTypeManager ntMgr) {
											return null;
										}

    /**
     * Creates a versionable state for a restore.
     *
     * @param version the version to restore.
     * @param history the history node of the version.
     * @param versionable the versionable node.
     * @param vMgr the version manager.
     * @param ntMgr the node type manager.
     * @return a versionable state.
     */
    static VersionableState forRestore(@Nonnull NodeBuilder version,
                                       @Nonnull NodeBuilder history,
                                       @Nonnull NodeBuilder versionable,
                                       @Nonnull ReadWriteVersionManager vMgr,
                                       @Nonnull ReadOnlyNodeTypeManager ntMgr) {
										return null;
									}

    /**
     * Creates a frozen node and initializes it with the basic
     * frozen properties (jcr:frozenPrimaryType, jcr:frozenMixinTypes and
     * jcr:frozenUuid) from the given node.
     *
     * @return this versionable state.
     */
    private VersionableState initFrozen(NodeBuilder frozen,
                                        NodeBuilder node,
                                        String nodeId) {
											return null;
										}

    /**
     * Creates the versionable state under the version.
     *
     * @return the frozen node.
     * @throws CommitFailedException if the operation fails. E.g. because the
     *              versionable node has a property with OPV ABORT.
     */
    NodeBuilder create() throws CommitFailedException {
		return null;
	}

    /**
     * Restore the versionable node to the given version.
     *
     * @param selector an optional version selector. If none is passed, this
     *                 method will use a date based version selector.
     * @return the versionable node.
     * @throws CommitFailedException if the operation fails.
     */
    public NodeBuilder restore(@Nullable VersionSelector selector)
            throws CommitFailedException {
				return null;
			}

    //--------------------------< internal >------------------------------------

    /**
     * Restores the state from {@code src} to a child node of
     * {@code destParent} with the same name as {@code src}.
     *
     * @param src the source node.
     * @param destParent the parent of the destination node.
     * @param name the name of the source node.
     * @param selector the version selector.
     */
    private void restoreState(@Nonnull NodeBuilder src,
                              @Nonnull NodeBuilder destParent,
                              @Nonnull String name,
                              @Nonnull VersionSelector selector)
            throws RepositoryException, CommitFailedException {
			}

    /**
     * Restore a nt:frozenNode.
     */
    private void restoreFrozen(@Nonnull NodeBuilder frozen,
                               @Nonnull NodeBuilder dest,
                               @Nonnull VersionSelector selector)
            throws RepositoryException, CommitFailedException {
			}

    /**
     * Restores the basic frozen properties (jcr:primaryType, jcr:mixinTypes
     * and jcr:uuid).
     */
    private void restoreFrozenTypeAndUUID(@Nonnull NodeBuilder frozen,
                                          @Nonnull NodeBuilder dest) {
										}

    /**
     * Restore a copied node.
     */
    private void restoreCopiedNode(NodeBuilder src,
                                   NodeBuilder dest,
                                   VersionSelector selector)
            throws RepositoryException, CommitFailedException {
			}

    /**
     * Restore an nt:versionedChild node.
     */
    private void restoreVersionedChild(NodeBuilder versionedChild,
                                       NodeBuilder dest,
                                       VersionSelector selector)
            throws RepositoryException, CommitFailedException {
			}

    /**
     * Restores children of a {@code src}.
     */
    private void restoreChildren(NodeBuilder src,
                                 NodeBuilder dest,
                                 VersionSelector selector)
            throws RepositoryException, CommitFailedException {
			}

    /**
     * 15.7.7 Simple vs. Full Versioning after Restore
     */
    private void restoreVersionable(@Nonnull NodeBuilder versionable,
                                    @Nonnull NodeBuilder version) {
									}

    private void resetToDefaultValue(NodeBuilder dest, PropertyState p)
            throws RepositoryException {
			}

    private void createFrozen(NodeBuilder src, String srcId, NodeBuilder dest)
            throws CommitFailedException, RepositoryException {
			}

    private void versionedChild(NodeBuilder src, NodeBuilder dest) {
	}

    private void copy(NodeBuilder src,
                      String srcId,
                      NodeBuilder dest)
            throws RepositoryException, CommitFailedException {
			}

    /**
     * Returns the id of the {@code child} node. The id is the value of the
     * jcr:uuid property of the child node if present, or the concatenation of
     * the {@code parentId} and the {@code name} of the child node.
     *
     * @param parentId the parentId.
     * @param child the child node.
     * @param name the name of the child node.
     * @return the identifier of the child node.
     */
    private String getChildId(String parentId, NodeBuilder child, String name) {
		return null;
	}

    private void copyProperties(NodeBuilder src,
                                NodeBuilder dest,
                                OPVProvider opvProvider,
                                boolean ignoreTypeAndUUID)
            throws RepositoryException, CommitFailedException {
			}

    private static boolean isHiddenProperty(@Nonnull String propName) {
		return false;
	}

    private int getOPV(NodeBuilder parent, NodeBuilder child, String childName)
            throws RepositoryException {
				return 0;
			}

    private int getOPV(NodeBuilder node, PropertyState property)
            throws RepositoryException {
				return 0;
			}

    private interface OPVProvider {

        int getAction(NodeBuilder src,
                NodeBuilder dest,
                PropertyState prop)
                throws RepositoryException {
			return 0;
		}
    }

    private static final class OPVForceCopy implements OPVProvider {

        private static final OPVProvider INSTANCE = new OPVForceCopy();

        @Override
        public int getAction(NodeBuilder src,
                             NodeBuilder dest,
                             PropertyState prop) {
								return 0;
							}
    }
}
