/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.oak.spi.blob;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.lang.ref.WeakReference;
import java.security.InvalidKeyException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.WeakHashMap;
import java.util.concurrent.atomic.AtomicReference;

import javax.annotation.Nonnull;
import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;

import com.google.common.base.Charsets;
import com.google.common.io.BaseEncoding;
import org.apache.commons.io.FileUtils;
import org.apache.jackrabbit.oak.commons.cache.Cache;
import org.apache.jackrabbit.oak.commons.IOUtils;
import org.apache.jackrabbit.oak.commons.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.base.Preconditions.checkNotNull;

/**
 * An abstract data store that splits the binaries in relatively small blocks,
 * so that each block fits in memory.
 * <p>
 * Each data store id is a list of zero or more entries. Each entry is either
 * <ul>
 * <li>data (a number of bytes), or</li>
 * <li>the hash code of the content of a number of bytes, or</li>
 * <li>the hash code of the content of a data store id (indirect hash)</li>
 * </ul>
 * Thanks to the indirection, blocks can be kept relatively small, so that
 * caching is simpler, and so that the storage backend doesn't need to support
 * arbitrary size blobs (some storage backends buffer blobs in memory) and fast
 * seeks (some storage backends re-read the whole blob when seeking).
 * <p>
 * The format of a 'data' entry is: type (one byte; 0 for data), length
 * (variable size int), data (bytes).
 * <p>
 * The format of a 'hash of content' entry is: type (one byte; 1 for hash),
 * level (variable size int, 0 meaning not nested), size (variable size long),
 * hash code length (variable size int), hash code.
 * <p>
 * The format of a 'hash of data store id' entry is: type (one byte; 1 for
 * hash), level (variable size int, nesting level), total size (variable size
 * long), size of data store id (variable size long), hash code length (variable
 * size int), hash code.
 */
public abstract class AbstractBlobStore implements GarbageCollectableBlobStore,
        Cache.Backend<AbstractBlobStore.BlockId, AbstractBlobStore.Data> {

    protected static final String HASH_ALGORITHM = "SHA-256";

    protected static final int TYPE_DATA = 0;
    protected static final int TYPE_HASH = 1;

    /**
     * The minimum block size. Blocks must be larger than that so that the
     * content hash is always shorter than the data itself.
     */
    protected static final int BLOCK_SIZE_LIMIT = 48;

    /**
     * The blob ids that are still floating around in memory. The blob store
     * assumes such binaries must not be deleted, because those binaries are not
     * referenced yet in a way the garbage collection algorithm can detect (not
     * referenced at all, or only referenced in memory).
     */
    protected Map<String, WeakReference<String>> inUse =
        Collections.synchronizedMap(new WeakHashMap<String, WeakReference<String>>());

    /**
     * The minimum size of a block. Smaller blocks are inlined (the data store id
     * is the data itself).
     */
    private int blockSizeMin = 4096;

    /**
     * The size of a block. 128 KB has been found to be as fast as larger
     * values, and faster than smaller values. 2 MB results in less files.
     */
    private int blockSize = 2 * 1024 * 1024;

    /**
     * The byte array is re-used if possible, to avoid having to create a new,
     * large byte array each time a (potentially very small) binary is stored.
     */
    private AtomicReference<byte[]> blockBuffer = new AtomicReference<byte[]>();

    /**
     * Encryption algorithm used to encrypt blobId as references
     */
    private static final String ALGORITHM = "HmacSHA1";

    /**
     * Encryption key for creating secure references from blobId
     */
    private byte[] referenceKey;

    private final Logger log = LoggerFactory.getLogger(getClass());

    public void setBlockSizeMin(int x) {
	}

    @Override
    public long getBlockSizeMin() {
		return 0L;
	}

    @Override
    public void setBlockSize(int x) {
	}

    private static void validateBlockSize(int x) {
	}

    public int getBlockSize() {
		return 0;
	}

    @Override
    public String writeBlob(String tempFilePath) throws IOException {
		return null;
	}

    @Override
    public String writeBlob(InputStream in) throws IOException {
		return null;
	}

    @Override
    public InputStream getInputStream(String blobId) throws IOException {
		return null;
	}

    //--------------------------------------------< Blob Reference >

    @Override
    public String getReference(@Nonnull String blobId) {
		return null;
	}

    @Override
    public String getBlobId(@Nonnull String reference) {
		return null;
	}

    /**
     * Returns the reference key of this blob store. If one does not already
     * exist, it is automatically created in an implementation-specific way.
     * The default implementation simply creates a temporary random key that's
     * valid only until the data store gets restarted. Subclasses can override
     * and/or decorate this method to support a more persistent reference key.
     * <p>
     * This method is called only once during the lifetime of a data store
     * instance and the return value is cached in memory, so it's no problem
     * if the implementation is slow.
     *
     * @return reference key
     */
    protected byte[] getOrCreateReferenceKey() {
		return null;
	}

    /**
     * Returns the reference key of this data store. Synchronized to
     * control concurrent access to the cached {@link #referenceKey} value.
     *
     * @return reference key
     */
    private synchronized byte[] getReferenceKey() {
		return null;
	}

    public void setReferenceKey(byte[] referenceKey) {
	}

    /**
     * Set the referenceKey from Base64 encoded byte array
     * @param encodedKey base64 encoded key
     */
    public void setReferenceKeyEncoded(String encodedKey) {
	}

    /**
     * Set the referenceKey from plain text. Key content would be UTF-8 encoding
     * of the string.
     * 
     * <p>
     * This is useful when setting key via generic bean property manipulation
     * from string properties. User can specify the key in plain text and that
     * would be passed on this object via
     * {@link org.apache.jackrabbit.oak.commons.PropertiesUtil#populate(Object, java.util.Map, boolean)}
     * 
     * @param textKey base64 encoded key
     * @see org.apache.jackrabbit.oak.commons.PropertiesUtil#populate(Object,
     *      java.util.Map, boolean)
     */
    public void setReferenceKeyPlainText(String textKey) {
	}

    protected void usesBlobId(String blobId) {
	}

    @Override
    public void clearInUse() {
	}

    private void convertBlobToId(InputStream in,
            ByteArrayOutputStream idStream, int level, long totalLength)
            throws IOException {
			}

    /**
     * Store a block of data.
     * 
     * @param digest the content hash (32 bytes)
     * @param level the indirection level (0 is for user data, 1 is a list of
     *            digests that point to user data, 2 is a list of digests that
     *            point to digests, and so on). This parameter is for
     *            informational use only, and it is not required to store it
     *            unless that's easy to achieve
     * @param data the data to be stored (the number of bytes is at most the block size)
     */
    protected abstract void storeBlock(byte[] digest, int level, byte[] data) throws IOException {
	}

    @Override
    public abstract void startMark() throws IOException {
	}

    @Override
    public abstract int sweep() throws IOException {
		return 0;
	}

    protected abstract boolean isMarkEnabled() {
		return false;
	}

    protected abstract void mark(BlockId id) throws Exception {
	}

    protected void markInUse() throws IOException {
	}

    @Override
    public int readBlob(String blobId, long pos, byte[] buff, int off,
            int length) throws IOException {
				return 0;
			}

    byte[] readBlock(byte[] digest, long pos) {
		return null;
	}

    @Override
    public Data load(BlockId id) {
		return null;
	}

    /**
     * Load the block from the storage backend. Returns null if the block was
     * not found.
     * 
     * @param id the block id
     * @return the block data, or null
     */
    protected abstract byte[] readBlockFromBackend(BlockId id) throws Exception {
		return null;
	}

    @Override
    public long getBlobLength(String blobId) throws IOException {
		return 0L;
	}

    protected void mark(String blobId) throws IOException {
	}

    private void mark(ByteArrayInputStream idStream) throws Exception {
	}

    @Override
    public Iterator<String> resolveChunks(String blobId) throws IOException {
		return null;
	}

    @Override
    public boolean deleteChunks(List<String> chunkIds, long maxLastModifiedTime) throws Exception {
		return false;
	}
    
    /**
     * A block id. Blocks are small enough to fit in memory, so they can be
     * cached.
     */
    public static class BlockId {

        /**
         * The digest (32 bytes).
         */
        final byte[] digest;

        final long pos;

        BlockId(byte[] digest, long pos) {
            this.digest = digest;
            this.pos = pos;
        }

        @Override
        public boolean equals(Object other) {
			return false;
		}

        @Override
        public int hashCode() {
			return 0;
		}

        @Override
        public String toString() {
			return null;
		}

        public byte[] getDigest() {
			return null;
		}

        public long getPos() {
			return 0L;
		}

    }

    /**
     * The data for a block.
     */
    public static class Data implements Cache.Value {

        final byte[] data;

        Data(byte[] data) {
            this.data = data;
        }

        @Override
        public String toString() {
			return null;
		}

        @Override
        public int getMemory() {
			return 0;
		}

    }

    class ChunkIterator implements Iterator<String> {
        
        private final static int BATCH = 2048;
        private final ArrayDeque<String> queue;
        private final ArrayDeque<ByteArrayInputStream> streamsStack;

        public ChunkIterator(String blobId) {
            byte[] id = StringUtils.convertHexToBytes(blobId);
            ByteArrayInputStream idStream = new ByteArrayInputStream(id);
            queue = new ArrayDeque<String>(BATCH);
            streamsStack = new ArrayDeque<ByteArrayInputStream>();
            streamsStack.push(idStream);
        }

        @Override
        public boolean hasNext() {
			return false;
		}

        @Override
        public String next() {
			return null;
		}

        @Override
        public void remove() {
		}
    }
}
