/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.rmi.client;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.rmi.RemoteException;
import java.security.AccessControlException;

import javax.jcr.Credentials;
import javax.jcr.Item;
import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.UnsupportedRepositoryOperationException;
import javax.jcr.ValueFactory;
import javax.jcr.Workspace;
import javax.jcr.retention.RetentionManager;
import javax.jcr.security.AccessControlManager;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.jackrabbit.rmi.remote.RemoteSession;
import org.apache.jackrabbit.rmi.value.SerialValueFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

/**
 * Local adapter for the JCR-RMI
 * {@link org.apache.jackrabbit.rmi.remote.RemoteSession RemoteSession}
 * interface. This class makes a remote session locally available using
 * the JCR {@link javax.jcr.Session Session} interface.
 *
 * @see javax.jcr.Session
 * @see org.apache.jackrabbit.rmi.remote.RemoteSession
 */
public class ClientSession extends ClientObject implements Session {

    /**
     * Logger instance.
     */
    private static final Logger log =
        LoggerFactory.getLogger(ClientSession.class);

    /** The current repository. */
    private final Repository repository;

    /**
     * Flag indicating whether the session is to be considered live of not.
     * This flag is initially set to <code>true</code> and reset to
     * <code>false</code> by the {@link #logout()} method. The {@link #isLive()}
     * method first checks this flag before asking the remote session.
     */
    private boolean live = true;

    /** The adapted remote session. */
    protected final RemoteSession remote;

    /**
     * The adapted workspace of this session. This field is set on the first
     * call to the {@link #getWorkspace()} method assuming, that a workspace
     * instance is not changing during the lifetime of a session, that is,
     * each call to the server-side <code>Session.getWorkspace()</code> allways
     * returns the same object.
     */
    private Workspace workspace;

    /**
     * Creates a client adapter for the given remote session.
     *
     * @param repository current repository
     * @param remote remote repository
     * @param factory local adapter factory
     */
    public ClientSession(Repository repository, RemoteSession remote,
            LocalAdapterFactory factory) {
        super(factory);
        this.repository = repository;
        this.remote = remote;
    }

    /**
     * Returns the current repository without contacting the remote session.
     *
     * {@inheritDoc}
     */
    public Repository getRepository() {
		return null;
	}

    /** {@inheritDoc} */
    public String getUserID() {
		return null;
	}

    /** {@inheritDoc} */
    public Object getAttribute(String name) {
		return null;
	}

    /** {@inheritDoc} */
    public String[] getAttributeNames() {
		return null;
	}

    /** {@inheritDoc} */
    public Workspace getWorkspace() {
		return null;
	}

    /** {@inheritDoc} */
    public Session impersonate(Credentials credentials)
            throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public Node getRootNode() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public Node getNodeByIdentifier(String id) throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public Node getNodeByUUID(String uuid) throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public Item getItem(String path) throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public Node getNode(String path) throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public Property getProperty(String path) throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public boolean itemExists(String path) throws RepositoryException {
		return false;
	}

    /** {@inheritDoc} */
    public boolean nodeExists(String path) throws RepositoryException {
		return false;
	}

    /** {@inheritDoc} */
    public boolean propertyExists(String path) throws RepositoryException {
		return false;
	}

    /** {@inheritDoc} */
    public void removeItem(String path) throws RepositoryException {
	}

    /** {@inheritDoc} */
    public void move(String from, String to) throws RepositoryException {
	}

    /** {@inheritDoc} */
    public void save() throws RepositoryException {
	}

    /** {@inheritDoc} */
    public void refresh(boolean keepChanges) throws RepositoryException {
	}

    /** {@inheritDoc} */
    public boolean hasPendingChanges() throws RepositoryException {
		return false;
	}

    /**
     * Returns the {@link SerialValueFactory#getInstance()}.
     *
     * {@inheritDoc}
     */
    public ValueFactory getValueFactory() {
		return null;
	}

    /** {@inheritDoc} */
    public void checkPermission(String path, String actions)
            throws AccessControlException, RepositoryException {
			}

    /** {@inheritDoc} */
    public boolean hasPermission(String path, String actions)
            throws RepositoryException {
				return false;
			}

    /** {@inheritDoc} */
    public void importXML(String path, InputStream xml, int mode)
            throws IOException, RepositoryException {
			}

    /** {@inheritDoc} */
    public ContentHandler getImportContentHandler(
            final String path, final int mode) throws RepositoryException {
				return null;
			}

    /** {@inheritDoc} */
    public void setNamespacePrefix(String prefix, String uri)
            throws RepositoryException {
			}

    /** {@inheritDoc} */
    public String[] getNamespacePrefixes() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public String getNamespaceURI(String prefix) throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public String getNamespacePrefix(String uri) throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public void logout() {
	}

    /** {@inheritDoc} */
    public void addLockToken(String name) {
	}

    /** {@inheritDoc} */
    public String[] getLockTokens() {
		return null;
	}

    /** {@inheritDoc} */
    public void removeLockToken(String name) {
	}

    /**
     * Exports the XML system view of the specified repository location
     * to the given XML content handler. This method first requests the
     * raw XML data from the remote session, and then uses an identity
     * transformation to feed the data to the given XML content handler.
     * Possible IO and transformer exceptions are thrown as SAXExceptions.
     *
     * {@inheritDoc}
     */
    public void exportSystemView(
            String path, ContentHandler handler,
            boolean binaryAsLink, boolean noRecurse)
            throws SAXException, RepositoryException {
			}

    /**
     * Exports the XML system view of the specified repository location
     * to the given output stream. This method first requests the
     * raw XML data from the remote session, and then writes the data to
     * the output stream.
     *
     * {@inheritDoc}
     */
    public void exportSystemView(
            String path, OutputStream output,
            boolean binaryAsLink, boolean noRecurse)
            throws IOException, RepositoryException {
			}

    /**
     * Exports the XML document view of the specified repository location
     * to the given XML content handler. This method first requests the
     * raw XML data from the remote session, and then uses an identity
     * transformation to feed the data to the given XML content handler.
     * Possible IO and transformer exceptions are thrown as SAXExceptions.
     *
     * {@inheritDoc}
     */
    public void exportDocumentView(
            String path, ContentHandler handler,
            boolean binaryAsLink, boolean noRecurse)
            throws SAXException, RepositoryException {
			}

    /**
     * Exports the XML document view of the specified repository location
     * to the given output stream. This method first requests the
     * raw XML data from the remote session, and then writes the data to
     * the output stream.
     *
     * {@inheritDoc}
     */
    public void exportDocumentView(
            String path, OutputStream output,
            boolean binaryAsLink, boolean noRecurse)
            throws IOException, RepositoryException {
			}

    /**
     * {@inheritDoc}
     */
    public boolean isLive() {
		return false;
	}

    public AccessControlManager getAccessControlManager()
            throws UnsupportedRepositoryOperationException, RepositoryException {
				return null;
			}

    public RetentionManager getRetentionManager()
            throws RepositoryException {
				return null;
			}

    public boolean hasCapability(
            String methodName, Object target, Object[] arguments)
            throws RepositoryException {
				return false;
			}

}
