/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.spi.commons;

import static org.apache.jackrabbit.spi.commons.iterator.Iterators.filterIterator;
import static org.apache.jackrabbit.spi.commons.iterator.Iterators.transformIterator;

import org.apache.jackrabbit.spi.ChildInfo;
import org.apache.jackrabbit.spi.ItemInfo;
import org.apache.jackrabbit.spi.Name;
import org.apache.jackrabbit.spi.NodeId;
import org.apache.jackrabbit.spi.NodeInfo;
import org.apache.jackrabbit.spi.Path;
import org.apache.jackrabbit.spi.PropertyId;
import org.apache.jackrabbit.spi.PropertyInfo;
import org.apache.jackrabbit.spi.QValue;
import org.apache.jackrabbit.spi.commons.identifier.IdFactoryImpl;
import org.apache.jackrabbit.spi.commons.iterator.Iterators;
import org.apache.jackrabbit.spi.commons.iterator.Predicate;
import org.apache.jackrabbit.spi.commons.iterator.Transformer;
import org.apache.jackrabbit.spi.commons.name.NameConstants;
import org.apache.jackrabbit.spi.commons.name.NameFactoryImpl;
import org.apache.jackrabbit.spi.commons.name.PathFactoryImpl;
import org.apache.jackrabbit.spi.commons.value.QValueFactoryImpl;

import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.net.URI;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;

/**
 * Utility class providing a {@link NodeInfoBuilder} for building {@link NodeInfo}.
 * Example usage:
 * <pre>
 * ItemInfoBuilder.nodeInfoBuilder()
 *     .createNodeInfo("node1")
 *         .createPropertyInfo("prop1", "value1").build()
 *         .createPropertyInfo("prop2")
 *             .addValue(1.2)
 *             .addValue(2.3)
 *         .build()
 *     .build()
 *     .createNodeInfo("node2")
 *         .setPrimaryType(NameConstants.NT_BASE)
 *         .addMixin(NameConstants.MIX_LOCKABLE)
 *         .createPropertyInfo("prop3")
 *             .setType(PropertyType.BINARY)
 *         .build()
 *     .build()
 * .build();
 * </pre>
 */
public final class ItemInfoBuilder {

    private ItemInfoBuilder() {
        super();
    }

    /**
     * Same as <code>nodeInfoBuilder("", listener)</code>
     * @param listener
     * @return
     */
    public static NodeInfoBuilder nodeInfoBuilder(Listener listener) {
		return null;
	}

    /**
     * Same as <code>nodeInfoBuilder("", null)</code>
     * @return
     */
    public static NodeInfoBuilder nodeInfoBuilder() {
		return null;
	}

    /**
     * Same as <code>nodeInfoBuilder(localName, null)</code>
     * @param localName
     * @return
     */
    public static NodeInfoBuilder nodeInfoBuilder(String localName) {
		return null;
	}

    /**
     * Return a {@link NodeInfoBuilder} for a node with a given <code>localName</code>.
     * @param localName  localName of the node
     * @param listener  {@link Listener} to receive notifications about {@link NodeInfo}s,
     *                  {@link PropertyInfo}s and {@link ChildInfo}s built.
     * @return
     */
    public static NodeInfoBuilder nodeInfoBuilder(String localName, Listener listener) {
		return null;
	}

    /**
     * Return a {@link NodeInfoBuilder} for a node with a given <code>name</code>.
     * @param name  name of the node
     * @param listener  {@link Listener} to receive notifications about {@link NodeInfo}s,
     *                  {@link PropertyInfo}s and {@link ChildInfo}s built.
     * @return
     */
    public static NodeInfoBuilder nodeInfoBuilder(Name name, Listener listener) {
		return null;
	}

    /**
     * A listener for receiving notifications about items built by the builders in this class.
     */
    public interface Listener {

        /**
         * Notification that a new {@link NodeInfo} has been built.
         * @param nodeInfo
         */
        void createNodeInfo(NodeInfo nodeInfo) {
		}

        /**
         * Notification that new {@link ChildInfo}s have been built.
         * @param id  Id of the parent to which the <code>childInfos</code> belong
         * @param childInfos
         */
        void createChildInfos(NodeId id, Iterator<ChildInfo> childInfos) {
		}

        /**
         * Notification that a new {@link PropertyInfo} has been built.
         * @param propertyInfo
         */
        void createPropertyInfo(PropertyInfo propertyInfo) {
		}
    }

    /**
     * Builder for {@link NodeInfo}s. Use one of the {@link ItemInfoBuilder#nodeInfoBuilder()}
     * methods to create instances of this class.
     */
    public static class NodeInfoBuilder {
        private final NodeInfoBuilder parent;
        private final Listener listener;

        private Path parentPath;
        private String localName;
        private String namespace;
        private Name name;
        private int index = Path.INDEX_DEFAULT;
        private String uuid;
        private Name primaryTypeName = NameConstants.NT_UNSTRUCTURED;
        private final List<Name> mixins = new ArrayList<Name>();
        private boolean includeChildInfos = true;

        private boolean stale;
        private final List<ItemInfo> itemInfos = new ArrayList<ItemInfo>();
        private NodeInfo nodeInfo;

        private NodeInfoBuilder(NodeInfoBuilder nodeInfoBuilder, String localName, Listener listener) {
            super();
            parent = nodeInfoBuilder;
            this.localName = localName;
            this.listener = listener;
        }

        private NodeInfoBuilder(NodeInfoBuilder nodeInfoBuilder, Name name, Listener listener) {
            super();
            parent = nodeInfoBuilder;
            this.name = name;
            this.listener = listener;
        }

        /**
         * Create a new child {@link PropertyInfo} with a given <code>localName</code> and a given
         * <code>value</code> of type <code>String</code> on this {@link NodeInfo}.
         *
         * @param localName
         * @param value
         * @return  <code>this</code>
         * @throws RepositoryException
         */
        public PropertyInfoBuilder createPropertyInfo(String localName, String value) throws RepositoryException {
			return null;
		}

        /**
         * Create a new child {@link PropertyInfo} with a given
         * <code>localName</code> on this {@link NodeInfo}.
         *
         * @param localName
         * @return  <code>this</code>
         */
        public PropertyInfoBuilder createPropertyInfo(String localName) {
			return null;
		}

        /**
         * Create a new child {@link PropertyInfo} on this {@link NodeInfo}.
         *
         * @return  <code>this</code>
         */
        public PropertyInfoBuilder createPropertyInfo() {
			return null;
		}

        /**
         * Create a new child {@link NodeInfo} on this NodeInfo with a given <code>localName</code>.
         * @param localName
         * @return  <code>this</code>
         */
        public NodeInfoBuilder createNodeInfo(String localName) {
			return null;
		}

        /**
         * Create a new child {@link NodeInfo} on this NodeInfo.

         * @return  <code>this</code>
         */
        public NodeInfoBuilder createNodeInfo() {
			return null;
		}

        /**
         * Set the <code>name</code> of the node
         *
         * @param name
         * @return
         */
        public NodeInfoBuilder setName(Name name) {
			return null;
		}

        /**
         * Set the <code>localName</code> of the node
         *
         * @param localName
         * @return
         */
        public NodeInfoBuilder setName(String localName) {
			return null;
		}
        /**
         * Set the namespace
         *
         * @param namespace
         * @return
         */
        public NodeInfoBuilder setNamespace(String namespace) {
			return null;
		}

        /**
         * Set the index.
         * @see NodeInfo#getIndex()
         *
         * @param index
         * @return
         */
        public NodeInfoBuilder setIndex(int index) {
			return null;
		}

        /**
         * Set the uuid
         *
         * @param uuid
         * @return
         */
        public NodeInfoBuilder setUUID(String uuid) {
			return null;
		}

        /**
         * Set the parent's path of the node
         * 
         * @param parentPath
         * @return
         */
        public NodeInfoBuilder setParentPath(Path parentPath) {
			return null;
		}

        /**
         * Set the name of the primary type.
         * @param name
         * @see NodeInfo#getNodetype()
         *
         * @return
         */
        public NodeInfoBuilder setPrimaryType(Name name) {
			return null;
		}

        /**
         * Add a mixin type
         * @see NodeInfo#getMixins()
         *
         * @param name
         * @return
         */
        public NodeInfoBuilder addMixin(Name name) {
			return null;
		}

        /**
         * Whether the {@link ChildInfo}s should be included or not.
         * @see NodeInfo#getChildInfos()
         *
         * @param include
         * @return
         */
        public NodeInfoBuilder includeChildInfos(boolean include) {
			return null;
		}

        /**
         * Build the {@link NodeInfo}. If a {@link Listener} is associated with this
         * instance, then its {@link Listener#createChildInfos(NodeId, Iterator)} and
         * its {@link Listener#createNodeInfo(NodeInfo)} methods are called.
         *
         * @return the parent builder of this builder
         * @throws RepositoryException
         * @throws IllegalStateException  if build has been called before
         */
        public NodeInfoBuilder build() throws RepositoryException {
			return null;
		}

        /**
         * @return  the parent builder of this builder
         */
        public NodeInfoBuilder getParent() {
			return null;
		}

        /**
         * Returns the {@link NodeInfo} which has been built by this builder.
         *
         * @return
         * @throws IllegalStateException  if {@link #build()} has not been called before.
         */
        public NodeInfo getNodeInfo() {
			return null;
		}

        /**
         * Add a {@link PropertyInfo}
         *
         * @param propertyInfo
         * @return <code>this</code>
         */
        public NodeInfoBuilder addPropertyInfo(PropertyInfo propertyInfo) {
			return null;
		}

        /**
         * Add a {@link NodeInfo}
         *
         * @param nodeInfo
         * @return <code>this</code>
         */
        public NodeInfoBuilder addNodeInfo(NodeInfo nodeInfo) {
			return null;
		}

        private NodeId getId() throws RepositoryException {
			return null;
		}

        private Path getPath() throws RepositoryException {
			return null;
		}

        private Iterator<ChildInfo> getChildInfos() {
			return null;
		}

        private Iterator<PropertyId> getPropertyIds() {
			return null;
		}

    }

    /**
     * Builder for {@link PropertyInfo}s. Use {@link NodeInfoBuilder#createPropertyInfo(String)}
     * to create an instance of this class.
     */
    public static class PropertyInfoBuilder {
        private final NodeInfoBuilder parent;
        private final Listener listener;

        private Name name;
        private String localName;
        private String namespace;
        private final List<QValue> values = new ArrayList<QValue>();
        private int type = PropertyType.UNDEFINED;
        private boolean isMultivalued = true;

        private boolean stale;
        private PropertyInfo propertyInfo;

        private PropertyInfoBuilder(NodeInfoBuilder nodeInfoBuilder, String localName, Listener listener) {
            super();
            parent = nodeInfoBuilder;
            this.localName = localName;
            this.listener = listener;
        }

        /**
         * Set the <code>name</code> of this property
         *
         * @param name
         * @return
         */
        public PropertyInfoBuilder setName(Name name) {
			return null;
		}

        /**
         * Set the <code>localName</code> of this property
         *
         * @param localName
         * @return
         */
        public PropertyInfoBuilder setName(String localName) {
			return null;
		}

        /**
         * Set the namespace
         *
         * @param namespace
         * @return
         */
        public PropertyInfoBuilder setNamespace(String namespace) {
			return null;
		}

        /**
         * Set the <code>{@link PropertyType type}</code> of this property
         *
         * @param type
         * @return <code>this</code>
         * @throws IllegalStateException  if a property of a different type has been added before.
         */
        public PropertyInfoBuilder setType(int type) {
			return null;
		}

        /**
         * Add a <code>value</code> to this property. Sets this property to single valued if
         * this is the first value. Otherwise sets this property to multi-valued.
         *
         * @param value
         * @return <code>this</code>
         * @throws IllegalStateException  if the type of the value does not match the type of this property
         */
        public PropertyInfoBuilder addValue(QValue value) {
			return null;
		}

        /**
         * Add a {@link PropertyType#STRING} value to this property.
         *
         * @param value
         * @return <code>this</code>
         * @throws RepositoryException
         * @throws IllegalStateException  if the type of the value does not match the type of this property
         */
        public PropertyInfoBuilder addValue(String value) throws RepositoryException {
			return null;
		}

        /**
         * Add a {@link PropertyType#DATE} value to this property.
         *
         * @param value
         * @return <code>this</code>
         * @throws RepositoryException
         * @throws IllegalStateException  if the type of the value does not match the type of this property
         */
        public PropertyInfoBuilder addValue(Calendar  value) throws RepositoryException {
			return null;
		}

        /**
         * Add a {@link PropertyType#DOUBLE} value to this property.
         *
         * @param value
         * @return <code>this</code>
         * @throws RepositoryException
         * @throws IllegalStateException  if the type of the value does not match the type of this property
         */
        public PropertyInfoBuilder addValue(double value) throws RepositoryException {
			return null;
		}

        /**
         * Add a {@link PropertyType#LONG} value to this property.
         *
         * @param value
         * @return <code>this</code>
         * @throws RepositoryException
         * @throws IllegalStateException  if the type of the value does not match the type of this property
         */
        public PropertyInfoBuilder addValue(long value) throws RepositoryException {
			return null;
		}

        /**
         * Add a {@link PropertyType#BOOLEAN} value to this property.
         *
         * @param value
         * @return <code>this</code>
         * @throws RepositoryException
         * @throws IllegalStateException  if the type of the value does not match the type of this property
         */
        public PropertyInfoBuilder addValue(boolean value) throws RepositoryException {
			return null;
		}

        /**
         * Add a {@link PropertyType#NAME} value to this property.
         *
         * @param value
         * @return <code>this</code>
         * @throws RepositoryException
         * @throws IllegalStateException  if the type of the value does not match the type of this property
         */
        public PropertyInfoBuilder addValue(Name value) throws RepositoryException {
			return null;
		}

        /**
         * Add a {@link PropertyType#PATH} value to this property.
         *
         * @param value
         * @return <code>this</code>
         * @throws RepositoryException
         * @throws IllegalStateException  if the type of the value does not match the type of this property
         */
        public PropertyInfoBuilder addValue(Path value) throws RepositoryException {
			return null;
		}

        /**
         * Add a {@link PropertyType#DECIMAL} value to this property.
         *
         * @param value
         * @return <code>this</code>
         * @throws RepositoryException
         * @throws IllegalStateException  if the type of the value does not match the type of this property
         */
        public PropertyInfoBuilder addValue(BigDecimal  value) throws RepositoryException {
			return null;
		}

        /**
         * Add a {@link PropertyType#URI} value to this property.
         *
         * @param value
         * @return <code>this</code>
         * @throws RepositoryException
         * @throws IllegalStateException  if the type of the value does not match the type of this property
         */
        public PropertyInfoBuilder addValue(URI value) throws RepositoryException {
			return null;
		}

        /**
         * Add a {@link PropertyType#BINARY} value to this property.
         *
         * @param value
         * @return <code>this</code>
         * @throws RepositoryException
         * @throws IllegalStateException  if the type of the value does not match the type of this property
         */
        public PropertyInfoBuilder addValue(byte[] value) throws RepositoryException {
			return null;
		}

        /**
         * Add a {@link PropertyType#BINARY} value to this property.
         *
         * @param value
         * @return <code>this</code>
         * @throws RepositoryException
         * @throws IllegalStateException  if the type of the value does not match the type of this property
         */
        public PropertyInfoBuilder addValue(InputStream value) throws RepositoryException, IOException {
			return null;
		}

        /**
         * Add a {@link PropertyType#BINARY} value to this property.
         *
         * @param value
         * @return <code>this</code>
         * @throws RepositoryException
         * @throws IllegalStateException  if the type of the value does not match the type of this property
         */
        public PropertyInfoBuilder addValue(File value) throws RepositoryException, IOException {
			return null;
		}

        /**
         * Set this property to multi-values.
         *
         * @param on
         * @return <code>this</code>
         * @throws IllegalStateException if this property does not contain exactly on value
         */
        public PropertyInfoBuilder setMultivalued(boolean on) {
			return null;
		}

        /**
         * Build the {@link PropertyInfo}. If a {@link Listener} is associated with this
         * instance, then its {@link Listener#createPropertyInfo(PropertyInfo)} methods
         * is called.
         *
         * @return the parent builder of this builder
         * @throws RepositoryException
         * @throws IllegalStateException  if build has been called before
         * @throws IllegalStateException  if the type is not set
         */
        public NodeInfoBuilder build() throws RepositoryException {
			return null;
		}

        /**
         * @return  the parent builder of this builder
         */
        public NodeInfoBuilder getParent() {
			return null;
		}

        /**
         * Returns the {@link PropertyInfo} which has been built by this builder.
         *
         * @return
         * @throws IllegalStateException  if {@link #build()} has not been called before.
         */
        public PropertyInfo getPropertyInfo() {
			return null;
		}

    }

}
