/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.spi.commons.nodetype;

import org.apache.jackrabbit.spi.Name;
import org.apache.jackrabbit.spi.QItemDefinition;
import org.apache.jackrabbit.spi.QNodeDefinition;
import org.apache.jackrabbit.spi.QNodeTypeDefinition;
import org.apache.jackrabbit.spi.QPropertyDefinition;
import org.apache.jackrabbit.spi.QValueConstraint;
import org.apache.jackrabbit.spi.commons.name.NameConstants;

import javax.jcr.PropertyType;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * A <code>NodeTypeDefDiff</code> represents the result of the comparison of
 * two node type definitions.
 * <p>
 * The result of the comparison can be categorized as one of the following types:
 * <p>
 * <b><code>NONE</code></b> indicates that there is no modification at all.
 * <p>
 * A <b><code>TRIVIAL</code></b> modification has no impact on the consistency
 * of existing content. The following modifications are considered
 * <code>TRIVIAL</code>:
 * <ul>
 * <li>changing node type <code>orderableChildNodes</code> flag
 * <li>changing node type <code>primaryItemName</code> value
 * <li>adding non-<code>mandatory</code> property/child node
 * <li>changing property/child node <code>protected</code> flag
 * <li>changing property/child node <code>onParentVersion</code> value
 * <li>changing property/child node <code>mandatory</code> flag to <code>false</code>
 * <li>changing property/child node <code>autoCreated</code> flag
 * <li>changing specific property/child node <code>name</code> to <code>*</code>
 * <li>changing child node <code>defaultPrimaryType</code>
 * <li>changing child node <code>sameNameSiblings</code> flag to <code>true</code>
 * <li>weaken child node <code>requiredPrimaryTypes</code> (e.g. by removing)
 * <li>weaken property <code>valueConstraints</code> (e.g. by removing a constraint
 * or by making a specific constraint less restrictive)
 * <li>changing property <code>defaultValues</code>
 * <li>changing specific property <code>requiredType</code> to <code>undefined</code>
 * <li>changing property <code>multiple</code> flag to <code>true</code>
 * </ul>
 * <p>
 * A <b><code>MAJOR</code></b> modification potentially <i>affects</i> the
 * consistency of existing content.
 *
 * All modifications that are not <b><code>TRIVIAL</code></b> are considered
 * <b><code>MAJOR</code></b>.
 *
 * @see #getType()
 */
public class NodeTypeDefDiff {

    /**
     * no modification
     */
    public static final int NONE = 0;
    /**
     * trivial modification: does not affect consistency of existing content
     */
    public static final int TRIVIAL = 1;
    /**
     * major modification: <i>does</i> affect consistency of existing content
     */
    public static final int MAJOR = 2;

    private final QNodeTypeDefinition oldDef;
    private final QNodeTypeDefinition newDef;
    private int type;

    private final List<PropDefDiff> propDefDiffs = new ArrayList<PropDefDiff>();
    private final List<ChildNodeDefDiff> childNodeDefDiffs = new ArrayList<ChildNodeDefDiff>();

    /**
     * Constructor
     * @param oldDef old definition
     * @param newDef new definition
     */
    private NodeTypeDefDiff(QNodeTypeDefinition oldDef, QNodeTypeDefinition newDef) {
        this.oldDef = oldDef;
        this.newDef = newDef;
        init();
    }

    /**
     *
     */
    private void init() {
	}

    /**
     * @param oldDef old definition
     * @param newDef new definition
     * @return the diff
     */
    public static NodeTypeDefDiff create(QNodeTypeDefinition oldDef, QNodeTypeDefinition newDef) {
		return null;
	}

    /**
     * @return <code>true</code> if modified
     */
    public boolean isModified() {
		return false;
	}

    /**
     * @return <code>true</code> if trivial
     */
    public boolean isTrivial() {
		return false;
	}

    /**
     * @return <code>true</code> if major
     */
    public boolean isMajor() {
		return false;
	}

    /**
     * Returns the type of modification as expressed by the following constants:
     * <ul>
     * <li><b><code>NONE</code></b>: no modification at all
     * <li><b><code>TRIVIAL</code></b>: does not affect consistency of
     * existing content
     * <li><b><code>MAJOR</code></b>: <i>does</i> affect consistency of existing
     * content
     * </ul>
     *
     * @return the type of modification
     */
    public int getType() {
		return 0;
	}

    /**
     * @return <code>true</code> if mixin flag diff
     */
    public int mixinFlagDiff() {
		return 0;
	}

    /**
     * @return <code>true</code> if abstract flag diff
     */
    public int abstractFlagDiff() {
		return 0;
	}

    /**
     * @return <code>true</code> if supertypes diff
     */
    public int supertypesDiff() {
		return 0;
	}

    /**
     * @return diff type
     */
    private int buildPropDefDiffs() {
		return 0;
	}

    /**
     * @return diff type
     */
    private int buildChildNodeDefDiffs() {
		return 0;
	}

    private Map<QNodeDefinitionId, List<QNodeDefinition>> collectChildNodeDefs(final QNodeDefinition[] cnda1) {
		return null;
	}

    @Override
    public String toString() {
		return null;
	}

    private String toString(List<? extends ChildItemDefDiff> childItemDefDiffs) {
		return null;
	}

    private String modificationTypeToString(int modificationType) {
		return null;
	}


    //--------------------------------------------------------< inner classes >

    abstract class ChildItemDefDiff {
        protected final QItemDefinition oldDef;
        protected final QItemDefinition newDef;
        protected int type;

        ChildItemDefDiff(QItemDefinition oldDef, QItemDefinition newDef) {
            this.oldDef = oldDef;
            this.newDef = newDef;
            init();
        }

        protected void init() {
		}

        public int getType() {
			return 0;
		}

        public boolean isAdded() {
			return false;
		}

        public boolean isRemoved() {
			return false;
		}

        public boolean isModified() {
			return false;
		}

        @Override
        public String toString() {
			return null;
		}

    }

    public class PropDefDiff extends ChildItemDefDiff {

        PropDefDiff(QPropertyDefinition oldDef, QPropertyDefinition newDef) {
            super(oldDef, newDef);
        }

        public QPropertyDefinition getOldDef() {
			return null;
		}

        public QPropertyDefinition getNewDef() {
			return null;
		}

        @Override
        protected void init() {
		}
    }

    public class ChildNodeDefDiff extends ChildItemDefDiff {

        ChildNodeDefDiff(QNodeDefinition oldDef, QNodeDefinition newDef) {
            super(oldDef, newDef);
        }

        public QNodeDefinition getOldDef() {
			return null;
		}

        public QNodeDefinition getNewDef() {
			return null;
		}

        @Override
        protected void init() {
		}
    }

    /**
     * Identifier used to identify corresponding property definitions
     */
    static class QPropertyDefinitionId {

        Name declaringNodeType;
        Name name;
        boolean definesResidual;

        QPropertyDefinitionId(QPropertyDefinition def) {
            declaringNodeType = def.getDeclaringNodeType();
            name = def.getName();
            definesResidual = def.definesResidual();
        }

        //---------------------------------------< java.lang.Object overrides >
        @Override
        public boolean equals(Object obj) {
			return false;
		}

        @Override
        public int hashCode() {
			return 0;
		}
    }

    /**
     * Identifier used to identify corresponding node definitions
     */
    static class QNodeDefinitionId {

        Name declaringNodeType;
        Name name;

        QNodeDefinitionId(QNodeDefinition def) {
            declaringNodeType = def.getDeclaringNodeType();
            name = def.getName();
        }

        //---------------------------------------< java.lang.Object overrides >
        @Override
        public boolean equals(Object obj) {
			return false;
		}

        @Override
        public int hashCode() {
			return 0;
		}
    }

    private class ChildNodeDefDiffs {

        private final List<QNodeDefinition> defs1;
        private final List<QNodeDefinition> defs2;

        private ChildNodeDefDiffs(final List<QNodeDefinition> defs1, final List<QNodeDefinition> defs2) {
            this.defs1 = defs1 != null ? defs1 : Collections.<QNodeDefinition>emptyList();
            this.defs2 = defs2 != null ? defs2 : Collections.<QNodeDefinition>emptyList();
        }

        private Collection<ChildNodeDefDiff> getChildNodeDefDiffs() {
			return null;
		}

        private boolean alreadyMatched(final List<ChildNodeDefDiff> result, final QNodeDefinition newDef, final QNodeDefinition oldDef, final AtomicInteger allowedNewNull, final AtomicInteger allowedOldNull) {
			return false;
		}
    }

}
