/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.spi.commons.query.sql;

import org.apache.jackrabbit.spi.commons.query.AndQueryNode;
import org.apache.jackrabbit.spi.commons.query.LocationStepQueryNode;
import org.apache.jackrabbit.spi.commons.query.NAryQueryNode;
import org.apache.jackrabbit.spi.commons.query.NodeTypeQueryNode;
import org.apache.jackrabbit.spi.commons.query.NotQueryNode;
import org.apache.jackrabbit.spi.commons.query.OrQueryNode;
import org.apache.jackrabbit.spi.commons.query.OrderQueryNode;
import org.apache.jackrabbit.spi.commons.query.PathQueryNode;
import org.apache.jackrabbit.spi.commons.query.QueryConstants;
import org.apache.jackrabbit.spi.commons.query.QueryNode;
import org.apache.jackrabbit.spi.commons.query.QueryRootNode;
import org.apache.jackrabbit.spi.commons.query.RelationQueryNode;
import org.apache.jackrabbit.spi.commons.query.TextsearchQueryNode;
import org.apache.jackrabbit.spi.commons.query.PropertyFunctionQueryNode;
import org.apache.jackrabbit.spi.commons.query.QueryNodeFactory;
import org.apache.jackrabbit.spi.commons.conversion.NameException;
import org.apache.jackrabbit.spi.commons.name.NameConstants;
import org.apache.jackrabbit.spi.commons.name.NameFactoryImpl;
import org.apache.jackrabbit.spi.commons.name.PathBuilder;
import org.apache.jackrabbit.spi.Name;
import org.apache.jackrabbit.spi.Path;
import org.apache.jackrabbit.spi.commons.conversion.MalformedPathException;
import org.apache.jackrabbit.spi.commons.conversion.NameResolver;
import org.apache.jackrabbit.util.ISO8601;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.apache.commons.collections.map.ReferenceMap;

import javax.jcr.query.InvalidQueryException;
import javax.jcr.NamespaceException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Map;
import java.io.StringReader;

/**
 * Implements the query builder for the JCR SQL syntax.
 */
public class JCRSQLQueryBuilder implements JCRSQLParserVisitor {

    /**
     * logger instance for this class
     */
    private static final Logger log = LoggerFactory.getLogger(JCRSQLQueryBuilder.class);

    /**
     * DateFormat pattern for type
     * {@link org.apache.jackrabbit.spi.commons.query.QueryConstants#TYPE_DATE}.
     */
    private static final String DATE_PATTERN = "yyyy-MM-dd";

    /**
     * Map of reusable JCRSQL parser instances indexed by NamespaceResolver.
     */
    private static Map parsers = new ReferenceMap(ReferenceMap.WEAK, ReferenceMap.WEAK);

    /**
     * The root node of the sql query syntax tree
     */
    private final ASTQuery stmt;

    /**
     * The root query node
     */
    private QueryRootNode root;

    /**
     * To resolve QNames
     */
    private NameResolver resolver;

    /**
     * Query node to gather the constraints defined in the WHERE clause
     */
    private final AndQueryNode constraintNode;

    /**
     * The Name of the node type in the from clause.
     */
    private Name nodeTypeName;

    /**
     * List of PathQueryNode constraints that need to be merged
     */
    private final List pathConstraints = new ArrayList();

    /**
     * The query node factory.
     */
    private final QueryNodeFactory factory;

    /**
     * Creates a new <code>JCRSQLQueryBuilder</code>.
     *
     * @param statement the root node of the SQL syntax tree.
     * @param resolver  a namespace resolver to use for names in the
     *                  <code>statement</code>.
     * @param factory   the query node factory.
     */
    private JCRSQLQueryBuilder(ASTQuery statement,
                               NameResolver resolver,
                               QueryNodeFactory factory) {
        this.stmt = statement;
        this.resolver = resolver;
        this.factory = factory;
        this.constraintNode =  factory.createAndQueryNode(null);
    }

    /**
     * Creates a <code>QueryNode</code> tree from a SQL <code>statement</code>
     * using the passed query node <code>factory</code>.
     *
     * @param statement the SQL statement.
     * @param resolver  the namespace resolver to use.
     * @return the <code>QueryNode</code> tree.
     * @throws InvalidQueryException if <code>statement</code> is malformed.
     */
    public static QueryRootNode createQuery(String statement,
                                            NameResolver resolver,
                                            QueryNodeFactory factory)
            throws InvalidQueryException {
				return null;
			}

    /**
     * Creates a String representation of the query node tree in SQL syntax.
     *
     * @param root     the root of the query node tree.
     * @param resolver to resolve QNames.
     * @return a String representation of the query node tree.
     * @throws InvalidQueryException if the query node tree cannot be converted
     *                               into a String representation due to restrictions in SQL.
     */
    public static String toString(QueryRootNode root, NameResolver resolver)
            throws InvalidQueryException {
				return null;
			}

    /**
     * Parses the statement and returns the root node of the <code>QueryNode</code>
     * tree.
     *
     * @return the root node of the <code>QueryNode</code> tree.
     */
    private QueryRootNode getRootNode() {
		return null;
	}

    //----------------< JCRSQLParserVisitor >------------------------------------

    public Object visit(SimpleNode node, Object data) {
		return null;
	}

    public Object visit(ASTQuery node, Object data) {
		return null;
	}

    public Object visit(ASTSelectList node, Object data) {
		return null;
	}

    public Object visit(ASTFromClause node, Object data) {
		return null;
	}

    public Object visit(ASTWhereClause node, Object data) {
		return null;
	}

    public Object visit(ASTPredicate node, Object data) {
		return null;
	}

    public Object visit(ASTOrExpression node, Object data) {
		return null;
	}

    public Object visit(ASTAndExpression node, Object data) {
		return null;
	}

    public Object visit(ASTNotExpression node, Object data) {
		return null;
	}

    public Object visit(ASTBracketExpression node, Object data) {
		return null;
	}

    public Object visit(ASTLiteral node, Object data) {
		return null;
	}

    public Object visit(ASTIdentifier node, Object data) {
		return null;
	}

    public Object visit(ASTOrderByClause node, Object data) {
		return null;
	}

    public Object visit(ASTOrderSpec node, Object data) {
		return null;
	}

    public Object visit(ASTAscendingOrderSpec node, Object data) {
		return null;
	}

    public Object visit(ASTDescendingOrderSpec node, Object data) {
		return null;
	}

    public Object visit(ASTContainsExpression node, Object data) {
		return null;
	}

    public Object visit(ASTLowerFunction node, Object data) {
		return null;
	}

    public Object visit(ASTUpperFunction node, Object data) {
		return null;
	}

    public Object visit(ASTExcerptFunction node, Object data) {
		return null;
	}

    //------------------------< internal >--------------------------------------

    /**
     * Creates a new {@link org.apache.jackrabbit.spi.commons.query.RelationQueryNode}.
     *
     * @param parent        the parent node for the created <code>RelationQueryNode</code>.
     * @param propertyName  the property name for the relation.
     * @param operationType the operation type.
     * @param literal       the literal value for the relation or
     *                      <code>null</code> if the relation does not have a
     *                      literal (e.g. IS NULL).
     * @return a <code>RelationQueryNode</code>.
     * @throws IllegalArgumentException if the literal value does not conform
     *                                  to its type. E.g. a malformed String representation of a date.
     */
    private RelationQueryNode createRelationQueryNode(QueryNode parent,
                                                      Name propertyName,
                                                      int operationType,
                                                      ASTLiteral literal)
            throws IllegalArgumentException {
				return null;
			}

    /**
     * Creates <code>LocationStepQueryNode</code>s from a <code>path</code>.
     *
     * @param path      the path pattern
     * @param operation the type of the parent node
     */
    private void createPathQuery(String path, int operation) {
	}

    /**
     * Translates a pattern using the escape character <code>from</code> into
     * a pattern using the escape character <code>to</code>.
     *
     * @param pattern the pattern to translate
     * @param from    the currently used escape character.
     * @param to      the new escape character to use.
     * @return the new pattern using the escape character <code>to</code>.
     */
    private static String translateEscaping(String pattern, char from, char to) {
		return null;
	}

    /**
     * Extends the <code>PathQueryNode</code> with merging capability. A
     * <code>PathQueryNode</code> <code>n1</code> can be merged with another
     * node <code>n2</code> in the following case:
     * <p>
     * <code>n1</code> contains a location step at position <code>X</code> with
     * a name test that matches any node and has the descending flag set. Where
     * <code>X</code> &lt; number of location steps.
     * <code>n2</code> contains no location step to match any node name and
     * the sequence of name tests is the same as the sequence of name tests
     * of <code>n1</code>.
     * The merged node then contains a location step at position <code>X</code>
     * with the name test of the location step at position <code>X+1</code> and
     * the descending flag set.
     * <p>
     * The following path patterns:<br/>
     * <code>/foo/%/bar</code> OR <code>/foo/bar</code><br/>
     * are merged into:<br/>
     * <code>/foo//bar</code>.
     * <p>
     * The path patterns:<br/>
     * <code>/foo/%</code> AND NOT <code>/foo/%/%</code><br/>
     * are merged into:<br/>
     * <code>/foo/*</code>
     */
    private static class MergingPathQueryNode extends PathQueryNode {

        /**
         * The operation type of the parent node
         */
        private int operation;

        /**
         * Creates a new <code>MergingPathQueryNode</code> with the operation
         * type of a parent node. <code>operation</code> must be one of:
         * {@link org.apache.jackrabbit.spi.commons.query.QueryNode#TYPE_OR},
         * {@link org.apache.jackrabbit.spi.commons.query.QueryNode#TYPE_AND} or
         * {@link org.apache.jackrabbit.spi.commons.query.QueryNode#TYPE_NOT}.
         *
         * @param operation the operation type of the parent node.
         * @param validJcrSystemNodeTypeNames names of valid node types under
         *        /jcr:system.
         */
        MergingPathQueryNode(
                int operation, Collection<Name> validJcrSystemNodeTypeNames) {
            super(null, validJcrSystemNodeTypeNames);
            if (operation != QueryNode.TYPE_OR && operation != QueryNode.TYPE_AND && operation != QueryNode.TYPE_NOT) {
                throw new IllegalArgumentException("operation");
            }
            this.operation = operation;
        }

        /**
         * Merges this node with a node from <code>nodes</code>. If a merge
         * is not possible an NoSuchElementException is thrown.
         *
         * @param nodes the nodes to try to merge with.
         * @return the merged array containing a merged version of this node.
         */
        MergingPathQueryNode[] doMerge(MergingPathQueryNode[] nodes) {
			return null;
		}

        /**
         * Merges two nodes into a node which selects any child nodes of a
         * given node.
         * <p>
         * Example:<br/>
         * The path patterns:<br/>
         * <code>/foo/%</code> AND NOT <code>/foo/%/%</code><br/>
         * are merged into:<br/>
         * <code>/foo/*</code>
         *
         * @param nodes the nodes to merge with.
         * @return the merged nodes.
         */
        private MergingPathQueryNode[] doAndMerge(MergingPathQueryNode[] nodes) {
			return null;
		}

        /**
         * Merges two nodes into one node selecting a node on the
         * descendant-or-self axis.
         * <p>
         * Example:<br/>
         * The following path patterns:<br/>
         * <code>/foo/%/bar</code> OR <code>/foo/bar</code><br/>
         * are merged into:<br/>
         * <code>/foo//bar</code>.
         *
         * @param nodes the node to merge.
         * @return the merged nodes.
         */
        private MergingPathQueryNode[] doOrMerge(MergingPathQueryNode[] nodes) {
			return null;
		}

        /**
         * Returns <code>true</code> if this node needs merging; <code>false</code>
         * otherwise.
         *
         * @return <code>true</code> if this node needs merging; <code>false</code>
         *         otherwise.
         */
        boolean needsMerge() {
			return false;
		}
    }
}
