/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.sling.commons.threads;

import java.util.concurrent.ThreadFactory;

/**
 * This is a modifiable thread pool configuration that can be instantiated
 * and then configured to create a new thread pool.
 *
 * The default values for this configuration are:
 * - min pool size: 5
 * - max pool size: 5
 * - queue size: -1
 * - keep alive time: 60000
 * - block policy: RUN
 * - shutdown graceful: false
 * - shutdown wait time: -1
 * - priority: NORM
 * - daemon: false
 * - factory: null (= default jvm thread factory)
 */
public final class ModifiableThreadPoolConfig implements ThreadPoolConfig {

    /** Configuration property for the min pool size. */
    public static final String PROPERTY_MIN_POOL_SIZE = "minPoolSize";
    /** Configuration property for the max pool size. */
    public static final String PROPERTY_MAX_POOL_SIZE = "maxPoolSize";
    /** Configuration property for the queue size. */
    public static final String PROPERTY_QUEUE_SIZE = "queueSize";
    /** Configuration property for the keep alive time. */
    public static final String PROPERTY_KEEP_ALIVE_TIME = "keepAliveTime";
    /** Configuration property for the block policy. */
    public static final String PROPERTY_BLOCK_POLICY = "blockPolicy";
    /** Configuration property for the shutdown graceful flag. */
    public static final String PROPERTY_SHUTDOWN_GRACEFUL = "shutdownGraceful";
    /** Configuration property for the shutdown wait time. */
    public static final String PROPERTY_SHUTDOWN_WAIT_TIME = "shutdownWaitTime";
    /** Configuration property for the priority. */
    public static final String PROPERTY_PRIORITY = "priority";
    /** Configuration property for the daemon flag. */
    public static final String PROPERTY_DAEMON = "daemon";
    /** Configuration property for the thread pool name. */
    public static final String PROPERTY_NAME = "name";

    /** The min pool size. */
    private int minPoolSize = 5;

    /** The max pool size. */
    private int maxPoolSize = 5;

    /** The queue size */
    private int queueSize = -1;

    /** The keep alive time. */
    private long  keepAliveTime = 60000L;

    /** The thread pool policy. Default is RUN. */
    private ThreadPoolPolicy blockPolicy = ThreadPoolPolicy.RUN;

    /** Try to shutdown gracefully? */
    private  boolean shutdownGraceful = false;

    /** Wait time during shutdown. */
    private  int shutdownWaitTimeMs = -1;

    /** Optional thread factory. */
    private  ThreadFactory factory;

    /** Thread priority. */
    private  ThreadPriority   priority = ThreadPriority.NORM;

    /** Create daemon threads? */
    private  boolean isDaemon = false;

    /**
     * Create a new default configuration.
     */
    public ModifiableThreadPoolConfig() {
        // nothing to do
    }

    /**
     * Clone an existing configuration
     * @param copy The config to clone
     */
    public ModifiableThreadPoolConfig(final ThreadPoolConfig copy) {
        if ( copy != null ) {
            this.minPoolSize = copy.getMinPoolSize();
            this.maxPoolSize = copy.getMaxPoolSize();
            this.queueSize = copy.getQueueSize();
            this.keepAliveTime = copy.getKeepAliveTime();
            this.blockPolicy = copy.getBlockPolicy();
            this.shutdownGraceful = copy.isShutdownGraceful();
            this.shutdownWaitTimeMs = copy.getShutdownWaitTimeMs();
            this.factory = copy.getFactory();
            this.priority = copy.getPriority();
            this.isDaemon = copy.isDaemon();
        }
    }

    /**
     * @see org.apache.sling.commons.threads.ThreadPoolConfig#getMinPoolSize()
     */
    public int getMinPoolSize() {
		return 0;
	}

    /**
     * Set the min pool size.
     * @param minPoolSize New min pool size.
     */
    public void setMinPoolSize(final int minPoolSize) {
	}

    /**
     * @see org.apache.sling.commons.threads.ThreadPoolConfig#getMaxPoolSize()
     */
    public int getMaxPoolSize() {
		return 0;
	}

    /**
     * Set the max pool size.
     * @param maxPoolSize New max pool size.
     */
    public void setMaxPoolSize(final int maxPoolSize) {
	}

    /**
     * @see org.apache.sling.commons.threads.ThreadPoolConfig#getQueueSize()
     */
    public int getQueueSize() {
		return 0;
	}

    /**
     * Set the queue size.
     * @param queueSize New queue size.
     */
    public void setQueueSize(final int queueSize) {
	}

    /**
     * @see org.apache.sling.commons.threads.ThreadPoolConfig#getKeepAliveTime()
     */
    public long getKeepAliveTime() {
		return 0L;
	}

    /**
     * Set the keep alive time.
     * @param keepAliveTime New keep alive time.
     */
    public void setKeepAliveTime(final long keepAliveTime) {
	}

    /**
     * @see org.apache.sling.commons.threads.ThreadPoolConfig#getBlockPolicy()
     */
    public ThreadPoolPolicy getBlockPolicy() {
		return null;
	}

    /**
     * Set the block policy.
     * @param blockPolicy The new block policy.
     * @throws IllegalArgumentException If blockPolicy is null.
     */
    public void setBlockPolicy(final ThreadPoolPolicy blockPolicy) {
	}

    /**
     * @see org.apache.sling.commons.threads.ThreadPoolConfig#isShutdownGraceful()
     */
    public boolean isShutdownGraceful() {
		return false;
	}

    /**
     * Set if the pool should be shutdown graceful.
     * @param shutdownGraceful The shutdown graceful setting.
     */
    public void setShutdownGraceful(final boolean shutdownGraceful) {
	}

    /**
     * @see org.apache.sling.commons.threads.ThreadPoolConfig#getShutdownWaitTimeMs()
     */
    public int getShutdownWaitTimeMs() {
		return 0;
	}

    /**
     * Set the shutdown wait time.
     * @param shutdownWaitTimeMs The new shutdown wait time.
     */
    public void setShutdownWaitTimeMs(final int shutdownWaitTimeMs) {
	}

    /**
     * @see org.apache.sling.commons.threads.ThreadPoolConfig#getFactory()
     */
    public ThreadFactory getFactory() {
		return null;
	}

    /**
     * Set the thread factory.
     * @param factory The thread factory to be used or <code>null</code> to use
     *                the default thread factory.
     */
    public void setFactory(final ThreadFactory factory) {
	}

    /**
     * @see org.apache.sling.commons.threads.ThreadPoolConfig#getPriority()
     */
    public ThreadPriority getPriority() {
		return null;
	}

    /**
     * Set the thread priority.
     * @param priority The thread priority.
     * @throws IllegalArgumentException If priority is null.
     */
    public void setPriority(final ThreadPriority priority) {
	}

    /**
     * @see org.apache.sling.commons.threads.ThreadPoolConfig#isDaemon()
     */
    public boolean isDaemon() {
		return false;
	}

    /**
     * Set the daemon handling.
     * @param isDaemon The daemon setting.
     */
    public void setDaemon(final boolean isDaemon) {
	}

    @Override
    public boolean equals(Object obj) {
		return false;
	}

    @Override
    public int hashCode() {
		return 0;
	}
}